#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/base/prototype.rb'
# =========================================================================== #
require 'fileutils'
begin
  require 'opn'
rescue LoadError; end

begin
  require 'class_docu_shower'
rescue LoadError; end

require 'cookbooks/constants/archives.rb'
require 'cookbooks/constants/constants.rb'
require 'cookbooks/constants/directory_constants.rb'
require 'cookbooks/constants/show_names.rb'
require 'cookbooks/directory/individual_cookbooks_directory.rb'
require 'cookbooks/base/cliner.rb'
require 'cookbooks/base/colours.rb'
require 'cookbooks/base/debug.rb'
require 'cookbooks/base/time.rb'
require 'cookbooks/base/reset.rb' # This also pulls in file 'display_md5sum.rb'
# =========================================================================== #
# Next require some module_methods-related code.
# =========================================================================== #
require 'cookbooks/module_methods/all_binaries.rb'
require 'cookbooks/module_methods/all_libraries.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/module_methods/is_on_roebe.rb'
require 'cookbooks/module_methods/remove_file_suffix.rb'
require 'cookbooks/module_methods/return_archive_type.rb'
require 'cookbooks/module_methods/verbose_truth.rb'
# =========================================================================== #
# Next, cookbooks requires.
# =========================================================================== #
require 'cookbooks/requires/require_yaml.rb'
# =========================================================================== #
# Next, cookbooks/constants will be required.
# =========================================================================== #
require 'cookbooks/constants/configuration_constants.rb'
require 'cookbooks/constants/cookbook_utility_scripts.rb'
require 'cookbooks/constants/encoding.rb'
require 'cookbooks/constants/file_cookbook_aliases.rb'
require 'cookbooks/constants/misc.rb'
require 'cookbooks/constants/predefined_installation_instructions.rb'
require 'cookbooks/constants/specification_of_registered_cookbook_entries.rb'
require 'cookbooks/configuration/do_initialize_configuration.rb'

module Cookbooks

class Base # === Cookbooks::Base

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint(
      optional_message      = nil,
      use_this_as_exit_code = 0
    )
    Signal.trap('SIGINT') {
      if optional_message
        e optional_message
      end
      exit(use_this_as_exit_code)
    }
  end

  # ========================================================================== #
  # === create_directory
  #
  # Consistently use this when you want to create a directory
  # for the Cookbooks-Project.
  # ========================================================================== #
  def create_directory(i)
    unless File.directory? i
      opnn; e 'Creating directory `'+sdir(i)+'` now.'
      FileUtils.mkdir_p(i)
    end
  end; alias mkdir                             create_directory # === mkdir (mkdir tag)
       alias ensure_that_this_directory_exists create_directory # === ensure_that_this_directory_exists

  # ========================================================================= #
  # === remove_newlines
  #
  # Get rid of the newlines.
  # ========================================================================= #
  def remove_newlines(i)
    i.delete("\n")
  end

  # ========================================================================= #
  # === remove_double_slashes                                       (rds tag)
  #
  # Replace // with / in a given string.
  # ========================================================================= #
  def remove_double_slashes(i)
    i.squeeze '/'
  end; alias rds remove_double_slashes # === rds

  # ========================================================================== #
  # === read_file
  #
  # Use this method whenever you want to read in a file.
  # ========================================================================== #
  def read_file(i)
    File.read(i)
  end

  # ========================================================================= #
  # === copy_file                               (copy file, cp tag, copy tag)
  #
  # Use this method to copy a file.
  # ========================================================================= #
  def copy_file(file, where_to)
    FileUtils.cp(file, where_to)
  end; alias cp copy_file # === cp

  # ========================================================================= #
  # === move_file                                                   (mv file)
  # ========================================================================= #
  def move_file(old, new)
    e 'Moving from '+sfile(old)+' to '+sfile(new)+'.'
    FileUtils.mv(old, new)
  end

  # ========================================================================= #
  # === remove_file                                              (remove tag)
  #
  # Use this method whenever you wish to remove a file.
  # ========================================================================= #
  def remove_file(i)
    if File.exist? i
      File.delete(i) if File.file? i
    end
  end; alias delete_file remove_file # === delete_file
       alias delete      remove_file # === delete_file

  # ========================================================================= #
  # === touch                                                     (touch tag)
  #
  # Use this method to create a new file.
  # ========================================================================= #
  def touch(
      i,
      be_verbose = false
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    if be_verbose
      e 'Next creating the file `'+sfile(i)+'`.'
    end
    FileUtils.touch(i)
  end; alias create_file touch # === create_file

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    Cookbooks.e(i)
  end

  # ========================================================================= #
  # === remove_unnecessary_data_from_url
  #
  # This method removes some meaningless information that can be found
  # in some URLs.
  # ========================================================================= #
  def remove_unnecessary_data_from_url(i)
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    i = File.basename(i)
    i.gsub(/download\?file=/,'').
      gsub(/\/download$/,'').
      gsub(/\?download$/,'').
      gsub(/-fullsrc/,'').
      gsub(/-source/,'').
      gsub(/\?use_mirror=dfn/,'').
      gsub(/\.src/,'').
      gsub(/-src/,'')
      # .gsub(/_/,'-') # This here may be controversial, hence it  
                       # was disabled it as of Jun 2010.
      # i = remove_file_extension(i)
      # ^^^ We can not do the above, because program_full_name
      # must include the archive.
  end; alias return_program_full_name_from_url remove_unnecessary_data_from_url # === return_program_full_name

  # ========================================================================= #
  # === determine_archive_type
  #
  # We determine the archive type with this method, which really is
  # a simplification.
  # ========================================================================= #
  def determine_archive_type(i)
    _ = ''
    if    i.include? '.tar.xz'
      _ = '.tar.xz'
    elsif i.include? '.tar.gz'
      _ = '.tar.gz'
    elsif i.include? '.tar.bz2'
      _ = '.tar.bz2'
    else # else use the extension name.
      _ = File.extname(i)
    end
    return _
  end

  # ========================================================================= #
  # === cookbook_files?
  #
  # This method will return all available cookbook files in an Array.
  #
  # Usage example:
  #   cookbook_files?(:show_full_path)
  # ========================================================================= #
  def cookbook_files?(show_how = :show_only_name)
    result = Dir[individual_cookbooks_directory?+'*yml'].sort
    case show_how
    when :show_only_name # In this case, show only the name.
      result.map! {|entry|
        File.basename(entry).gsub( File.extname(entry), '' )
      }
    when :show_full_path # In this case make no modifications.
    end
    return result
  end; alias available_cookbook_files? cookbook_files? # === available_cookbook_files?

  # ========================================================================= #
  # === chop_off_archive
  #
  # Remove the trailing part of an archive. This isn't very sophisticated
  # but it "just works" (TM).
  # ========================================================================= #
  def chop_off_archive(i)
    if i.include? '#' # Chop off at '#' characters.
      i = i[0..(i.index('#')-1)]
    end
    i.sub!(/\.source$/,'') if i.include? 'source'
    i.sub!(/-stable$/,'')  if i.end_with? '-stable'
    i = i.sub(/&type=zip$/,'').
          sub(/\.lz$/,'').
          sub(/\.gz$/,'').
          sub(/\.xz$/,'').
          sub(/\.bz2$/,'').
          sub(/\.tar$/,'').
          sub(/\.zip$/,'').
          sub(/\.js$/,'')
    return i
  end

  # ========================================================================== #
  # === opnn
  # ========================================================================== #
  def opnn(namespace = NAMESPACE)
    if namespace.is_a? String
      namespace = namespace.delete('.')
      namespace = { namespace: namespace }
    end
    Opn.opn(namespace)
  end

  # ========================================================================= #
  # === be_verbose=
  # ========================================================================= #
  def be_verbose=(i)
    @be_verbose = i
  end

  # ========================================================================= #
  # === on_roebe?
  # ========================================================================= #
  def on_roebe?
    ::Cookbooks.on_roebe?
  end; alias is_on_roebe? on_roebe? # === is_on_roebe?

  # ========================================================================= #
  # === set_be_verbose
  #
  # By default we will be verbose.
  # ========================================================================= #
  def set_be_verbose(i = true)
    @be_verbose = i
  end; alias be_verbose set_be_verbose # Not sure if we will keep this here.

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @be_verbose
  end

  # ========================================================================= #
  # === be_verbose
  # ========================================================================= #
  def be_verbose
    @be_verbose = true
  end

  # ========================================================================= #
  # === set_be_silent
  #
  # Set whether we will be verbose or whether we will not.
  # ========================================================================= #
  def set_be_silent(i = false) # false because we assign to @be_verbose
    @be_verbose = i
  end; alias be_silent set_be_silent # === be_silent()
       alias be_quiet  set_be_silent # === be_quiet()

  # ========================================================================= #
  # === system_directory?
  # ========================================================================= #
  def system_directory?
    ::Cookbooks.system_directory?
  end

  # ========================================================================= #
  # === store_into_this_directory?
  # ========================================================================= #
  def store_into_this_directory?
    _ = ::Cookbooks.store_into_this_directory?
    ensure_that_this_directory_exists(_) # Make sure that the directory exists.
    return _
  end; alias store_where? store_into_this_directory? # === store_where?

  # ========================================================================= #
  # === remove_file_extension
  #
  # This method will try to remove anything we declare to be an improper
  # file extension. We do this by delegating towards class RemoveFileSuffix,
  # an external dependency.
  # ========================================================================= #
  def remove_file_extension(i, also_remove_the_basename = true)
    case also_remove_the_basename
    when :do_not_strip_anything
      also_remove_the_basename = false
    end
    if also_remove_the_basename
      i = File.basename(i)
    end
    if    i.end_with? '.src'
      i[-4,4] = ''
    elsif i.end_with? '-src'
      i[-4,4] = ''
    end
    i = remove_unnecessary_data_from_url(i.to_s.dup)
    ::Cookbooks.remove_file_suffix(i) # Always return it.
  end; alias remove_extension   remove_file_extension # === remove_extensions
       alias remove_file_suffix remove_file_extension # === remove_file_suffix

  # ========================================================================= #
  # === display_md5sum?
  # ========================================================================= #
  def display_md5sum?
    ::Cookbooks.display_md5sum?
  end; alias show_md5sum? display_md5sum? # === show_md5sum?

  # ========================================================================= #
  # === do_not_show_names
  # ========================================================================= #
  def do_not_show_names
    ::Cookbooks.do_not_show_names
  end

  # ========================================================================= #
  # === all_binaries?
  # ========================================================================= #
  def all_binaries?
    ::Cookbooks.all_binaries?
  end

  # ========================================================================= #
  # === all_libraries?
  # ========================================================================= #
  def all_libraries?
    ::Cookbooks.all_libraries?
  end

  # ========================================================================= #
  # === project_yaml_directory?
  # ========================================================================= #
  def project_yaml_directory?
    ::Cookbooks::PROJECT_YAML_DIRECTORY
  end

  # ========================================================================= #
  # === programs_dir?
  # ========================================================================= #
  def programs_dir?
    ::Cookbooks.programs_directory?.dup
  end; alias programs_directory? programs_dir? # === programs_directory?

  # ========================================================================= #
  # === return_archive_type
  # ========================================================================= #
  def return_archive_type(i)
    ::Cookbooks.return_archive_type(i)
  end

  # ========================================================================= #
  # === esystem
  #
  # Combine system() with output of the command.at hand.
  # ========================================================================= #
  def esystem(i)
    e i
    system i
  end

  # ========================================================================= #
  # === source_directory?
  # ========================================================================= #
  def source_directory?
    ::Cookbooks.source_directory?
  end; alias source_directory source_directory? # === source_directory
       alias source_dir?      source_directory? # === source_dir?
       alias archive_dir?     source_directory? # === archive_dir?

  # ========================================================================= #
  # === change_directory
  #
  # Use this to change directory.
  # ========================================================================= #
  def change_directory(i, be_verbose = true)
    case be_verbose
    when :be_quiet
      be_verbose = false
    end
    if be_verbose
      e 'Changing into the directory `'+sdir(i.to_s)+'` now.'
    end
    FileUtils.cd(i)
  end; alias cd change_directory # === cd (cd tag)

  # ========================================================================= #
  # === verbose_truth
  #
  # This will give us back "yes" or "no", in String form.
  # ========================================================================= #
  def verbose_truth(i)
    ::Cookbooks.verbose_truth(i)
  end; alias vt verbose_truth # === vt

  # ========================================================================== #
  # === to_camelcase
  # ========================================================================== #
  def to_camelcase(i)
    i.split('_').map { |_| _.capitalize }.join
  end

  # ========================================================================= #
  # === word_wrap
  #
  # The first argument is the text that will be reformatted.
  #
  # The second argument is at which position we will wrap it.
  # ========================================================================= #
  def word_wrap(this_text, n_characters_limit = 78)
    unless this_text.encoding.to_s.include? main_encoding?
      this_text = ensure_main_encoding_for(this_text)
    end
    this_text.gsub(/(.{1,#{n_characters_limit}})(\s+|$)/, "\\1\n")
  end

  # ========================================================================= #
  # === ensure_main_encoding_for
  #
  # The input to this method should be a String object.
  # ========================================================================= #
  def ensure_main_encoding_for(i)
    unless i.encoding.to_s.include? USE_MAIN_ENCODING
      i = i.force_encoding(USE_MAIN_ENCODING)
    end
    return i
  end

  # ========================================================================= #
  # === main_encoding?
  # ========================================================================= #
  def main_encoding?
    USE_THIS_ENCODING
  end

  # ========================================================================= #
  # === individual_cookbooks_directory?
  #
  # Pointer towards the local individual_cookbooks_directory/ directory.
  # ========================================================================= #
  def individual_cookbooks_directory? 
    ::Cookbooks.individual_cookbooks_directory?
  end; alias cookbooks_directory?      individual_cookbooks_directory? # === cookbooks_directory?
       alias cookbook_directory?       individual_cookbooks_directory? # === cookbook_directory?
       alias individual_cookbooks_dir? individual_cookbooks_directory? # === individual_cookbooks_dir?

  # ========================================================================= #
  # === temp_directory?
  # ========================================================================= #
  def temp_directory?
    ::Cookbooks.temp_directory?
  end; alias temp_dir? temp_directory? # === temp_dir?
       alias log_dir?  temp_directory? # === log_dir?

  # ========================================================================== #
  # === expanded_cookbooks_directory_exists?
  #
  # This method will try if the expanded-cookbooks directory exists, and
  # false otherwise.
  # ========================================================================== #
  def expanded_cookbooks_directory_exists?
    File.directory? temp_directory?+'CookbookDirectory/expanded_cookbooks/'
  end

  # ========================================================================== #
  # === CookbookDirectory?
  #
  # The name is not pretty, but we use it for now. Perhaps at a later
  # time we will change it to "cookbook_directory?" or something like
  # that, but presently (Jan 2018) this conflicts with an alias. We
  # may have to get rid of the alias, and also change all the code
  # that uses that alias.
  # ========================================================================== #
  def CookbookDirectory?
    temp_directory?+'CookbookDirectory/'
  end

end; end

if __FILE__ == $PROGRAM_NAME
  require 'colour_e/autoinclude' # Some testing code follows next.
  x = Cookbooks::Base.new
  e x.return_weekday_based_on_utc
  e x.return_utc_time_in_a_format_similar_to_slackware
end