#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CheckForNewReleaseOnRubygems
#
# This class accepts a single argument, the remote URL to a .gem project.
# It will then check for an update, and if there is one, automatically
# update this entry by tapping into class UpdateEntry.
# =========================================================================== #
# require 'cookbooks/check_for_updates/check_for_new_release_on_rubygems.rb'
# =========================================================================== #
require 'cookbooks/check_for_updates/base.rb'
require 'cookbooks/utility_scripts/update_entry.rb'

module Cookbooks

class CheckForNewReleaseOnRubygems < CheckForRemoteWebpages

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === RUBYGEMS_ORG_DOWNLOADS_URL
  # ========================================================================= #
  RUBYGEMS_ORG_DOWNLOADS_URL = 'https://rubygems.org/downloads/'

  # ========================================================================= #
  # === USE_THIS_REGEX
  #
  # This is the main regex that will be used.
  #
  # See: http://rubular.com/r/QcooOoIXVU
  # ========================================================================= #
  USE_THIS_REGEX =
    /    <a rel="nofollow" class="gem__link t-list__item" id="download" href="\/downloads\/(.+).gem">Download<\/a>/

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      remote_url,
      run_already = true
    )
    reset
    set_remote_url(remote_url)
    case remote_url
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_remote_url
  # ========================================================================= #
  def set_remote_url(i)
    i = i.first if i.is_a? Array
    @remote_url = i
  end

  # ========================================================================= #
  # === remote_url?
  # ========================================================================= #
  def remote_url?
    @remote_url
  end

  # ========================================================================= #
  # === check_this_remote_url
  # ========================================================================= #
  def check_this_remote_url(remote_url)
    set_remote_url(remote_url)
    run
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === identify_homepage
  #
  # This method will determine the homepage of the given .gem file at
  # hand. Keep in mind that a homepage on rubygems.org may have '-'
  # characters.
  # ========================================================================= #
  def identify_homepage
    _ = remote_url?
    if _.end_with? '.gem'
      basename = File.basename(_)
      basename = ProgramInformation.new(basename).program_name?
      basename << '/' unless basename.end_with? '/'
      @homepage = 'https://rubygems.org/gems/'+basename
      opnn; e 'Next checking for any updates '\
              'on the remote URL '+sfancy(@homepage)
      obtain_dataset
      apply_the_main_regex_onto_the_dataset
      compare_the_remote_program_version_with_the_local_program_version
    end
  end

  # ========================================================================= #
  # === compare_the_remote_program_version_with_the_local_program_version
  # ========================================================================= #
  def compare_the_remote_program_version_with_the_local_program_version
    array_we_did_update_these_programs = []
    require 'cookbooks/class/class.rb' unless Cookbooks.const_defined? :Cookbook
    program_name = File.basename(@homepage)
    local_version = ::Cookbooks::Cookbook.new(program_name) { :bypass_menu_check }.program_version?
    remote_program_version = ProgramInformation.new(@remote_program_version).version?
    if remote_program_version == local_version
      opnn; e "The local program #{sfancy(program_name)} is "\
              "#{turquoise('up to date')}"\
              "! (At version: #{simp(local_version)})"
    else
      # ===================================================================== #
      # We must build up the remote download-URL next.
      # ===================================================================== #
      remote_download_url = RUBYGEMS_ORG_DOWNLOADS_URL+@remote_program_version+'.gem'
      opnn; e 'Now updating to a new program version '\
              'at `'+sfancy(remote_download_url)+'`.'
      UpdateEntry.new(remote_download_url) # <- Delegate towards UpdateEntry.
      array_we_did_update_these_programs << remote_download_url
    end
    unless array_we_did_update_these_programs.empty?
      opnn; e "The following "\
              "#{simp(array_we_did_update_these_programs.size.to_s)}"\
              " programs were updated:"
      array_we_did_update_these_programs.each {|remote_url|
        e sfancy('  '+remote_url)
      }
    end
  end

  # ========================================================================= #
  # === apply_the_main_regex_onto_the_dataset
  # ========================================================================= #
  def apply_the_main_regex_onto_the_dataset
    @dataset =~ USE_THIS_REGEX
    @remote_program_version = $1.to_s.dup
  end

  # ========================================================================= #
  # === obtain_dataset
  #
  # The first URL to this method should be the remote homepage of the
  # given .gem at hand.
  # ========================================================================= #
  def obtain_dataset(
      use_this_url = @homepage
    )
    begin
      @dataset = open(use_this_url).read
    rescue OpenURI::HTTPError
      opnn; e 'OpenURI::HTTPError error for `'+sfancy(use_this_url)+'`.'
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    identify_homepage
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CheckForNewReleaseOnRubygems.new(ARGV)
end # check_for_new_release_on_rubygems https://rubygems.org/downloads/actionmailer-5.1.4.gem