#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CheckForSlackwareUpdates
#
# Usage example:
#   Cookbooks::CheckForSlackwareUpdates.new
# =========================================================================== #
# require 'cookbooks/check_for_updates/check_for_slackware_updates.rb'
# =========================================================================== #
require 'cookbooks/check_for_updates/base.rb'

module Cookbooks

class CheckForSlackwareUpdates < CheckForRemoteWebpages # === Cookbooks::CheckForSlackwareUpdates.new

  # ========================================================================= #
  # === USE_THIS_SLACKWARE_SEPARATOR
  # ========================================================================= #
  USE_THIS_SLACKWARE_SEPARATOR = '+--------------------------+'

  # ========================================================================= #
  # === USE_THIS_REGEX_FOR_SLACKWARE
  #
  # See this link: http://rubular.com/r/XejliI9eeg
  # ========================================================================= #
  USE_THIS_REGEX_FOR_SLACKWARE =
    /[a-z]{1}\/(.+)\.txz:\s*Upgraded.<br \/>/

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === remote_webpage?
  # ========================================================================= #
  def remote_webpage?
    REMOTE_SLACKWARE_HOMEPAGE
  end; alias remote_website? remote_webpage? # === remote_website?

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === main_regex?
  # ========================================================================= #
  def main_regex?
    USE_THIS_REGEX_FOR_SLACKWARE
  end

  # ========================================================================= #
  # === scan_dataset_for_matches_against_the_regex
  # ========================================================================= #
  def scan_dataset_for_matches_against_the_regex
    @scanned_results = @dataset.scan(main_regex?).flatten
    # ======================================================================= #
    # Next, sanitize these results. We will chop off trailing
    # '-x86_64-1' tokens. 'Keep in mind that we may also have entries
    # such as 'kernel-headers-4.9.49-x86-1'.
    # ======================================================================= #
    @scanned_results.map! {|entry|
      entry.sub!(/-x86_64-1$/,'')
      entry.sub!(/-x86-1$/,'')
      entry.sub!(/-noarch-1$/,'')
      entry
    }
  end

  # ========================================================================= #
  # === limit_dataset_to_last_separator
  #
  # This will limit the dataset to the last separator.
  # ========================================================================= #
  def limit_dataset_to_last_separator
    @dataset = @dataset[0..(@dataset.index(USE_THIS_SLACKWARE_SEPARATOR)-1)]
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === iterate_over_the_results
  # ========================================================================= #
  def iterate_over_the_results
    n_ljust = 45
    # ======================================================================= #
    # Make use of ProgramInformation next.
    # ======================================================================= #
    @scanned_results.each {|this_result|
      _ = ProgramInformation.new(this_result)
      real_short_name = _.real_short_name?
      short_name      = _.short_name?
      remote_version  = _.program_version?
      if does_the_cookbook_include_this_program? real_short_name
        string = 'The local version of '.dup
        string << short_name.dup
        string << ' is: '
        string = string.ljust(n_ljust)
        string.sub!(/(#{short_name.dup})/, sfancy('\\1'))
        opnn; e string+
                simp(return_local_version_of_this_program(short_name))
        opnn; e 'The remote version is: '.ljust(n_ljust)+
                orange(remote_version)
      else
        opnn; e 'Not found a local program called `'+sfancy(this_result)+'`.'
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_the_remote_webpage
    obtain_dataset_from_the_remote_webpage
    limit_dataset_to_last_separator
    scan_dataset_for_matches_against_the_regex
    iterate_over_the_results
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CheckForSlackwareUpdates.new(ARGV)
end # check_for_slackware_updates