#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Cookbook
#
# This is the general entry-point class for the Cookbooks project.
#
# From holding the dataset of a specific cookbook at hand, over to
# displaying said dataset - this is the class to go for.
# =========================================================================== #
# require 'cookbooks/class/class.rb'
# =========================================================================== #
begin # Add the program_information gem - and a notification if it is not there.
  require 'program_information'
rescue LoadError
  puts 'Please install the gem called program_information'
end
require 'cookbooks/project_settings/project_yaml_directory.rb'
require 'cookbooks/requires/common_requires.rb'
require 'cookbooks/constants/encoding.rb'
require 'cookbooks/constants/misc.rb'
require 'cookbooks/constants/archives.rb'
require 'cookbooks/constants/directory_constants.rb'
require 'cookbooks/constants/registered_cookbook_entries.rb'
require 'cookbooks/constants/show_names.rb'
require 'cookbooks/directory/individual_cookbooks_directory.rb'
# =========================================================================== #
# Next, some important cookbooks module_methods.
# =========================================================================== #
require 'cookbooks/module_methods/are_we_on_gobolinux.rb'
require 'cookbooks/module_methods/time.rb'
require 'cookbooks/module_methods/available_programs.rb'
require 'cookbooks/module_methods/is_on_roebe.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/module_methods/remove_archive_from_the_end.rb'
# =========================================================================== #
# Next, pull in the cookbooks/class/ .rb files.
# =========================================================================== #
require 'cookbooks/class/initialize.rb'
require 'cookbooks/class/constants.rb'
require 'cookbooks/class/dataset.rb'
require 'cookbooks/class/ensure.rb'
require 'cookbooks/class/feedback.rb'
require 'cookbooks/class/feedback/feedback.rb'
require 'cookbooks/class/guess.rb'
require 'cookbooks/class/menu.rb'
require 'cookbooks/class/obtain.rb'
require 'cookbooks/class/report.rb'
require 'cookbooks/class/reset.rb'
require 'cookbooks/class/run.rb'
require 'cookbooks/class/sanitize.rb'
require 'cookbooks/class/set.rb'
require 'cookbooks/class/class_methods/class_methods.rb'
# =========================================================================== #
# And next, cookbooks/base related code.
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/aliases/cookbook_aliases.rb'

module Cookbooks

class Cookbook < Base # === Cookbooks::Cookbook.new

  # ========================================================================= #
  # === replace_plus_with_long_name?
  # ========================================================================= #
  def replace_plus_with_long_name?
    @internal_hash[:replace_plus_with_long_name]
  end

  # ========================================================================= #
  # === enable_debug
  # ========================================================================= #
  def enable_debug
    @internal_hash[:debug] = true
  end

  # ========================================================================= #
  # === valid_key?
  #
  # You can use this method to determine whether we have a specific, valid
  # key or whether we don't.
  # ========================================================================= #
  def valid_key?(key)
    _ = true
    _ = false unless registered_cookbook_entries?.include? key.to_s # The key must be a String.
    return _
  end; alias is_a_valid_key? valid_key? # === is_a_valid_key?

  # ========================================================================= #
  # === show_tags
  # ========================================================================= #
  def show_tags
    e return_tags
  end

  # ========================================================================= #
  # === to_bool
  # ========================================================================= #
  def to_bool(i)
    case i
    when 't','true','yes','y'
      return true
    when 'f','false','no','n'
      return false
    end
    i
  end; alias to_boolean to_bool # === to_boolean

  # ========================================================================= #
  # === the_program_is_included
  #
  # This method will set-toggle that the program we are looking for,
  # is included; aka that it was found.
  # ========================================================================= #
  def the_program_is_included
    @internal_hash[:is_the_program_included] = true
  end

  # ========================================================================= #
  # === show_n_programs_available
  # ========================================================================= #
  def show_n_programs_available
    e return_n_programs_available
  end

  # ========================================================================= #
  # === calculate_archive_size
  #
  # This method can be used to determine the size of the target archive
  # in question. The corresponding entry will be set in the main Hash,
  # called 'archive_size'.
  #
  # In order for this to work, we need the content of program_path defined
  # before, however. Whenever we set program_path, we can re-calculate
  # the filesize anew.
  # ========================================================================= #
  def calculate_archive_size
    if found_result?
      if File.exist? program_path?.to_s
        size = File.size?(program_path?.to_s)
        set_archive_size(size)
      end
    end
  end

  # ========================================================================= #
  # === calculate_md5sum
  #
  # We will set the proper md5sum here via this method.
  # ========================================================================= #
  def calculate_md5sum(i = md5sum?)
    set_md5sum(i)
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(
      namespace:   NAMESPACE,
      use_colours: Cookbooks.use_colours?
    )
  end

  # ========================================================================= #
  # === show_path_to_the_source_directory_yaml_file
  #
  # This method will simply show the path to the source_dir.yml
  # file.
  # ========================================================================= #
  def show_path_to_the_source_directory_yaml_file
    begin
      require 'rbt/project/project_yaml_directory.rb'
      _ = RBT::PROJECT_YAML_DIRECTORY+'source_dir.yml'
      e sfile(_)
    rescue LoadError
      e 'The gem rbt is unavailable.'
    end
  end

  # ========================================================================= #
  # === shall_we_display_the_md5sum?
  #
  # Query whether we shall display the md5sum or whether we shall not.
  # ========================================================================= #
  def shall_we_display_the_md5sum?
    ihash?[:shall_we_display_the_md5sum]
  end; alias display_md5sum? shall_we_display_the_md5sum? # === display_md5sum?

  # ========================================================================= #
  # === quiet_down
  #
  # This method will cause the cookbook to be silent and not report
  # how many programs are registered.
  # ========================================================================= #
  def quiet_down
    set_be_silent # This is defined in the parent class.
    @internal_hash[:report_how_many_programs_are_registered] = false
  end

  # ========================================================================= #
  # === do_extract
  #
  # You can use this method, if you have class Extracter available,
  # to force class Extracter to extract this program.
  # ========================================================================= #
  def do_extract(i = program_path?)
    begin
      require 'extracter' unless Object.const_defined? :Extracter
    rescue LoadError; end
    if Object.const_defined? :Extracter
      # ===================================================================== #
      # Ok, extracter is available so we can continue.
      # ===================================================================== #
      Extracter.new(i?).extracted_path? # Specify the path.
    else
      opnn; e 'class Extracter is not available. Consider installing it via:'
      e
      opnn; e orange('  gem install extracter')
      e
    end
  end

  # ========================================================================= #
  # === last_url?
  #
  # Returns the last url. Should be used if you want to access URLs.
  # ========================================================================= #
  def last_url?
    dataset?.remote_urls.empty? ? '' : dataset?.remote_urls.last  
  end

  # ========================================================================= #
  # === first_url?
  #
  # Returns the first url. Should be used if you want to access URLs.
  # ========================================================================= #
  def first_url?
    dataset?.remote_urls.empty? ? '' : dataset?.remote_urls.first  
  end

  # ========================================================================= #
  # === do_show_full_description
  #
  # This method will allow us to show the full description of a given
  # program.
  # ========================================================================= #
  def do_show_full_description
    # ======================================================================= #
    # The following method-call will toggle the proper behaviour.
    # ======================================================================= #
    ::Cookbooks::Cookbook.do_not_honour_threshold_value
  end

  # ========================================================================= #
  # === check_for_library_names?
  # ========================================================================= #
  def check_for_library_names?
    CHECK_FOR_LIBRARY_NAMES
  end

  # ========================================================================= #
  # === return_default_location_of_this_cookbook
  #
  # This will use the default location based on the search_term.
  # ========================================================================= #
  def return_default_location_of_this_cookbook(
      i = search_term?
    )
    individual_cookbooks_directory?+i+'.yml'
  end; alias return_default_location return_default_location_of_this_cookbook # == return_default_location

  # ========================================================================= #
  # === entry_exists?
  # ========================================================================= #
  def entry_exists?
    File.exist? location?
  end

  # ========================================================================= #
  # === show_configuration_options
  # ========================================================================= #
  def show_configuration_options
    ShowConfigurationOptions.new # This is automatically required.
  end

  # ========================================================================= #
  # === check_for_binary_names?
  # ========================================================================= #
  def check_for_binary_names?
    CHECK_FOR_BINARY_NAMES
  end

  # ========================================================================= #
  # === report_binary_or_library_program_name
  #
  # Only call this method when we have found an alias.
  # ========================================================================= #
  def report_binary_or_library_program_name(
      i, new_program_name, binary_or_library = 'binary'
    )
    case binary_or_library
    when :binary
      binary_or_library = 'binary'
    when :library
      binary_or_library = 'library'
    end
    opnn; e '`'+simp(i)+'` was found as name of a '+simp(binary_or_library)+' file.'
    opnn; e 'We will now assume that you want to get this whole package '\
            'instead and will'
    opnn; e 'thus set the name to this program now, which belongs to '\
            'the package `'+sfancy(new_program_name)+'`.'+N
  end

  # ========================================================================= #
  # === output_main_dataset
  # ========================================================================= # 
  def output_main_dataset
    do_not_use_colours
    pp yaml_dataset?
  end

  # ========================================================================= #
  # === n_programs_available?
  # ========================================================================= #
  def n_programs_available?
    Cookbooks.n_programs?
  end

  # ========================================================================= #
  # === return_n_programs_available
  # ========================================================================= # 
  def return_n_programs_available
    '  -> '+sfancy(n_programs_available?.to_s)+' programs '\
    'registered as of '+simp(today?)+'.'
  end

  # ========================================================================= #
  # === do_change_to_appdir_layout
  #
  # You can use this method to change to the AppDir layout.
  # ========================================================================= #
  def do_change_to_appdir_layout
    sanitize_prefix(:return_appdir_path)
  end

  # ========================================================================= #
  # === are_we_on_gobolinux?
  #
  # Determine whether we are on GoboLinux or whether we are not.
  # ========================================================================= #
  def are_we_on_gobolinux?
    ::Cookbooks.are_we_on_gobolinux?
  end

  # ========================================================================= #
  # === return_appdir_path
  #
  # This method will return an App-Dir like path.
  #
  # It may be used from within sanitize_prefix() - that is, when
  # you wish to sanitize the prefix passed.
  #
  # This method requires that short_name? and program_version? were
  # already defined prior to invocation.
  #
  # The appdir components would be like this:
  #   /Programs/Ruby/2.4.1
  #   programs_dir?+real_short_name?.capitalize+'/'+program_version?
  # ========================================================================= #
  def return_appdir_path(
      programs_dir_to_use    = programs_dir?, 
      short_name_to_use      = real_short_name?.capitalize,
      program_version_to_use = program_version?
    )
    _ = programs_dir_to_use.dup
    if are_we_on_gobolinux?
      _ << program_compile_name?.dup
    else
      _ << short_name_to_use.dup
    end
    _ << "/#{program_version_to_use.dup}/"
    return _
  end; alias build_appdir_path    return_appdir_path # === build_appdir_path
       alias return_appdir_prefix return_appdir_path # === return_appdir_prefix

  # ========================================================================= #
  # === update_tags
  #
  # This will simply update the tags, by tapping into the Cookbooks project.
  # ========================================================================= #
  def update_tags
    require 'cookbooks/utility_scripts/registered_tags.rb'
    Cookbooks::RegisteredTags.new
  end

  # ========================================================================= #
  # === load_and_display_the_allowed_tags
  # ========================================================================= #
  def load_and_display_the_allowed_tags
    dataset = YAML.load_file(FILE_ALLOWED_TAGS).sort
    opnn; e 'The following tags are allowed/registered/valid:'
    dataset.each {|entry|
      e '  '+simp(entry)
    }
  end

  # ========================================================================= #
  # === remove_archive_from_the_end
  # ========================================================================= #
  def remove_archive_from_the_end(i)
    ::Cookbooks.remove_archive_from_the_end(i)
  end

  # ========================================================================= #
  # === batch_validate_the_cookbooks
  # ========================================================================= #
  def batch_validate_the_cookbooks
    unless Cookbooks.const_defined? :BatchValidateTheCookbookRecipes
      require 'cookbooks/validation/batch_validate_the_cookbook_recipes.rb'
    end
    BatchValidateTheCookbookRecipes.new
  end

  # ========================================================================= #
  # === do_permanently_use_psych
  #
  # Use this method if you wish to permanently use psych, rather than
  # syck.
  # ========================================================================= #
  def do_permanently_use_psych
    opnn; e 'We will use the psych module for yaml-datasets from now on.'
    opnn; e 'This will be stored in the file '+sfile(FILE_USE_PSYCH_OR_SYCK)
    write_what_into(YAML.dump(:psych), FILE_USE_PSYCH_OR_SYCK)
    if is_on_roebe?
      write_what_into(YAML.dump(:psych),
        ROEBE_PATH_TO_COOKBOOKS+'yaml/configuration/'+
        File.basename(FILE_USE_PSYCH_OR_SYCK)
      )
    end
  end

  # ========================================================================= #
  # === do_permanently_use_syck
  #
  # Use this method if you wish to permanently use syck, rather
  # than psych.
  # ========================================================================= #
  def do_permanently_use_syck
    opnn; e 'We will use the syck module for yaml-datasets from now on.'
    opnn; e 'This will be stored in the file '+sfile(FILE_USE_PSYCH_OR_SYCK)
    write_what_into(YAML.dump(:syck), FILE_USE_PSYCH_OR_SYCK)
    if is_on_roebe?
      write_what_into(YAML.dump(:syck),
        ROEBE_PATH_TO_COOKBOOKS+'yaml/configuration/'+
        File.basename(FILE_USE_PSYCH_OR_SYCK)
      )
    end
  end

  # ========================================================================= #
  # === update_kde_plasma
  #
  # Use this method, as delegation, in order to update the KDE5 plasma
  # components.
  # ========================================================================= #
  def update_kde_plasma
    unless Cookbooks.const_defined? :UpdateKdePlasma
      require 'cookbooks/check_for_updates/update_kde_plasma.rb'
    end
    UpdateKdePlasma.new
  end

  # ========================================================================= #
  # === update_kde_framework
  # ========================================================================= #
  def update_kde_framework
    unless Cookbooks.const_defined? :UpdateKdeFramework
      require 'cookbooks/check_for_updates/update_kde_framework.rb'
    end
    UpdateKdeFramework.new
  end

  # ========================================================================= #
  # === update_kde_porting_aids
  #
  # To invoke this, try:
  #   cookbooks --update-kde-porting-aids
  # ========================================================================= #
  def update_kde_porting_aids
    unless Cookbooks.const_defined? :UpdateKdePortingAids
      require 'cookbooks/check_for_updates/update_kde_porting_aids.rb'
    end
    UpdateKdePortingAids.new
  end

  # ========================================================================= #
  # === update_mate_desktop_components
  #
  # Update mate-desktop via this method.
  #
  # Invocation example:
  #   cookbooks --update-mate
  # ========================================================================= #
  def update_mate_desktop_components
    unless Cookbooks.const_defined? :CheckForMateDesktopUpdates
      require 'cookbooks/check_for_updates/check_for_mate_desktop_updates.rb'
    end
    CheckForMateDesktopUpdates.new
  end

  # ========================================================================= #
  # === show_all_about
  # ========================================================================= #
  def show_all_about(show_this_program)
    require 'rbt/utility_scripts/show_all_about.rb'
    RBT::ShowAllAbout.new(show_this_program)
  end

  # ========================================================================= #
  # === show_available_aliases
  #
  # To invoke this method, do:
  #   cookbooks --aliases?
  # ========================================================================= #
  def show_available_aliases
    _ = Cookbooks.cookbook_aliases?
    opnn; e "The available #{sfancy(_.size.to_s)} aliases are:"
    index = 0
    _.each_pair {|key, value| index += 1
      colourized_index = slateblue((index.to_s+')').rjust(5))
      e colourized_index+' '+sfancy(key.ljust(18))+
        mediumseagreen(' ->  ')+simp(value)
    }
  end

  # ========================================================================= #
  # === does_include?
  # ========================================================================= #
  def does_include?(
      i, honour_aliases = make_use_of_aliases_kept_in_the_file_cookbook_aliases?
    )
    ::Cookbooks.does_include? i, honour_aliases # Delegate towards the class method.
  end; alias is_included? does_include? # === is_included?

  # ========================================================================= #
  # === do_not_make_use_of_aliases_kept_in_the_file_cookbook_aliases
  # ========================================================================= #
  def do_not_make_use_of_aliases_kept_in_the_file_cookbook_aliases
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases] = false
  end

  # ========================================================================= #
  # === set_use_menu_interface
  #
  # This method allows you to bypass the menu-interface.
  # ========================================================================= #
  def set_use_menu_interface(
      i = :bypass_menu
    )
    case i
    when :bypass_menu, :bypass_menu_check, :skip_menu_interface
      i = false
    end
    @use_menu_interface = i
  end

  # ========================================================================= #
  # === make_use_of_aliases_kept_in_the_file_cookbook_aliases?
  #
  # This method can be used to query whether we will make use of aliases
  # kept in the cookbook_aliases.yml file or whether we will not.
  # ========================================================================= #
  def make_use_of_aliases_kept_in_the_file_cookbook_aliases?
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases]
  end; alias make_use_of_cookbook_aliases? make_use_of_aliases_kept_in_the_file_cookbook_aliases? # === make_use_of_cookbook_aliases?

  # ========================================================================= #
  # === this_program_was_not_found
  # ========================================================================= #
  def this_program_was_not_found(i)
    unless i.to_s.empty?
      opnn; e 'The program '+sfancy(i.to_s)+' does not appear to be registered.'
    end
  end

  # ========================================================================= #
  # === update_kde_applications
  # ========================================================================= #
  def update_kde_applications
    unless Cookbooks.const_defined? :UpdateKdeApplications
      require 'cookbooks/check_for_updates/update_kde_applications.rb'
    end
    UpdateKdeApplications.new
  end

  # ========================================================================= #
  # === register_invalid_key
  #
  # We store invalid entries in an Array, via this method here.
  # ========================================================================= #
  def register_invalid_key(i)
    i = i.to_s # To prevent nil-related errors.
    unless array_invalid_keys?.include? i
      array_invalid_keys? << i
    end unless i.empty?
  end

  # ========================================================================= #
  # === registered_cookbook_entries?
  # ========================================================================= #
  def registered_cookbook_entries?
    ::Cookbooks.registered_cookbook_entries?
  end

  # ========================================================================= #
  # === substitute_macros
  #
  # Do "macro" expansion with this method. We will sanitize some
  # variables, which we assume were meant to be "macros" - that
  # is, to be substituted with another value..
  #
  # Right now this is only used for kernel variable, PROGRAM_NAME, PNAME,
  # NAME, KERNEL_V and PROGRAM_VERSION but maybe someone else wants to do
  # more macro processing, so we will keep this bundled here together.
  # ========================================================================= #
  def substitute_macros(input)
    if input.is_a? Array
      input.map {|entry| substitute_macros(entry) } # Recursive call.
    else
      i = input.to_s.dup # Work on a new copy here.
      if i.include? 'PROGRAM_VERSION'
        i.gsub!(/PROGRAM_VERSION/, program_version?)
      end
      if i.include? 'PNAME'
        i.gsub!(/PNAME/,           short_name?)
      elsif i.include? 'NAME'
        i.gsub!(/NAME/,            short_name?)
      elsif i.include? 'PROGRAM_NAME'
        i.gsub!(/PROGRAM_NAME/,    short_name?)
      end
      if ENV['KERNEL_V'] && i.include?('KERNEL_V')
        i.gsub!(/#{i}/, ENV['KERNEL_V'])
      end
      i
    end
  end

end

# =========================================================================== #
# === Cookbooks.new
#
# Invocation example:
#   ruby     = Cookbooks.new 'ruby'
#   zip      = Cookbooks.new 'zip'
#   xxf86dga = Cookbooks.new 'xxf86dga'
# =========================================================================== #
def self.new(i = 'htop')
  ::Cookbooks::Cookbook.new(i) { :bypass_menu }
end; self.instance_eval { alias [] new } # === Cookbooks[]

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::Cookbook.new(ARGV)
end # cookbooks_rb_file htop