#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
module Cookbooks

class Cookbook

  # ========================================================================= #
  # === initialize_yaml_dataset
  # ========================================================================= #
  def initialize_yaml_dataset
    set_archive_size 0 # Initialize the program size to 0. It will may be changed lateron.
    # ======================================================================= #
    # Add nil, array, empty values and booleans.
    # ======================================================================= #
    ensure_that_the_nil_values_are_there
    ensure_that_the_array_values_are_there
    ensure_that_the_empty_values_are_there
    # ======================================================================= #
    # === Ensure that the boolean values (true and false) will be there
    # ======================================================================= #
    ensure_that_the_boolean_values_are_there
    # ======================================================================= #
    # Last but not least, add the missing entries.
    # ======================================================================= #
    fill_in_missing_entries
  end

  # ========================================================================= #
  # === set_yaml_dataset
  #
  # Keep track of the yaml_dataset through this method.
  # ========================================================================= #
  def set_yaml_dataset(i = nil)
    case i
    when :clear, :reset
      i = {} # Must be a Hash.
    end
    ihash[:yaml_dataset] = i
  end

  # ========================================================================= #
  # === has_key?
  #
  # This query method will delegate towards the yaml dataset, whether
  # it has a given key or whether it does not.
  # ========================================================================= #
  def has_key?(i = :blfs)
    yaml_dataset?.has_key?(i.to_s)
  end

  # ========================================================================= #
  # === clear_yaml_dataset
  # ========================================================================= #
  def clear_yaml_dataset
    set_yaml_dataset :clear
  end

  # ========================================================================= #
  # === load_yaml_dataset_from_this_file
  #
  # This is the main method, the powerhorse of the class - it will load
  # up the dataset for a given program, from the .yml file at hand.
  #
  # In order for this to work, the .yml file must of course exist
  # locally.
  #
  # You can also pass the name of the program, such as "htop", rather than
  # specify the full path. The method will automatically expand in such
  # a case.
  # ========================================================================= #
  def load_yaml_dataset_from_this_file(
      i = search_for_this_program?
    )
    original_search_term = i.dup
    clear_yaml_dataset
    # ======================================================================= #
    # Initialize the yaml dataset first.
    # ======================================================================= #
    initialize_yaml_dataset
    # ======================================================================= #
    # Next, check whether there is any '/' character in the given input.
    # If not, then we will expand to the real, full path.
    # ======================================================================= #
    unless i.empty? # Ignore empty Strings.
      unless i.include?('/')
        if make_use_of_cookbook_aliases?
          if Cookbooks.is_an_alias?(i) and
            !Cookbooks.program_is_included?(i, :do_not_check_for_aliases)
            original_name = i.dup
            # ================================================================= #
            # Ok, it is an alias. This must also re-rewrite the program
            # that we are searching for.
            # ================================================================= #
            i = Cookbooks.find_alias_for(original_name)
            opnn; e "The alias name #{sfancy(i)} will be "\
                    "used for #{simp(original_name)}."
            set_search_for_this_program(i)
          end
          i = return_default_location_of_this_cookbook(i)
        end
      end
    end
    if debug?
      opnn; e 'Now loading the dataset from the file `'+sfile(i)+'`.'
    end
    unless i.end_with? '.yml'
      i = return_default_location_of_this_cookbook(i)
    end
    # ======================================================================= #
    # The file must exist, because otherwise we can not load up the yaml
    # dataset. We will also have to ensure that it is a file.
    # ======================================================================= #
    if File.exist?(i) and File.file?(i)
      the_program_is_included
      dataset = YAML.load_file(i)
      update_yaml_dataset(dataset)
      set_short_name nil # Must be specifically set to nil.
      if yaml_dataset_exists?
        sync_yaml_dataset_down_one_level
        sanitize_yaml_dataset
      end
    else
      this_program_was_not_found(original_search_term)
    end
  end

  # ========================================================================= #
  # === sync_yaml_dataset_down_one_level
  # ========================================================================= #
  def sync_yaml_dataset_down_one_level(
      this_program = seek_which_program?
    )
    dataset = ihash[:yaml_dataset]
    # ======================================================================= #
    # Obtain the dataset, through the pointer to the program being sought.
    # ======================================================================= #
    shortened_dataset = dataset[this_program]
    ensure_that_this_hash_has_only_valid_keys(shortened_dataset) # Must come early.
    update_yaml_dataset(shortened_dataset)
    # ======================================================================= #
    # Next, remove the old pointer.
    # ======================================================================= #
    dataset.delete(this_program)
    # ======================================================================= #
    # We can also sync towards real_short_name, since real_short_name
    # is the variant without any '-' characters. We can not yet determine
    # short_name though.
    # ======================================================================= #
    set_real_short_name(this_program)
  end

  # ========================================================================= #
  # === update_yaml_dataset
  #
  # This can be used to update the yaml dataset.
  #
  # However had, for boolean values that are to be added, we will
  # ALWAYS sanitize them, so that "t" becomes true, and "f" becomes
  # false. But only if it is a boolean value.
  # ========================================================================= #
  def update_yaml_dataset(i)
    if i.keys.size > 1 # This check excludes Hashes that were not yet synced down one level.
      i.update(i) {|key, value|
        if is_a_boolean_configuration_value?(key)
          value = to_boolean(value)
        else
          value
        end
      }
    end
    ihash[:yaml_dataset].update(i) # .update() will replace in-place.
  end

  # ========================================================================= #
  # === update_with_this_program
  #
  # This is a compound-method. It will first set the new program
  # to be searched for; and then it will conditionally load the
  # yaml dataset, if the program is included.
  # ========================================================================= #
  def update_with_this_program(i = search_which_program?)
    i = i.to_s.dup if i.frozen?
    i.delete!('-') if i.include? '-'
    if i.end_with? '.yml'
      i.sub!(/\.yml$/,'')
    elsif i.end_with? '.pc'
      i.sub!(/\.pc$/,'')
    end
    set_search_for_this_program(i)
    load_yaml_dataset_from_this_file(i)
  end; alias update            update_with_this_program # === update
       alias set_dataset       update_with_this_program # === set_dataset
       alias assign_and_run    update_with_this_program # === assign_and_run
       alias assign            update_with_this_program # === assign
       alias find_this_program update_with_this_program # === find_this_program
       alias find              update_with_this_program # === find

end; end