#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# class Cookbook also has to ensure that certain values are set to default
# (and thus correct) values. This file here ensures this.
#
# All methods that have something to do with ensure-functionality are
# bundled together in this file here.
# =========================================================================== #
# require 'cookbooks/class/ensure.rb'
# =========================================================================== #
module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === ensure_that_the_nil_values_are_there                        (nil tag)
  # ========================================================================= #
  def ensure_that_the_nil_values_are_there(
      nil_values = ARRAY_NIL_ENTRIES
    )
    nil_values.each {|this_entry_must_be_nil|
      set(this_entry_must_be_nil, nil) unless has_key? this_entry_must_be_nil
    }
  end

  # ========================================================================= #
  # === ensure_proper_true_false_array_and_empty_values          (ensure tag)
  #
  # This method will ensure that the main hash will have certain
  # default values set.
  # ========================================================================= #
  def ensure_proper_true_false_array_and_empty_values
    ensure_that_the_boolean_values_are_there # <- Will set true and false values.
    ensure_that_the_empty_values_are_there
    ensure_that_the_array_values_are_there
  end

  # ========================================================================= #
  # === ensure_that_the_boolean_values_are_there
  # ========================================================================= #
  def ensure_that_the_boolean_values_are_there
    ensure_that_the_false_values_are_there
    ensure_that_the_true_values_are_there
  end

  # ========================================================================= #
  # === ensure_that_the_array_values_are_there
  #
  # This method will ensure that all specified Array entries are part
  # of the internal yaml dataset.
  # ========================================================================= #
  def ensure_that_the_array_values_are_there
    ARRAY_ARRAY_ENTRIES.each {|this_entry_must_become_an_array|
      set(this_entry_must_become_an_array, []) unless has_key? this_entry_must_become_an_array
    }
  end

  # ========================================================================= #
  # === ensure_that_the_empty_values_are_there
  #
  # We must ensure that the empty values are also properly set to ""
  # on startup, in a mandatory fashion.
  # ========================================================================= #
  def ensure_that_the_empty_values_are_there
    ARRAY_EMPTY_VALUES.each {|empty_value|
      set(empty_value, '') unless has_key? empty_value
    }
  end

  # ========================================================================= #
  # === ensure_that_the_true_values_are_there
  #
  # Now come the true values, array_true_values. Right now only
  # prefix must default to true - but only if it is empty.
  # ========================================================================= #
  def ensure_that_the_true_values_are_there
    ARRAY_TRUE_VALUES.each {|true_value|
      if has_key? true_value
        sanitize_boolean_value(true_value)
      else
        set(true_value, true)
      end
    }
  end

  # ========================================================================= #
  # === ensure_that_the_false_values_are_there                    (false tag)
  #
  # Here we will set all false values into our main dataset.
  #
  # We do this only if we do not yet have such a value in our dataset though.
  # ========================================================================= #
  def ensure_that_the_false_values_are_there
    # ======================================================================= #
    # Set the false values to false next.
    # ======================================================================= #
    ARRAY_FALSE_VALUES.each {|this_false_value|
      if has_key? this_false_value
        sanitize_boolean_value(this_false_value)
      else
        set(this_false_value, false)
      end
    }
  end

  # ========================================================================= #
  # === ensure_that_this_hash_has_only_valid_keys
  #
  # The input to this method must be a Hash.
  #
  # This method will ensure that the given Hash has only valid keys.
  #
  # In the past, we could use it to convert it into Symbols but
  # this is no longer in use. We now make no further modifications
  # to the original dataset in use.
  #
  # If all keys are valid then we won't have to notify the user about
  # it at all.
  # ========================================================================= #
  def ensure_that_this_hash_has_only_valid_keys(
      i = yaml_dataset?
    )
    if i.nil?
      cliner
      opnn; e sfancy('nil')+' was passed into the method '+
              orange('ensure_that_this_hash_has_only_valid_keys()')
      opnn; e 'This can not be right. Program name was: '+
               sfancy(program_name?.to_s)+'.'
      pp caller()
      pp self
      cliner
    else
      i.each_pair {|key, value|
        unless registered_cookbook_entries?.include?(key)
          seek_this_yaml_file = "#{seek_this_program?}.yml"
          opnn; e "#{swarn('Warning')} - the cookbook-file for `"\
                  "#{sfancy(seek_this_yaml_file)}"\
                  "` contains a key that was"
          opnn; e "#{swarn('NOT')} registered. The name of that (invalid / "\
                  "ignored) key is: `#{sfancy(key.to_s)}`."
          opnn; e 'This is defined in the file '+Colours.sfile(
                  'registered_cookbook_entries.yml')
          register_invalid_key(key)
          # ================================================================= #
          # Next, also try to find a name that is similar.
          # ================================================================= #
          if registered_cookbook_entries?.any? {|entry|
              entry.include? key.to_s
            }
            select_this_entry = registered_cookbook_entries?.select {|entry|
              entry.include? key.to_s
            }.first
            opnn; e "Did you mean this entry instead? "\
                    "#{sfancy(select_this_entry)}"  
          end
        end
      }
    end
  end

end; end