#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# Any query-method that is part of class Cookbooks::Cookbook will be
# stored here.
#
# The proper assembly chain for any dataset in question - which all entries
# need to adhere to - is the following, starting from 1 up to 8:
#
#     (1) real_short_name   like: 'gnomemahjonng'  # This is like short_name but without any possible '-' and also without any '_'
#     (2) short_name        like: 'gnome-mahjonng' # This may invoke (1) and may include '-' characters.   
#     (3) program_version   like: '5.08'
#     (4) archive_type      like: '.tar.bz2' # Yes, must have a leading '.'
#     (5) base_dir          like: '/Users/x/SRC/gnomemahjongg/'
#     (6) program_name      like: 'gnome-mahjonng-5.08' # This one includes the version. It is short_name+'-'+program_version
#     (7) program_full_name like: 'gnome-mahjonng-5.08.tar.xz', so this is program_name?+archive_type?
#     (8) program_path      like: '/Users/x/SRC/gnomemahjongg/gnome-mahjonng-5.08.tar.xz' # base_dir?+program_full_name?
#
# These numbers are also important because we can invoke methods based
# on the information stored in them. For instance, entry (8) can be
# assembled from entry (5) and entry (7).
# =========================================================================== #
# require 'cookbooks/obtain.rb'
# =========================================================================== #
require 'cookbooks/base/prototype.rb'

begin
  require 'md5sum' # Used in this file here.
rescue LoadError; end

module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === has_tags?
  #
  # This method will return true or false depending on whether the given
  # program at hand has any tags registered at all, or whether it has not.
  # ========================================================================= #
  def has_tags?
    return false if tags?.empty?
    return true
  end

  # ========================================================================= #
  # === configure_command?
  #
  # The specific configure command that we will use.
  # ========================================================================= #
  def configure_command?
    obtain(:configure_command)
  end; alias configure_command configure_command? # === configure_command

  # ========================================================================= #
  # === headers?
  # ========================================================================= #
  def headers?
    obtain(:headers)
  end; alias headers headers? # === headers

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    obtain(:url2)
  end; alias url2 url2? # === url2

  # ========================================================================= #
  # === url3?
  # ========================================================================= #
  def url3?
    obtain(:url3)
  end; alias url3 url3? # === url3

  # ========================================================================= #
  # === short_name?
  #
  # This method queries the short name of the program we are trying to
  # find.
  #
  # For example, this method will return the program name without the
  # version, such as:
  #
  #  "ruby" (which is part of the program_name "ruby-2.2.0")
  #
  # This method must always return a String, even if it is an empty String.
  #
  # Characters such as '-' are allowed to be part of the short_name variable.
  # ========================================================================= #
  def short_name?
    obtain(:short_name)
  end; alias short_name short_name? # === short_name
       alias sname?     short_name? # === sname?
       alias shortname? short_name? # === shortname?
       alias program?   short_name? # === program?
       alias name       short_name? # === name
       alias name?      short_name? # === name?

  # ========================================================================= #
  # === use_glib_schema?
  # ========================================================================= #
  def use_glib_schema?
    obtain(:use_glib_schema)
  end; alias use_glib_schema use_glib_schema? # === use_glib_schema

  # ========================================================================= #
  # === symlink_headers?
  # ========================================================================= #
  def symlink_headers?
    obtain(:symlink_headers)
  end; alias symlink_headers               symlink_headers? # === symlink_headers
       alias shall_we_symlink_the_headers? symlink_headers? # === shall_we_symlink_the_headers?

  # ========================================================================= #
  # === postinstall?
  # ========================================================================= #
  def postinstall?
    obtain(:postinstall)
  end

  # ========================================================================= #
  # === use_autogen?
  # ========================================================================= #
  def use_autogen?
    obtain(:use_autogen)
  end; alias use_autogen use_autogen? # === use_autogen

  # ========================================================================= #
  # === preinstall?
  # ========================================================================= #
  def preinstall?
    obtain(:preinstall)
  end

  # ========================================================================= #
  # === optional_start_message
  # ========================================================================= #
  def optional_start_message?
    obtain(:optional_start_message)
  end; alias optional_start_message optional_start_message? # === optional_start_message

  # ========================================================================= #
  # === program_full_name?
  #
  # This entry will have everything except the leading PATH.
  #
  # For instance:
  #  "gpm-1.20.1.tar.xz"
  # ========================================================================= #
  def program_full_name?
    obtain(:program_full_name)
  end; alias program_full_name program_full_name? # === program_full_name

  # ========================================================================= #
  # === use_this_make_command?
  # ========================================================================= #
  def use_this_make_command?
    obtain(:use_this_make_command)
  end; alias use_this_make_command use_this_make_command? # === use_this_make_command

  # ========================================================================= #
  # === real_short_name
  #
  # Reader method for real_short_name. This can only work after
  # we invoked merge_basic_dataset().
  #
  # real_short_name? may never include '-' characters.
  # ========================================================================= #
  def real_short_name?
    obtain(:real_short_name)
  end; alias rshort?    real_short_name? # === rshort?
       alias shortname? real_short_name? # === shortname?

  # ========================================================================= #
  # === symlink_pkgconfig_files?
  # ========================================================================= #
  def symlink_pkgconfig_files?
    obtain(:symlink_pkgconfig_files)
  end; alias symlink_pkgconfig_files symlink_pkgconfig_files? # === symlink_pkgconfig_files

  # ========================================================================= #
  # === pkgconfig_files?
  # ========================================================================= #
  def pkgconfig_files?
    obtain(:pkgconfig_files)
  end; alias pkgconfig_files pkgconfig_files? # === pkgconfig_files

  # ========================================================================= #
  # === configure_base_dir?
  # ========================================================================= #
  def configure_base_dir?
    obtain(:configure_base_dir)
  end

  # ========================================================================= #
  # === base_dir?
  #
  # This is entry (5), a simple query method. It will give us back the
  # base directory of our program in question, such as '/Users/x/SRC/htop/'.
  # ========================================================================= #
  def base_dir?
    obtain(:base_dir)
  end; alias base_dir base_dir? # === base_dir

  # ========================================================================= #
  # === keep_extracted?
  # ========================================================================= #
  def keep_extracted?
    obtain(:keep_extracted)
  end; alias keep_extracted keep_extracted? # === keep_extracted

  # ========================================================================= #
  # === extract_to?
  #
  # This must return a String.
  # ========================================================================= #
  def extract_to?
    obtain(:extract_to)
  end

  # ========================================================================= #
  # === use_autoconf?
  # ========================================================================= #
  def use_autoconf?
    obtain(:use_autoconf)
  end

  # ========================================================================= #
  # === use_build_directory?
  # ========================================================================= #
  def use_build_directory?
    obtain(:use_build_directory)
  end

  # ========================================================================= #
  # === enable_static?
  # ========================================================================= #
  def enable_static?
    obtain(:enable_static)
  end

  # ========================================================================= #
  # === use_this_make_install_command?
  #
  # Which variant of "make install" we will use, if we will use a
  # different variant that is.
  # ========================================================================= #
  def use_this_make_install_command?
    obtain(:use_this_make_install_command)
  end

  # ========================================================================= #
  # === do_not_download?
  # ========================================================================= #
  def do_not_download?
    obtain(:do_not_download)
  end

  # ========================================================================= #
  # === enable_shared?
  # ========================================================================= #
  def enable_shared?
    obtain(:enable_shared)
  end

  # ========================================================================= #
  # === svn_url?
  # ========================================================================= #
  def svn_url?
    obtain(:svn_url)
  end

  # ========================================================================= #
  # === apply_patch?
  #
  # Query method whether we shall apply a patch or whether we shall not.
  # ========================================================================= #
  def apply_patch?
    obtain(:apply_patch)
  end

  # ========================================================================= #
  # === do_not_symlink?
  #
  # Reader method over whether we will not symlink.
  # ========================================================================= #
  def do_not_symlink?
    obtain(:do_not_symlink)
  end

  # ========================================================================= #
  # === sed?
  # ========================================================================= #
  def sed?
    obtain(:sed)
  end

  # ========================================================================= #
  # === blfs?                                                      (blfs tag)
  #
  # The "Beyond linux from scratch" variable will be returned here,
  # should it exist.
  # ========================================================================= #
  def blfs?
    obtain(:blfs)
  end

  # ========================================================================= #
  # === tags?
  # ========================================================================= #
  def tags?
    obtain(:tags)
  end; alias return_tags tags? # === return_tags
       alias tags        tags? # === tags

  # ========================================================================= #
  # === recommended_deps_on?
  # ========================================================================= #
  def recommended_deps_on?
    obtain(:recommended_deps_on)
  end

  # ========================================================================= #
  # === program_version?
  #
  # This method must always return a String.
  #
  # We use it to query the program version of the specific program at hand.
  #
  # For instance, if the program name is "python-2.4" then this
  # method will return the version string part, i.e. "2.4".
  # ========================================================================= #
  def program_version?
    obtain(:program_version)
  end; alias version                program_version? # === version An alias to the above ^^^.
       alias version?               program_version? # === version?
       alias pversion?              program_version? # === pversion?
       alias program_version        program_version? # === program_version
       alias return_program_version program_version? # === return_program_version

  # ========================================================================= #
  # === use_this_build_directory?
  # ========================================================================= #
  def use_this_build_directory?
    obtain(:use_this_build_directory)
  end; alias build_dir?               use_this_build_directory? # === build_dir?
       alias use_this_build_directory use_this_build_directory? # === use_this_build_directory

  # ========================================================================= #
  # === build_static?
  # ========================================================================= #
  def build_static?
    obtain(:build_static)
  end; alias build_static build_static? # === build_static

  # ========================================================================= #
  # === libraries?
  #
  # Return the given libraries that will be installed by the given program
  # at hand.
  # ========================================================================= #
  def libraries?
    obtain(:libraries)
  end

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @internal_hash[:debug]
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @internal_hash[:commandline_arguments]
  end; alias input? commandline_arguments? # === input?

  # ========================================================================= #
  # === array_invalid_keys?
  #
  # All invalid cookbook-entries will be kept in this Array.
  # ========================================================================= #
  def array_invalid_keys?
    @internal_hash[:array_invalid_keys]
  end; alias array_invalid_keys array_invalid_keys? # === array_invalid_keys

  # ========================================================================= #
  # === pre_make_commands?
  # ========================================================================= #
  def pre_make_commands?
    obtain(:pre_make_commands)
  end; alias pre_make_commands pre_make_commands? # === pre_make_commands

  # ========================================================================= #
  # === md5sum?
  #
  # Conditionally use md5sum.
  #
  # This method must accomodate three different possibilities.
  #
  # (1) We have generated a .yml file in a special directory. We
  #     will preferentially use that one, if it exists. This requires
  #     of the user to have generated these values already. If the
  #     user did so then we will use these values rather than
  #     calculate them anew again. The advantage of this way is that
  #     it will significantly speed up the process.
  #
  # (2) IF we do not have such a file as described in (1), then we will
  #     first try to use the ruby Md5sum class, specifically its Md5sum[]
  #     class method. Note that the ruby Md5sum class is fairly slow for
  #     large programs, which is why option (1) is the default since as
  #     of April 2016.
  #
  # (3) The other two options (1) and (2) not being available, we will
  #     try to use the unix coreutils "md5sum" instead, which is a binary,
  #     and should be pretty fast.
  #
  # Note that as of May 2016, we also make available a way to NOT
  # calculate the md5sum at all. Not everyone will need the md5sum
  # information after all.
  # ========================================================================= #
  def md5sum?(use_this_path = path?)
    # ======================================================================= #
    # === Try to use yaml files first
    #
    # The expanded directory on my system will be here:
    #
    #   /Depot/Temp/CookbookDirectory/expanded_cookbooks/
    #
    # ======================================================================= #
    if File.directory?(::Cookbooks.base_dir_to_store_expanded_cookbooks?) and
       File.exist?(use_this_path.to_s) and
       use_this_path.end_with?('.yml')
       begin
         dataset = YAML.load_file(use_this_path)
         return dataset['md5sum'].to_s
       rescue ArgumentError => error
         opnn; e 'An error happened for the path `'+
                  sfile(use_this_path)+'`. We will exit as a consequence.'
         pp error
         exit
       end
    # ======================================================================= #
    # === Use the pure ruby way next
    # ======================================================================= #
    elsif Object.const_defined? :Md5sum
      Md5sum[use_this_path].md5sum? # Calculate it anew via Md5sum[].
    # ======================================================================= #
    # === Try to use md5sum binary directly next
    # ======================================================================= #
    elsif (result = `md5sum bogus 2>&1`) and
          !result.include?('command not found')
      if File.directory? use_this_path # A workaround for now.
        return '0'
      else
        # =================================================================== #
        # Delegate to the system tool here, part of coreutils.
        # =================================================================== #
        result = `md5sum #{use_this_path}`.split(' ').first
        result.strip! if result
        return result
      end
    else
      '0' # Return 0 then. But that should not ever happen.
    end
  end; alias md5sum md5sum? # === md5sum

  # ========================================================================= #
  # === archive_size?
  #
  # This method keeps track of the size of the program tarball in question.
  #
  # For this to work, we should have initialized the program size at 0.
  # ========================================================================= #
  def archive_size?
    obtain(:archive_size)
  end; alias size          archive_size? # === size
       alias size?         archive_size? # === size?
       alias program_size  archive_size? # === program_size
       alias program_size? archive_size? # === program_size?
       alias archive_size  archive_size? # === archive_size
       alias file_size?    archive_size? # === file_size?
       alias file_size     archive_size? # === file_size?

  # ========================================================================= #
  # === required_deps_on?
  #
  # This method will list the required dependencies of a given program and
  # returns them, at least those that are defined in the
  # 'required_deps_on:' entry of the respective yaml file.
  #
  # Please keep in mind that our dependencies must be kept in Array form.
  # ========================================================================= #
  def required_deps_on?
    obtain(:required_deps_on)
  end; alias program_dependencies_on required_deps_on? # === program_dependencies_on
       alias deps_on?                required_deps_on? # === deps_on?
       alias dependencies_on?        required_deps_on? # === dependencies_on?
       alias return_dependencies     required_deps_on? # === return_dependencies
       alias required_deps?          required_deps_on? # === required_deps?
       alias dependencies?           required_deps_on? # === dependencies?
       alias deps?                   required_deps_on? # === deps?

  # ========================================================================= #
  # === obtain                                            (obtain tag, o tag)
  #
  # Use this getter-method to obtain elements of the main yaml dataset
  # directly, stored within the @internal_hash.
  #
  # The main reason why we will use this method here is because the keys
  # can be either Symbols or Strings. If we ever wish to change to another
  # format, we can do so thanks to the flexibility of this setup.
  # ========================================================================= #
  def obtain(i)
    i = i.to_s unless i.is_a? String
    # ======================================================================= #
    # We can feedback input that was not found - but this can become noisy,
    # so I have disabled this as of Sep 2015.
    # ======================================================================= #
    # begin
    #   return _[i]
    # rescue NoMethodError; end
    # ======================================================================= #
    if @internal_hash[:yaml_dataset]
      @internal_hash[:yaml_dataset][i] # [] is better than .fetch() here.
    end
  end; alias o            obtain # === o (o tag)
       alias get_value_of obtain # === get_value_of

  # ========================================================================= #
  # === program_path?
  #
  # This method will return the 'program_path' value, through the reader
  # method obtain().
  #
  # The program_path variable has the FULL path to the archive in question,
  # for instance:
  #
  #   /Users/x/SRC/gnomemahjonng/gnome-mahjonng-5.08.tar.xz
  #   /Users/x/SRC/gpm/gpm-1.20.1.tar.bz2
  #
  # Do note that the program_path value is the most important entry, as
  # we can retrieve ALL other information from the information stored
  # in this variable alone.
  # ========================================================================= #
  def program_path?
    obtain(:program_path)
  end; alias full_path           program_path? # === full_path
       alias full_path?          program_path? # === full_path?
       alias can_be_found_where? program_path? # === can_be_found_where?
       alias local_url?          program_path? # === local_url?
       alias stored_where?       program_path? # === stored_where?
       alias program_path        program_path? # === program_path
       alias path?               program_path? # === path?
       alias ppath?              program_path? # === ppath?
       alias location?           program_path? # === location?

  # ========================================================================= #
  # === return_program_path
  #
  # This method will return the full program path, consisting of
  # the base_dir and program_full_name component.
  #
  # Example:
  #   '/Users/x/SRC/gnomemahjonng/' + 'gnome-mahjonng-5.08.tar.xz'
  # ========================================================================= #
  def return_program_path
    base_dir?+program_full_name?
  end; alias return_default_program_path return_program_path # === return_default_program_path

  # ========================================================================= #
  # === is_a_game?
  #
  # This method will simply return true if the program in question is
  # a game, and false if it is not a game. That way we can quickly
  # find out whether the input is a game or whether it is not.
  #
  #   ProgramInformation.return_name @program_name
  #   ProgramInformation.return_name 'supertux-0.1.1'
  # ========================================================================= #
  def is_a_game?
    return true if tags?.to_s.include? 'game'
    return false # Else return false here.
  end

  # ========================================================================= #
  # === search_for_this_program?                                   (seek tag)
  #
  # This method will tell us as for which program we will search.
  # ========================================================================= #
  def search_for_this_program?
    @internal_hash[:search_for_this_program]
  end; alias which_program?        search_for_this_program? # === which_program?
       alias seek_which_program?   search_for_this_program? # === seek_which_program?
       alias seek_this_program?    search_for_this_program? # === seek_this_program?
       alias search_which_program? search_for_this_program? # === search_which_program?

  # ========================================================================= #
  # === wrap_result?
  # ========================================================================= #
  def wrap_result?
    @internal_hash[:wrap_result]
  end; alias use_word_wrap? wrap_result? # === use_word_wrap?

  # ========================================================================= #
  # === program_is_included?
  #
  # Query method to determine whether the search term (the program we
  # are seeking) is included or whether it is not.
  #
  # Returns a boolean.
  # ========================================================================= #
  def program_is_included?
    @internal_hash[:is_the_program_included]
  end; alias found?             program_is_included? # == found?
       alias found_result?      program_is_included? # === found_result?
       alias program_was_found? program_is_included? # === program_was_found?

  # ========================================================================= #
  # === set_env_variables?
  #
  # Query the environment variables that this cookbook may set.
  # ========================================================================= #
  def set_env_variables?
    obtain(:set_env_variables)
  end; alias set_env_variables set_env_variables? # === set_env_variables
       alias env_variables?    set_env_variables? # === env_variables?

  # ========================================================================= #
  # === last_update?
  #
  # The last_update tag keeps track over as to when that particular program
  # was last updated.
  # ========================================================================= #
  def last_update?
    obtain(:last_update)
  end; alias last_update last_update? # === last_update

  # ========================================================================= #
  # === main_eight?
  #
  # This method will return the main 8 entries that are the most important
  # ones for any given program at hand.
  # ========================================================================= #
  def main_eight?(
      use_padded_output = true
    )
    _ = ''.dup
    one   = 'real_short_name: '.dup
    two   = 'short_name: '.dup
    three = 'program_version: '.dup
    four  = 'archive_type: '.dup
    five  = 'base_dir: '.dup
    six   = 'program_name: '.dup
    seven = 'program_full_name: '.dup
    eight = 'program_path: '.dup
    # ======================================================================= #
    # Next make sure to pad the output, if the user requested this.
    # ======================================================================= #
    if use_padded_output
      n_ljust = 19
      one   = one.ljust(n_ljust)
      two   = two.ljust(n_ljust)
      three = three.ljust(n_ljust)
      four  = four.ljust(n_ljust)
      five  = five.ljust(n_ljust)
      six   = six.ljust(n_ljust)
      seven = seven.ljust(n_ljust)
      eight = eight.ljust(n_ljust)
    end
    _ << one+real_short_name?.dup+N
    _ << two+short_name?.dup+N
    _ << three+program_version?.dup+N
    _ << four+archive_type?.dup+N
    _ << five+base_dir?.dup+N
    _ << six+program_name?.dup+N
    _ << seven+program_full_name?.dup+N
    _ << eight+program_path?.dup+N
    return _
  end

  # ========================================================================= #
  # === use_this_program_name
  # ========================================================================= #
  def use_this_program_name
    obtain(:use_this_program_name).to_s
  end

  # ========================================================================= #
  # === use_this_program_name_on_gobolinux?
  # ========================================================================= #
  def use_this_program_name_on_gobolinux?
    obtain(:use_this_program_name_on_gobolinux).to_s
  end

  # ========================================================================= #
  # === program_compile_name?
  # ========================================================================= #
  def program_compile_name?
    obtain(:program_compile_name).to_s
  end

  # ========================================================================= #
  # === program_name?
  #
  # Feedback the program_name through this method. The program_name in
  # this context, would mean a String such as gnome-mahjonng-5.08'.
  # ========================================================================= #
  def program_name?
    obtain(:program_name)
  end; alias name_and_version?                 program_name? # === name_and_version?
       alias pname?                            program_name? # === pname?
       alias program_name                      program_name? # === program_name
       alias program_name_and_program_version  program_name? # === program_name_and_program_version
       alias program_name_and_program_version? program_name? # === program_name_and_program_version?

  # ========================================================================= #
  # === url?
  #
  # This method will "hold" the "most" important URL of a program, which
  # is the entry called url1. Every entry of our cookbooks must have
  # this entry available.
  # ========================================================================= #
  def url?
    this_url = obtain(:url1).to_s
    return this_url
  end; alias main_url    url? # === main_url
       alias remote_url  url? # === remote_url
       alias main_url?   url? # === main_url?
       alias url1?       url? # === url1?
       alias url1        url? # === url1
       alias remote_url? url? # === remote_url?

  # ========================================================================= #
  # === urls?
  #
  # This method will return all available URLs in Array format.
  #
  # We only support 5 URLs at max right now. Since Jul 2012 we will
  # not return empty values, as they are useless anyway.
  # ========================================================================= #
  def urls?
    # ======================================================================= #
    # Build our Array that will hold all 5 URLs.
    # ======================================================================= #
    array = %w( url1 url2 url3 url4 url5 )
    _ = dataset?.values_at(*array) # We require the * here.
    _.compact!
    _.reject!(&:empty?) # We don't want empty entries, hence we filter them away.
    return _
  end; alias remote_urls  urls? # === remote_urls
       alias remote_urls? urls? # === remote_urls?

  # ========================================================================= #
  # === return_default_program_full_name
  #
  # program_full_name is something like 'gnome-mahjonng-5.08.tar.xz'.
  # ========================================================================= #
  def return_default_program_full_name
    program_name?+archive_type?
  end; alias return_program_full_name return_default_program_full_name # === return_program_full_name

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    obtain(:prefix)
  end; alias prefix prefix? # === prefix

  # ========================================================================= #
  # === configure_options?
  #
  # Convenience method to query the configuration options available.
  # ========================================================================= #
  def configure_options?
    obtain(:configure_options)
  end; alias configuration_options  configure_options? # === configuration_options
       alias configuration_options? configure_options? # === configuration_options?
       alias configure_options      configure_options? # === configure_options

  # ========================================================================= #
  # === archive_type?
  #
  # Simply query the archive type here. The archive_type is something
  # such as ".tar.gz" or ".tar.bz2".
  # ========================================================================= #
  def archive_type?
    obtain(:archive_type)
  end; alias archive?      archive_type? # === archive?
       alias archive_type  archive_type? # === archive_type
       alias archive_name? archive_type? # === archive_name?

  # ========================================================================= #
  # === use_this_connective_token?
  # ========================================================================= #
  def use_this_connective_token?
    @internal_hash[:use_this_connective_token]
  end; alias connective_token? use_this_connective_token? # === connective_token?

  # ========================================================================= #
  # === extra_information?
  #
  # We must not get rid of information from the original dataset.
  # ========================================================================= #
  def extra_information?
    result = obtain(:extra_information)
    if result.encoding.to_s.include? 'UTF-8'
      result.force_encoding(ENCODING_ISO)
    end
    if result
      if use_word_wrap?
        result = word_wrap(result)
      end if result # Do not work on nil-values here.
    end
    return result
  end; alias extra_information extra_information? # === extra_information

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    obtain(:binaries)
  end; alias binaries binaries? # === binaries
       alias bin?     binaries? # === bin?

  # ========================================================================= #
  # === licence?
  # ========================================================================= #
  def licence?
    obtain(:licence)
  end; alias license? licence? # === license?

  # ========================================================================= #
  # === short_description?
  #
  # This method will obtain the short description of a given program.
  #
  # Not every program will have a short description, but it is recommended
  # that all programs SHOULD have a short description.
  # ========================================================================= #
  def short_description?
    obtain(:short_description)
  end; alias short_desc? short_description? # === short_desc?

  # ========================================================================= #
  # === description?                                        (description tag)
  #
  # The description value of a program will be returned via this method.
  # ========================================================================= #
  def description?(wrap_result = wrap_result?)
    result = obtain(:description)
    case wrap_result
    when :retain_newlines
      wrap_result = false
    end
    if wrap_result
      if use_word_wrap?
        result = word_wrap(result)
      end if result # Do not work on nil-values here.
    end
    return result
  end; alias desc?           description? # === desc?
       alias description_of? description? # === description_of?

  # ========================================================================= #
  # === raw_description?
  # ========================================================================= #
  def raw_description?
    obtain(:raw_description)
  end; alias raw_description raw_description? # === raw_description

  # ========================================================================= #
  # === return_default_program_path
  #
  # This is entry number (8), program_path, which will look like:
  #   '/Users/x/SRC/gnomemahjongg/gnome-mahjonng-5.08.tar.xz' # base_dir?+program_full_name?
  # ========================================================================= #
  def return_default_program_path(
      token_to_use = use_this_connective_token?
    )
    base_dir?+short_name?+token_to_use+program_version?+archive_type?
  end

  # ========================================================================= #
  # === internal_hash?
  # ========================================================================= #
  def internal_hash?
    @internal_hash
  end; alias ihash?      internal_hash? # === ihash?
       alias ihash       internal_hash? # === ihash

  # ========================================================================= #
  # === yaml_dataset_exists?
  # ========================================================================= #
  def yaml_dataset_exists?
    ihash[:yaml_dataset]
  end

  # ========================================================================= #
  # === yaml_dataset?
  #
  # Query method over the main dataset.
  # ========================================================================= #
  def yaml_dataset?
    @internal_hash[:yaml_dataset]
  end; alias hash?          yaml_dataset? # === hash?
       alias return_dataset yaml_dataset? # === return_dataset
       alias dataset?       yaml_dataset? # === dataset?
       alias return_hash    yaml_dataset? # === return_hash

  # ========================================================================= #
  # === symlink_glib_schema?
  # ========================================================================= #
  def symlink_glib_schema?
    obtain(:symlink_glib_schema)
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    obtain(:homepage)
  end; alias return_homepage homepage? # === return_homepage
       alias homepage        homepage? # === homepage

  # ========================================================================= #
  # === skip_configure?
  #
  # Query-method over as to whether we will skip configure or whether
  # we will not.
  # ========================================================================= #
  def skip_configure?
    obtain(:skip_configure)
  end; alias skip_configure skip_configure? # === skip_configure

  # ========================================================================= #
  # === has_to_be_compiled?
  # ========================================================================= #
  def has_to_be_compiled?
    obtain(:has_to_be_compiled)
  end; alias has_to_be_compiled has_to_be_compiled? # === has_to_be_compiled

  # ========================================================================= #
  # === make_options?
  # ========================================================================= #
  def make_options?
    obtain(:make_options)
  end; alias make_options make_options? # === make_options

  # ========================================================================= #
  # === parameters_to_make?
  # ========================================================================= #
  def parameters_to_make?
    obtain(:parameters_to_make)
  end

end; end