#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
require 'cookbooks/check_for_updates/remote_urls.rb'

module Cookbooks

class Cookbook < Base

  begin # On my system only so far.
    require 'x/tools/human_readable.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === @honour_threshold_value
  # ========================================================================= #
  @honour_threshold_value = true

  # ========================================================================= #
  # === Cookbooks::SanitizeCookbookDataset.do_not_honour_threshold_value
  # ========================================================================= #
  def self.do_not_honour_threshold_value
    @honour_threshold_value = false
  end

  # ========================================================================= #
  # === Cookbooks::Cookbook.honour_threshold_value?
  # ========================================================================= #
  def self.honour_threshold_value?
    @honour_threshold_value
  end

  # ========================================================================= #
  # === report_size_of_these_programs
  #
  # This method will report the size of the issued programs.
  # ========================================================================= #
  def report_size_of_these_programs(i, size_of_the_programs = 0)
    if i.is_a? String and i.include?('=')
      i = i.to_s.dup if i.frozen?
      i.gsub!(/--size=/,'')
      i = i.split(',') if i.include? ','
    end
    i = [i] unless i.is_a? Array
    i.each {|entry|
      dataset = self.new(entry)
      unless use_colours?
        dataset.disable_colours
      end
      size = dataset.size?.to_i
      opnn; e 'Now working on '+sfancy(entry.rjust(8))+
              ' (Size: '+size.to_s.rjust(8)+' bytes)'
      size_of_the_programs += size
    }
    e
    opnn; e 'The size of the programs '+sfancy(i.join(','))+' is:'
    e
    if Object.const_defined? :HumanReadable
      size_of_the_programs = HumanReadable[size_of_the_programs]
    end
    result = "  #{simp(size_of_the_programs.to_s)}".dup
    result << ' bytes' unless size_of_the_programs.to_s.strip.end_with? 'B'
    opnn; e result
    e
  end

  # ========================================================================= #
  # === consider_reporting_how_many_programs_are_registered
  # ========================================================================= #
  def consider_reporting_how_many_programs_are_registered(
      shall_we_report = false
    )
    if shall_we_report
      Cookbooks.report_the_registered_programs
    end
  end

  # ========================================================================= #
  # === report
  #
  # This is just a general-purpose method that will output some values.
  #
  # It will output useful information about a program.
  #
  # For a full report system, use the specialized class
  # DisplayCookbookDataset.
  # ========================================================================= #
  def report(
      program_name = short_name?
    )
    return if program_name.to_s.empty?
    @array_show_these_entries.clear
    @array_show_these_entries << :description
    # ======================================================================= #
    # Add a bunch of symbols to the main array here.
    # ======================================================================= #
    @array_show_these_entries << %i(
      program_path
      program_full_name
      program_name
      real_short_name
      short_name
      program_compile_name
      program_version
      archive_type
      base_dir
      remote_url
      extract_to
      prefix
      configure_options
      md5sum
      file_size
      keep_extracted
    )
    @array_show_these_entries.flatten!
    # ======================================================================= #
    # === short_description
    # ======================================================================= #
    unless short_description?.empty?
      @array_show_these_entries << :short_description
    end if short_description?
    # ======================================================================= #
    # Unless the pkgconfig_files Array is empty, we will display the
    # .pc files of the given program.
    # ======================================================================= #
    if pkgconfig_files?
      unless pkgconfig_files?.empty?
        @array_show_these_entries << :pkgconfig_files
      end
    end
    unless extra_information?.empty?
      @array_show_these_entries << :extra_information
    end
    unless configure_command?.empty?
      unless configure_command? == 'configure'
        @array_show_these_entries << :configure_command
      end
    end
    # ======================================================================= #
    # === Show the main homepage if it exists
    # ======================================================================= #
    if homepage?
      @array_show_these_entries << :homepage
    end
    # ======================================================================= #
    # === Add the headers, if they exist
    # ======================================================================= #
    unless headers?.empty?
      @array_show_these_entries << :headers
    end
    if build_static? == true
      @array_show_these_entries << :build_static
    end
    unless use_this_make_command?.to_s == 'make'
      @array_show_these_entries << :use_this_make_command
    end
    if (sed? == true) and !sed?.to_s.empty?
      @array_show_these_entries << :sed
    end
    @array_show_these_entries << :do_not_download     if do_not_download?
    @array_show_these_entries << :required_deps_on    unless deps?.empty?
    @array_show_these_entries << :apply_patch         if apply_patch?
    @array_show_these_entries << :libraries           unless libraries?.empty?
    @array_show_these_entries << :make_options        unless make_options?.empty?
    @array_show_these_entries << :enable_shared       if enable_shared?
    @array_show_these_entries << :binaries            unless binaries?.empty?
    @array_show_these_entries << :preinstall          unless preinstall?.empty?
    @array_show_these_entries << :postinstall         unless postinstall?.empty?
    @array_show_these_entries << :tags                unless tags?.empty?
    @array_show_these_entries << :svn_url             if svn_url? and !svn_url?.empty?
    @array_show_these_entries << :pre_make_commands   unless pre_make_commands?.empty?
    @array_show_these_entries << :use_build_directory if use_build_directory?
    @array_show_these_entries << :licence # <- Mandatory entry.
    # ======================================================================= #
    # === Use autogen will always be shown.
    # ======================================================================= #
    @array_show_these_entries << :use_autogen
    @array_show_these_entries << :configure_base_dir  if configure_base_dir? and !configure_base_dir?.to_s.empty?
    @array_show_these_entries << :blfs                unless blfs?.empty?
    @array_show_these_entries << :last_update
    @array_show_these_entries << :has_to_be_compiled
    @array_show_these_entries << :do_not_symlink      if do_not_symlink?
    @array_show_these_entries << :md5sum              if show_md5sum?
    @array_show_these_entries << :use_this_make_install_command if use_this_make_install_command?
    @array_show_these_entries << :skip_configure      if skip_configure?
    # ======================================================================= #
    # Next, display it with a coloured "border".
    # ======================================================================= #
    use_this_colour_or_no_colour = :lightblue
    unless use_colours?
      use_this_colour_or_no_colour = :do_not_use_colours
    end
    cliner(use_this_colour_or_no_colour) {
      e 'Feedbacking all about the program `'+simp(program_name)+'`.'
      e
      show_this_array(@array_show_these_entries)
      # ===================================================================== #
      # Show important URLs next.
      # ===================================================================== #
      if urls?.first
        e; e '  Important URLs:'
        _ = urls?.map {|entry| entry.to_s.chomp }.reject {|x| x.empty?}
        if _
          e; _.each_with_index {|line, index|
            e "#{grey('    (')}#{seagreen(index+1).to_s}#{grey(') ')}#{grey(line)}"
          }; e
        end
      end
    }
    e
  end; alias feedback                   report # === feedback
       alias feedback_colourized_result report # === report (report tag)
       alias print_hash                 report # === print_hash
       alias display_all                report # === display_all
       alias full_report                report # === full_report

  # ========================================================================= #
  # === show_this_array
  # ========================================================================= #
  def show_this_array(array)
    # ======================================================================= #
    # Determine the threshold value to use for this display.
    # ======================================================================= #
    threshold_value = 70
    left_pad = '  '
    ljust_value = 28
    array.each {|entry|
      case entry
      # ===================================================================== #
      # === has_to_be_compiled
      # ===================================================================== #
      when :has_to_be_compiled
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          steelblue(
            yes_or_no(send_this_method_and_sanitize_arrays(entry.to_s+'?')).to_s
          )
      # ===================================================================== #
      # === skip_configure
      # ===================================================================== #
      when :skip_configure
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          mediumorchid(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === archive_size
      # ===================================================================== #
      when :archive_size, :file_size, :program_size # colourize program_size.
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          mediumorchid(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === homepage
      # ===================================================================== #
      when :homepage
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          cadetblue(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === short_description
      # ===================================================================== #
      when :short_description
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          mediumturquoise(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === program_name
      # ===================================================================== #
      when :program_name
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          mediumslateblue(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === program_compile_name
      # ===================================================================== #
      when :program_compile_name
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          mediumslateblue(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === description
      #
      # Handle the description entry here.
      # ===================================================================== #
      when :description
        result = description?.chomp.rstrip
        if (result.size > threshold_value)
          result = result.tr("\n",' ')[0..threshold_value]+' [...]'
        end
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          limegreen(result)+rev
      # ===================================================================== #
      # === extra_information
      # ===================================================================== #
      when :extra_information
        # =================================================================== #
        # === Handle extra_information
        #
        # Next, limit the size for extra_description. We do not want to show
        # too much here.
        # =================================================================== #
        result = send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s
        if self.class.honour_threshold_value?
          if (result.size > threshold_value)
            result = result[0, result.index("\n")].delete('"')+' [...]'
          end
        end
        result.chomp!
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          darkslateblue(result)
      # ===================================================================== #
      # === use_autogen
      # ===================================================================== #
      when :use_autogen
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          cadetblue(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === extract_to
      # ===================================================================== #
      when :extract_to
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          mediumpurple(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)+rev
      # ===================================================================== #
      # === program_path
      # ===================================================================== #
      when :program_path
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          orange(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === configure_options
      # ===================================================================== #
      when :configure_options
        unless configure_options?.empty?
          e (left_pad+entry.to_s+': ').ljust(ljust_value)+
            mediumseagreen(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)+rev
        end
      # ===================================================================== #
      # === do_not_symlink
      # ===================================================================== #
      when :do_not_symlink
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          lightslategray(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === keep_extracted
      # ===================================================================== #
      when :keep_extracted
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
         seagreen(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === last_update
      # ===================================================================== #
      when :last_update
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          cornflowerblue(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === prefix
      # ===================================================================== #
      when :prefix
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          darkseagreen(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === program_version
      # ===================================================================== #
      when :program_version
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          springgreen(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === url1
      # ===================================================================== #
      when :url1,:url2,:url3,:url4,:url5
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sandybrown(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === pkgconfig_files
      # ===================================================================== #
      when :pkgconfig_files
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          deepskyblue(send_this_method_and_sanitize_arrays(entry.to_s+'?').to_s)
      # ===================================================================== #
      # === headers
      # ===================================================================== #
      when :headers
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          darkseagreen(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === md5sum
      # ===================================================================== #
      when :md5sum
        if show_md5sum?
          e (left_pad+entry.to_s+': ').ljust(ljust_value)+
            sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
        end
      # ===================================================================== #
      # === remote_url
      # ===================================================================== #
      when :remote_url
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === program_full_name
      # ===================================================================== #
      when :program_full_name
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === base_dir
      # ===================================================================== #
      when :base_dir
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === real_short_name
      # ===================================================================== #
      when :real_short_name
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === archive_type
      # ===================================================================== #
      when :archive_type
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      # ===================================================================== #
      # === short_name
      #
      # Is steelblue.
      # ===================================================================== #
      when :short_name
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          steelblue(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      else
        e (left_pad+entry.to_s+': ').ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays(entry.to_s+'?'))
      end
    }
  end

  # ========================================================================= #
  # === send_this_method_and_sanitize_arrays
  # ========================================================================= #
  def send_this_method_and_sanitize_arrays(i)
    result = send(i)
    if result.is_a? Array
      result = '['+result.join(',')+']' unless result.compact.empty?
    end
    return result
  end

  # ========================================================================= #
  # === report_total_size_of_locally_available_files
  #
  # This method will simply calculate the total file size of all
  # locally available programs.
  # ========================================================================= #
  def report_total_size_of_locally_available_files
    total_file_size = 0
    _ = available_programs?
    _.each {|name_of_the_program|
      size = ::Cookbooks::Cookbook.new(name_of_the_program) { :bypass_menu }.file_size?
      opnn; e 'Now adding the file size of the '\
              'program `'+sfancy(name_of_the_program)+'` ('+
              orange(size.to_s)+' bytes).'
      total_file_size += size
    }
    opnn; e "Total file size of #{sfancy(_.size.to_s)} programs: "\
            "#{slateblue(total_file_size.to_s+' bytes')}"
  end

  # ========================================================================= #
  # === yes_or_no
  # ========================================================================= #
  def yes_or_no(i)
    case i
    when true
      'Yes.'
    when false
      'No.'
    end
  end

  # ========================================================================= #
  # === report_the_kde_status
  #
  # This method will report the "status" of the KDE project, as far as the
  # cookbooks project is concerned.
  #
  # What is meant here is that this method will simply report the main
  # constants that handle the remote URL to kde projects.
  #
  # To invoke this method, do this on the commandline:
  #   cookbooks --report-the-kde-status
  # ========================================================================= #
  def report_the_kde_status
    kde_frameworks_version = File.basename(
      REMOTE_URL_FOR_KDE_FRAMEWORKS
    )
    kde_applications_version = File.basename(
      REMOTE_URL_FOR_KDE_APPLICATIONS.sub(/src\/$/,'')
    )
    kde_plasma_version = File.basename(
      REMOTE_URL_FOR_KDE_PLASMA
    ).sub(/\.php$/,'').sub(/^plasma-/,'')
    e
    e 'KDE Frameworks version:   '+cyan(kde_frameworks_version)
    e 'KDE Applications version: '+cyan(kde_applications_version)
    e 'KDE Plasma version:       '+cyan(kde_plasma_version)
    e
  end

end; end