#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# Entries are:
#
#   (1) real_short_name   like: 'gnomemahjonng'  # This is like short_name but without any possible '-' and also without any '_'
#   (2) short_name        like: 'gnome-mahjonng' # This may invoke (1) and may include '-' characters.   
#   (3) program_version   like: '5.08'
#   (4) archive_type      like: '.tar.bz2' # Yes, must have a leading '.'
#   (5) base_dir          like: '/Users/x/SRC/gnomemahjongg/'
#   (6) program_name      like: 'gnome-mahjonng-5.08' # This one includes the version. It is short_name+'-'+program_version
#   (7) program_full_name like: 'gnome-mahjonng-5.08.tar.xz', so this is program_name?+archive_type?
#   (8) program_path      like: '/Users/x/SRC/gnomemahjongg/gnome-mahjonng-5.08.tar.xz' # base_dir?+program_full_name?
#
# =========================================================================== #
require 'cookbooks/constants/regexes.rb'
require 'cookbooks/constants/constants.rb'

begin
  require 'convert_global_env'
rescue LoadError; end

module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === fill_in_missing_entries
  #
  # This will fill in the missing entries. They will default to empty
  # String.
  # ========================================================================= #
  def fill_in_missing_entries
    registered_cookbook_entries?.each {|key|
      set(key, '') unless has_key? key
    }
  end

  # ========================================================================= #
  # === sanitize_postinstall
  #
  # This will sanitize the postinstall stuff, by assigning to the
  # respective entry in the main Hash.
  #
  # Note that we have to invoke this method only after real_short_name
  # and program_version was set, because one entry requires this.
  #
  # As of 20.09.2014, postinstall must be an Array.
  # ========================================================================= #
  def sanitize_postinstall
    _ = postinstall?
    if _.nil? # Safeguard to ensure Array.
      set_postinstall []
      _ = postinstall?
    end
    _.compact! if _.is_a? Array # Get rid of nil values.
    unless _.empty?
      if _.is_a? Array
        _.map! {|entry| substitute_macros(entry) }
      end
      _.gsub!(/#{N}/,',') if _.include?(N) # Replace newlines with ',' char.
      # ======================================================================= #
      # Replace universal targets with real targets.
      # ======================================================================= #
      case prefix?
      when false # Yup. Only one option for false..
        point_to_libraries   = SYSLIB_DIR
        point_to_binaries    = SYSBIN_DIR
        point_to_includes    = SYSINCLUDE_DIR
        point_to_program_dir = rds( # Remove double slashes.
          programs_dir?+
          real_short_name?.to_s.capitalize+'/'+
          program_version?.to_s+'/bin'
        )
      when true
        # =================================================================== #
        # FHS installation method comes here. It will defaults to /usr/bin.
        # It also includes true of course.
        # =================================================================== #
        point_to_libraries   = '/usr/lib'
        point_to_binaries    = '/usr/bin'
        point_to_includes    = '/usr/include'
        point_to_program_dir = point_to_binaries
      else
        point_to_libraries   = "{prefix?}lib"
        point_to_binaries    = "{prefix?}bin" # For example, /Programs/Glib/2.40.0/bin/
        point_to_includes    = "{prefix?}include"
        point_to_program_dir = point_to_binaries
      end
      if _.is_a? String
        _ = [_]
      end
      # ======================================================================= #
      # Sanitize the @postinstall variable next.
      # ======================================================================= #
      if _.is_a? Array
        _.map! {|entry|
          if entry.include? 'UNIVERSAL_BINARY_DIR'
            entry.gsub!(/UNIVERSAL_BINARY_DIR/,  point_to_binaries)
          end
          entry.gsub!(/UNIVERSAL_BINARIES_DIR/,  point_to_binaries)
          entry.gsub!(/UBD/,                     point_to_binaries)
          entry.gsub!(/UNIVERSAL_LIBRARIES_DIR/, point_to_libraries)
          entry.gsub!(/ULD/,                     point_to_libraries)
          entry.gsub!(/UNIVERSAL_INCLUDE_DIR/,   point_to_includes)
          entry.gsub!(/PROGRAM_DIR/,             point_to_program_dir)
          entry.gsub!(/PROGRAMS_DIR/,            point_to_program_dir)
          entry.gsub!(/,cp/,                     ',copy_file')
          entry.strip! # Strip it.
          entry = rds(entry) # Replace // with // here.
          entry
        }
        set_postinstall _ # Reassign.
      end
    end
    unless postinstall?.is_a? Array
      _ = rds(postinstall?)
      _ = _.split(',') if _.include? ','
      set_postinstall [_].flatten
    end
  end

  # ========================================================================= #
  # === sanitize_configure_base_dir
  #
  # We must ensure that this variable has a trailing / character.
  # ========================================================================= #
  def sanitize_configure_base_dir
    if configure_base_dir?
      unless configure_base_dir?.empty?
        set_configure_base_dir rds(configure_base_dir?+'/')
      end
    end
  end

  # ========================================================================= #
  # === sanitize_pre_make_commands
  #
  # We must keep the pre_make_commands in Array format. The method
  # here will make sure of this.
  #
  # We can only invoke this after having set program_version before,
  # because some macro-substitutions require the program version to
  # be properly set before the make-commands can be sanitized.
  # ========================================================================= #
  def sanitize_pre_make_commands
    _ = pre_make_commands?
    _.compact! if _.is_a? Array # Get rid of nil values.
    array = []
    if _
      unless _.empty?
        _ = substitute_macros(_)
        array << _
      end
    end
    array.flatten!
    array.compact!
    set_pre_make_commands(array) # Turn it into an Array here.
  end

  # ========================================================================= #
  # === sanitize_blfs_entry
  #
  # This method will clean up the BLFS entry, should it exist.
  #
  # It must be guaranteed to be an Array.
  # ========================================================================= #
  def sanitize_blfs_entry
    if has_key? :blfs
      _ = blfs?
      unless _.is_a? Array
        if _.nil?
          _ = []
        elsif _.empty?
          _ = [] # Empty Array then.
        else
          _ = [_]
        end
        set_blfs _
      end
    else
      set_blfs []
    end
  end

  # ========================================================================= #
  # === sanitize_preinstall
  # ========================================================================= #
  def sanitize_preinstall
    _ = preinstall?
    if _.is_a?(String) and (_.count(N) > 1)
      _ = _.split(N)
    end
    if _.nil? # Safeguard to ensure Array.
      set_preinstall []
    else
      _ = [_] if _.is_a? String
      if _.is_a? Array
        _.compact!
        _.map!(&:strip)
        _.reject! {|entry| entry.empty? } # Weed out empty entries.
        _.flatten!
      end
      set_preinstall _
    end
  end

  # ========================================================================= #
  # === sanitize_binaries                                    (binaries tag)
  #
  # This method will sanitize the binaries entry, @_[:binaries].
  # We must keep the binaries in Array format.
  # ========================================================================= #
  def sanitize_binaries
    _ = binaries? # Obtain a pointer.
    _ = [] if _.nil?
    if _.is_a?(String) and !_.empty?
      _ = _.split(',')
    elsif _.is_a? String
      _ = [_]
    end
    if _.is_a? Array
      if _.first.include? ',' # Assume that we must split then.
        _.map! {|entry| entry.split(',') }
      end if _.first
    end
    _ = _.flatten.compact.map(&:strip)
    set_binaries(_)
  end

  # ========================================================================= #
  # === sanitize_libraries
  #
  # This sanitizes the libraries entry, @_[:libraries].
  #
  # We must keep this entry in Array format.
  # ========================================================================= #
  def sanitize_libraries
    _ = libraries?
    unless _.is_a? Array
      _ = [_].compact
    end
    # ======================================================================= #
    # Next, we will assume something such as:
    #   ["libdevkit-power-gobject.{so,a},libupower-glib.{so,a}"] 
    # ======================================================================= #
    unless _.empty? # safeguard against nil entries.
      if _.first.include?('}') and _.first.include?('{')
        splitted = _.first.split(/\},/)
        splitted.map! {|entry|
          entry =~ REGEX_TO_SANITIZE_LIBRARIES
          [$1+'.'+$2, $1+'.'+$3]
        }
        splitted.flatten!
        _ = splitted
      end
    end
    if _.is_a? Array
      if _.first.include? ','
        _ = _.first.split(',')
        _.map!(&:strip) # It should not include empty whitespace.
        # =================================================================== #
        # Some libraries may have components such as foo{so.a}
        # We need to split these up into foo.so and foo.a.
        # =================================================================== #
      end if _.first
    end
    new_value = [] # default.
    new_value = [_].flatten unless _.empty?
    new_value.compact! if new_value.is_a? Array
    set_libraries(new_value)
  end

  # ========================================================================= #
  # === try_to_sanitize_sed_entry                                   (sed tag)
  #
  # sed entry allows us to make some very simple replacements in files.
  #
  # We will guarantee that it will be an Array.
  # ========================================================================= #
  def try_to_sanitize_sed_entry
    # ======================================================================= #
    # Only sanitize the sed entry if we have that key.
    # ======================================================================= #
    if has_key? :sed
      if sed?
        _ = sed?
        _ = _.split(N) if _.is_a? String
        if _
          _ = [_] unless _.is_a? Array # Will be an Array.
          _.compact! # We don't want to store nil values here.
          set_sed(_)
        end
      end
    end
  end

  # ========================================================================= #
  # === cleanup_unusued_variables
  # ========================================================================= #
  def cleanup_unusued_variables
    # ======================================================================= #
    # Must safeguard to query whether the key still exists. When it does,
    # we will simply delete them.
    # ======================================================================= #
    if ihash?.has_key? :array_invalid_keys
      ihash?.delete :array_invalid_keys
    end
  end

  # ========================================================================= #
  # === sanitize_apply_patch
  #
  # Here we sanitize the value for apply_patch.
  # ========================================================================= #
  def sanitize_apply_patch
    _ = apply_patch?
    if _
      if _.include? '#'
        _ = _[0 .. (_.index('#') - 1)]
        set_apply_patch _
      end
      if _.include? 'if PROGRAM_VERSION' # Ok, we specified a specific version.
        version_to_compare = _.split(' ').last
        if version_to_compare == program_version? # Ok, patch can be applied.
        else # Else do not apply this patch.
          set_apply_patch ''
        end
      end
    end
  end

  # ========================================================================= #
  # === sanitize_env_variables
  # ========================================================================= #
  def sanitize_env_variables
    _ = env_variables?
    if _ and _.is_a? Array # Must also check for nil.
      # ===================================================================== #
      # If it is an Array, then it must be wrong, so correct this here
      # silently. We need a Hash, as specified in the specification.
      # ===================================================================== #
      hash = {}
      _.each {|inner_hash|
        if inner_hash.is_a? Hash
          key   = inner_hash.keys.first
          value = inner_hash.values.first
          hash[key] = value
        end
      }
      set_env_variables(hash)
    end
  end

  # ========================================================================= #
  # === sanitize_program_compile_name
  # ========================================================================= #
  def sanitize_program_compile_name
    _ = program_compile_name?
    if are_we_on_gobolinux? and
      !use_this_program_name_on_gobolinux?.to_s.empty?
      _ = use_this_program_name_on_gobolinux?
    end
    set_program_compile_name(_.to_s) # Will always be at the least an empty String.
  end

  # ========================================================================= #
  # === sanitize_extra_information
  # ========================================================================= #
  def sanitize_extra_information
    _ = extra_information?.chomp.rstrip
    set_extra_information(_)
  end

  # ========================================================================= #
  # === sanitize_base_dir
  #
  # This method will currently only downcase the last part, so that
  # "Eterm/" becomes "eterm/" instead.
  # ========================================================================= #
  def sanitize_base_dir
    _ = base_dir?
    if _.include? '/'
      splitted = _.split('/')
      last_part = splitted.last
      unless last_part == last_part.downcase
        splitted[-1] = last_part.downcase
        _ = splitted.join('/')
        set_base_dir(_)
      end
    end
  end

  # ========================================================================= #
  # === sanitize_configure_command
  # ========================================================================= #
  def sanitize_configure_command
    _ = configure_command?
    if _
      if _.empty?
        set_configure_command 'configure' # set default, reset every time.
      else
        _ = ConvertGlobalEnv[_] if _.include? '$'
        set_configure_command(_)
      end
    else
      if _.include? '$'
        _ = ConvertGlobalEnv[_]
      end
      set_configure_command(_) # set rescue.
    end
  end

  # ========================================================================= #
  # === return_url_replace_underscore
  # ========================================================================= #
  def return_url_replace_underscore(i = url?)
    _ = File.basename(i).dup
    if _.include? '_'
      _.tr!('_','-') # Replace all '_' with '-'.
    elsif _.include? '.' # Else replace just the first '.'.
      _.sub!('.','-')
    end
    _ = remove_file_extension(_)
    return _
  end

  # ========================================================================= #
  # === return_first_dot_as_hyphen_from_this_url
  # ========================================================================= #
  def return_first_dot_as_hyphen_from_this_url(i = url?)
    new_url = File.basename(i).dup # Obtain the URL here.
    new_url.sub!(/\./,'-') # This replaces only the first '.' found.
    return new_url
  end

  # ========================================================================= #
  # === return_first_underscore_as_hyphen_from_this_url
  # ========================================================================= #
  def return_first_underscore_as_hyphen_from_this_url(i = url?)
    new_url = File.basename(i).dup # Obtain the URL here.
    new_url.sub!(/_/,'-') # This replaces only the first '_' found.
    return new_url
  end

  # ========================================================================= #
  # === return_first_underscore_as_hyphen_and_other_undscores_as_dots
  #
  # This method exists specifically for input such as 'boost_1_61_0'.
  # ========================================================================= #
  def return_first_underscore_as_hyphen_and_other_undscores_as_dots(i = url?)
    i = return_first_underscore_as_hyphen_from_this_url(i).dup
    i.tr!('_','.')
    return i
  end

  # ========================================================================= #
  # === sanitize_url
  #
  # We will chop off some parts of the URL.
  # ========================================================================= #
  def sanitize_url(i = url1?)
    if i.end_with? '/download'
      i.gsub!(/\/download$/,'')
      set_url1(i)
    end
  end

  # ========================================================================= #
  # === sanitize_configure_options                          (configure tag)
  #
  # Work on the configure options, i.e. @dataset['configure_options'].
  #
  # Note that this depends on program_version? having been set properly.
  # ========================================================================= #
  def sanitize_configure_options
    _ = configure_options?.to_s.strip # Don't want empty thingies here.
    # ======================================================================= #
    # Get rid of multi-blanks next.
    # ======================================================================= #
    if _
      _.squeeze!('/')
      _.gsub!(/#{N}/,' ')
    end
    # ======================================================================= #
    # Treat 'VERSION' as a macro, for expansion.
    # ======================================================================= #
    if _.include? 'PROGRAM_VERSION'
      _.gsub!(/PROGRAM_VERSION/, program_version?.to_s) if _
    elsif _.include? 'VERSION'
      _.gsub!(/VERSION/, program_version?.to_s) if _
    end
    # ======================================================================= #
    # Treat 'CURRENT' as a macro, for expansion.
    # ======================================================================= #
    if _.include? 'CURRENT'
      _.gsub!(/CURRENT/, program_version?.to_s) if _
    end
    # ======================================================================= #
    # === NAME_OF_THE_DISTRIBUTION
    # ======================================================================= #
    if _.include? 'NAME_OF_THE_DISTRIBUTION'
      name = YAML.load_file(
        project_yaml_directory?+'configuration/name_of_the_distribution.yml'
      )
      _.gsub!(/NAME_OF_THE_DISTRIBUTION/, name)
    end
    unless _.empty?
      # ======================================================================= #
      # Finally, run through the configure options. If they start with
      # enable or disable, with or without, we prepend -- if this is not
      # yet the case. Since Sep 2011, we will always prepend --, unless
      # the first character already is a '-' (this is done to 
      # ======================================================================= #
      _ = _.split(' ').map {|configure_option|
        # If configure_option =~ /^disable|enable|with|without/
        # =================================================================== #
        # Only add when not yet first char is NOT a '-'.
        # =================================================================== #
        unless configure_option.start_with? '-' or
               configure_option.start_with? '$'
          configure_option = '--'+configure_option
        end
        # end
        configure_option
      }.join(' ')
    end
    _ = _.tr(N, ' ') # Replace all newlines again.
    set_configure_options(_) # Last but not least, reassign.
  end

  # ========================================================================= #
  # === sanitize_prefix                                 (sanitize prefix tag)
  #
  # The value for the prefix of the program at hand will be sanitized
  # here, in this method.
  #
  # Since as of May 2013, we will never use a shortcut anymore, we will
  # *always* store the full, absolute path to our entry.
  #
  # "false" as value, is  an indication of using the path to an AppDir.
  # For example, this  would be something such as /Programs/Htop/2.0.0 -
  # that would be a proper appdir.
  #
  # The --prefix is always a directory, so as of Jan 2014 we will
  # append a / to this unless the last character is already a /.
  # ========================================================================= #
  def sanitize_prefix
    _ = prefix?.dup
    # ======================================================================= #
    # General way to overrule the prefix setting in this case.
    # ======================================================================= #
    _ = false if COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY
    if _.is_a?(FalseClass) or _.is_a?(TrueClass)
      _ = _.to_s
    end
    _ = _.to_s.dup
    if _.is_a? String
      case _
      when 't','true','yes','y','traditional'
        _ = '/usr/'
      when 'f','false','no','AppDir','return_appdir_path','appdir_path'
        _ = return_appdir_prefix.dup # <- This is the AppDir variant.
      end
      if _.is_a? String
        _ = _.dup if _.frozen? # Unfreeze it.
        # =================================================================== #
        # As of January 2014, we will pass the String through the macro-check,
        # which has to be done because our string may include PROGRAM_NAME as
        # a prefix.
        # =================================================================== #
        _ = substitute_macros(_).dup
        # =================================================================== #
        # Append '/' unless we already have a trailing '/'.
        # =================================================================== #
        _ << '/' unless _.end_with? '/'
        _.gsub!(/ULOCAL/,'/usr/local') if _.include? 'ULOCAL' # ULOCAL gets expanded to /usr/local
        _.gsub!(/ULOC/,  '/usr/local') if _.include? 'ULOC'   # same counts for ULOC
        # =================================================================== #
        # We used to delete '-' here in the past, but as of Sep 2017 this
        # is no longer the case. Some programs may have a '-' as part of
        # the program version, and it may be better to include this rather
        # than remove it.
        # =================================================================== #
        # _.delete!('-') if _.include? '-'
      end
      set_prefix(_)
    end
  end

  # ========================================================================= #
  # === sanitize_last_update_entry
  #
  # The only sanitize-operation we do here is if it is an valid
  # date such as "32 Aug 2017".
  # ========================================================================= #
  def sanitize_last_update_entry
    _ = last_update?
    if _.include? ' '
      splitted = _.split(' ')
      day = splitted.first
      if day.to_i > 31
        opnn; e swarn('The last_update entry for ')+sfancy(program_name?)+
                swarn(' is wrong (too high).')
        opnn; e swarn('Please correct this entry.')
        # =================================================================== #
        # We could exit here but this is mostly an annoyance rather than
        # a bug, so we continue.
        # =================================================================== #
      end
    elsif _.count('.') == 2 # Input such as '24.09.2017'
      splitted = _.split('.')
      splitted[1] = return_month_based_on_this_number(splitted[1])[0,3] # This is the month but only the first 3 characters.
      _ = splitted.join(' ')
      set_last_update(_)
    end
  end; alias sanitize_last_update sanitize_last_update_entry # === sanitize_last_update

  # ========================================================================= #
  # === sanitize_program_name
  #
  # This method can be used to sanitize the program_name entry.
  #
  # The program_name entry is something such as 'gnome-mahjonng-5.08'.
  # ========================================================================= #
  def sanitize_program_name
    _ = program_name?.dup # Obtain a reference-copy first.
    i = return_sanitized_program_name(_)
    set_program_name(i) # And set the new variant.
  end

  # ========================================================================= #
  # === sync_archive_type_entries
  #
  # This method will sync archive-type entries.
  # ========================================================================= #
  def sync_archive_type_entries
    reassemble_program_full_name
    reassemble_program_path
  end

  # ========================================================================= #
  # === return_sanitized_program_name
  #
  # This will also get rid of archives.
  # ========================================================================= #
  def return_sanitized_program_name(i = program_name?.dup)
    url = url?.dup
    case i # case tag
    # ======================================================================= #
    # === FIRST_UNDERSCORE_AS_HYPHEN
    #
    # This will convert input such as 'boost_1_61_0' into 'boost-1_61_0'.
    # ======================================================================= #
    when 'FIRST_UNDERSCORE_AS_HYPHEN',
         'FIRST_UNDERSCORE_IS_HYPHEN'
      i = return_first_underscore_as_hyphen_from_this_url(url) 
    # ======================================================================= #
    # === FIRST_UNDERSCORE_AS_HYPHEN_AND_OTHER_UNDERSCORES_AS_DOTS
    #
    # This is due to boost again - apparnetly the boost developers make
    # use of a very strange packaging schemata.
    #
    # So this entry point will convert input such as 'boost_1_61_0' into
    # 'boost-1.61.0'.
    # ======================================================================= #
    when 'FIRST_UNDERSCORE_AS_HYPHEN_AND_OTHER_UNDERSCORES_AS_DOTS'
      i = return_first_underscore_as_hyphen_and_other_undscores_as_dots(url)
    # ======================================================================= #
    # === FIRST_DOT_AS_HYPHEN
    #
    # The following case is for input such as "sendmail.5.2.18"
    # which should become "sendmail-5.2.18".
    #
    # We only manipulate the last part though, so if we have input that
    # contains a '/', we assume that this is a path and split it up.
    # This is the reason why we will use File.basename()
    # ======================================================================= #
    when 'FIRST_DOT_AS_HYPHEN'
      i = return_first_dot_as_hyphen_from_this_url(url)
    # ======================================================================= #
    # === URL_REPLACE_UNDERSCORE
    #
    # This entry point handles the case when the input has '_' characters
    # in the String. If this is the case then all '_' will be replaced
    # via '-' characters.
    # ======================================================================= #
    when 'URL_REPLACE_UNDERSCORE',
         'URL_CONVERT_UNDERSCORE',
         'USE_URL_SANITIZED',
         'REPLACE_UNDERSCORES',
         'USE_URL_MIDDLE_HYPHENS'
      i = return_url_replace_underscore(url)
    # ======================================================================= #
    # === USE_URL_DOWNCASE
    #
    # Use the URL as program name but downcase it.
    # ======================================================================= #
    when 'USE_URL_DOWNCASE','USE_DOWNCASED'
      i = url.downcase
    # ======================================================================= #
    # === USE_URL
    #
    # Simply use the URL as-is, for the program name.
    # ======================================================================= #
    when 'USE_URL'
      i = url
    # ======================================================================= #
    # === PROGRAM_VERSION
    # ======================================================================= #
    when /PROGRAM_VERSION/
      program_version = program_version?
      if program_version.empty?
        program_version = remove_archive_from_the_end(
          File.basename(url?.dup).delete('a-z')
        )
      end
      i.sub!(/PROGRAM_VERSION/, program_version)
      i.squeeze!('.')
    end
    # ======================================================================= #
    # Get rid of any .tar.xz or similar archive end here.
    # ======================================================================= #
    i = remove_archive_from_the_end(i)
    return i
  end

  # ========================================================================= #
  # === sanitize_short_description
  #
  # right now this will only get rid of newlines.
  # ========================================================================= #
  def sanitize_short_description
    if short_description?.to_s.include? N
      set_short_description(short_description?.delete(N))
    end
  end

  # ========================================================================= #
  # === sanitize_required_deps
  #
  # The entry required_deps_on will be sanitized here - it must be an 
  # Array at all times.
  #
  # ========================================================================= #
  def sanitize_required_deps
    _ = required_deps_on?
    if _.is_a?(String) # Input such as 'perl cmake'. Has to become an Array.
      if _.include? ' '
        unless _.include?('>') or _.include?('<') # But not when we have comparison-operators.
          _ = _.split(' ')
        end
      end
    end
    array = []
    _ = [_] unless _.is_a? Array # Must be an Array.
    _.compact!
    _.map! {|entry|
      if entry.include? ','
        entry = entry.split(',').map(&:strip)
      end 
      entry
    } unless _.empty?
    _ = [] if _.nil? # Safeguard to keep this as an Array.
    array << _
    array = array.flatten.compact
    # ======================================================================= #
    # At this point the entry is a sanitized Array. But the cookbook
    # file may still have made a mistake, such as when an entry
    # was accidentally listed twice. (I had this problem in Feb
    # 2017, which was why I added the following code.)
    # So we need to check whether the entries are all unique - and
    # if they are not, we will notify the user that something may
    # be wrong with the yaml file in question.
    # ======================================================================= #
    if array.uniq.size != array.size
      path = Cookbooks.individual_cookbooks_dir?+program?+'.yml'
      non_unique_entries = array.uniq.map { |e| [array.count(e), e] }.select { | c, _ | c > 1 }.sort.reverse.first.last
      unless non_unique_entries.is_a? Array
        non_unique_entries = [non_unique_entries] # We need an Array.
      end
      opnn; e swarn('There may be one (or several) duplicate dependencies '\
                    ' listed in the yaml file.')
      opnn; e swarn('  Path: ')+sfile(path)
      opnn; e swarn('Here are the duplicate entries:')
      non_unique_entries.each {|duplicate_entry|
        e '  '+simp(duplicate_entry)
      }
      e swarn('If you wish to check for that on your own, have a look at the')
      e swarn('yaml file at `')+sfile(path)+swarn('`.')
      e swarn('Check the "required_deps_on:" entry of the .yml file.')
    end
    set_required_deps_on(array)
  end

  # ========================================================================= #
  # === sanitize_license
  #
  # Empty license should default to "unknown" license. This method
  # will ensure that this is the case.
  # ========================================================================= #
  def sanitize_license
    _ = licence?
    begin
      if _.nil? or _.empty?
        set_licence('unknown')
      else
        if _.is_a?(String) and _.include?(',') # <- In this case, turn it into an Array.
          set_licence(_.split(',').map(&:strip))
        end
      end
    rescue Exception => error
      opnn; e 'An error happened.'
      pp data?
      pp error
      pp error.class
      opnn; e 'Exiting now - this error (in sanitize_license()) '\
              'should be fixed though.'
      exit
    end
  end

  # ========================================================================= #
  # === sanitize_program_path
  # ========================================================================= #
  def sanitize_program_path
    reassemble_program_path
  end

  # ========================================================================= #
  # === sanitize_tags
  #
  # Keep in mind that the tags must always be in Array format.
  #
  # We will update the tags entry through this method here.
  # ========================================================================= #  
  def sanitize_tags
    _ = tags?
    format = SPECIFICATION_OF_REGISTERED_COOKBOOK_ENTRIES['tags']
    unless _.class.to_s == format
      case format
      when 'Array'
        if _.is_a? String
          if _.include? ','
            _ = _.split(',').map(&:strip)
          end
        end 
        _ = [_].compact.flatten # Convert the String into an Array here and remove nil values.
      end
    end
    if _.is_a? Array
      first = _.first
      if first
        if first.include? ' ' # We have a situation such as this one here: ["video gnome"]
          first = first.split(' ')
          _ = first
        elsif first.is_a?(String) and first.include?(',') # Some entries may have something like:  ruby,gnome  in the yaml file, we check for these here.
          first = first.split(',')
          _ = first
        end
      end
    end
    # ======================================================================= #
    # Since as of Feb 2017, we will also ensure that the tags that are
    # in use, conform to our specification. If not then we will notify
    # the user, and exit.
    # ======================================================================= #
    allowed_tags = YAML.load_file(FILE_ALLOWED_TAGS)
    _.each {|this_tag|
      if this_tag.to_s.empty? or # Empty tags are skipped as well.
         allowed_tags.include?(this_tag)
      else
        this_program = short_name?
        e
        e 'The specified tag called `'+simp(this_tag)+'` for the program '+sfancy(this_program)+
          ' has not'
        e 'been registered in the file:'
        e
        e '  `'+sfile(FILE_ALLOWED_TAGS)+'`.'
        e
        e 'This is currently not allowed, so we will exit. Please correct the '
        e 'faulty cookbook file in question, or add this tag to the registry '
        e 'of allowed tags in the specification file. '\
          '('+sfile('allowed_tags.yml')+')'
        e
        exit # And exit here.
      end
    }
    set_tags _ # Finally set the Array that has our tags.
  end

  # ========================================================================= #
  # === sanitize_yaml_dataset                        (sanitize tag, sani tag)
  #
  # Sanitize the whole yaml dataset. Note that some entries have to come
  # in a specific order.
  #
  # This method must ensure that the mandatory entries will exist in the
  # main yaml dataset.
  #
  # Additionally, we will run various sanitize-commands, through
  # subsequent method calls.
  # ========================================================================= #
  def sanitize_yaml_dataset
    set_default_raw_description
    sanitize_libraries
    sanitize_binaries
    if url1? and (short_name?.nil? or short_name?.empty?)
      guess_entries_based_on_the_first_url
    end
    sanitize_blfs_entry
    sanitize_required_deps
    sanitize_license
    sanitize_preinstall
    sanitize_use_this_make_command
    sanitize_pre_make_commands
    sanitize_configure_options
    sanitize_configure_command
    sanitize_configure_base_dir
    sanitize_url
    sanitize_apply_patch
    sanitize_short_description
    sanitize_headers
    sanitize_make_options
    sanitize_description
    sanitize_extra_information
    sanitize_tags
    sanitize_env_variables
    sanitize_last_update_entry
    sanitize_has_to_be_compiled
    try_to_sanitize_sed_entry
    sanitize_base_dir
    sanitize_program_compile_name
    sanitize_program_name
    sanitize_program_path
    sanitize_skip_configure
    # ======================================================================= #
    # The prefix setting can only be sanitized after the program_version
    # has been sanitized, since the actual prefix may depend on the
    # program version in use, such as for AppDirs.
    # ======================================================================= #
    sanitize_prefix # Must come BEFORE sanitize_postinstall.
    # ======================================================================= #
    # === sanitize_postinstall
    #
    # sanitize_postinstall should only be called after the prefix was
    # already defined, since a few internal-parts of the method
    # sanitize_postinstall may need to know the real prefix in use.
    # ======================================================================= #
    sanitize_postinstall
    unless File.exist? program_path?
      # ===================================================================== #
      # Try to see whether we can modify archive_type, in order to find
      # a proper match.
      # ===================================================================== #
      target = base_dir?+'*'
      possible_matches = Dir[target]
      unless possible_matches.empty?
        # =================================================================== #
        # This means we may have found another archive.
        # =================================================================== #
        first_entry = possible_matches.first # Obtain an entry to it.
        set_archive_type(File.basename(first_entry))
        # =================================================================== #
        # Check whether the program name is like 'apt_1.2.1'
        # =================================================================== #
        if program_name.include?('_') and !program_name.include?('-')
          unless File.basename(first_entry).include?('_')
            name, version = ProgramInformation.return_name_and_version_of(File.basename(first_entry))
            set_short_name(name)
            set_program_version(version)
            use_default_connective_token_again 
            assemble_program_name
          end
        end
        sync_archive_type_entries # This method will also update the program path.
      end
    end
    if archive_type?.empty? and !program_name?.empty?
      try_to_guess_the_archive_type
    end
    # ======================================================================= #
    # Do some syncing next - first, make sure that the program_full_name
    # setting is not empty.
    # ======================================================================= #
    if program_full_name?.empty?
      sync_archive_type_entries
    end
    # ======================================================================= #
    # Last but not least, find out the file size of the program in question.
    # ======================================================================= #
    calculate_archive_size
    calculate_md5sum if shall_we_display_the_md5sum?
    # ======================================================================= #
    # Last but not least, delete the array that holds the invalid keys,
    # since this information will no longer be relevant.
    # ======================================================================= #
    cleanup_unusued_variables
  end

  # ========================================================================= #
  # === sanitize_headers                                        (headers tag)
  #
  # This method was added as of May 2014.
  #
  # The headers entry must be kept as an Array.
  # ========================================================================= #
  def sanitize_headers
    _ = headers?
    if _.is_a? Array
      if _.first.include? ','
        _ = _.first.split(',')
      end if _.first
    end
    if _.is_a? String
      _ = _.to_s.strip
      _ = _.split(',') if _.include? ','
      _ = nil if _.empty?
    end
    _ = [_] unless _.is_a? Array
    _ = _.flatten.compact
    # ======================================================================= #
    # The individual entries may have a newline or an empty String at
    # the beginning. The next line of code ensures that we will get
    # rid of these.
    # ======================================================================= #
    _.map! {|entry| entry.delete(N).strip }
    # ======================================================================= #
    # Must keep it sorted, since as of July 2017.
    # ======================================================================= #
    _.sort!
    set_headers(_)
  end

  # ========================================================================= #
  # === sanitize_boolean_value
  #
  # This method will turn 't' into true and 'f' into false.
  # ========================================================================= #
  def sanitize_boolean_value(i)
    _ = obtain(i)
    case _
    when 't','true','yes','y'
      set(i, true)
    when 'f','false','no','n'
      set(i, false)
    end
  end

  # ========================================================================= #
  # === sanitize_skip_configure
  # ========================================================================= #
  def sanitize_skip_configure
    _ = skip_configure?
    if _.is_a? String
      case _
      when 'f'
        _ = false
      when 't'
        _ = true
      end
    end
    set_skip_configure(_)
  end

  # ========================================================================= #
  # === sanitize_description                      (description tag, desc tag)
  #
  # Get rid of trailing newlines in our description, and apply
  # some other operations. 
  # ========================================================================= #
  def sanitize_description
    _ = description?.to_s.chomp
    _.chop! if _.end_with? '"' # Get rid of leading and trailing ".
    _[0,1] = '' if _.start_with? '"'
    # ======================================================================= #
    # Some macros have to be expanded before we invoke substitute_macros().
    # SELF_NAME is such an example.
    # ======================================================================= #
    if _.include? 'SELF_NAME'
      _.gsub!(/SELF_NAME/, short_name?)
    end
    _ = substitute_macros(_)
    begin # Replace newlines with ' ' characters.
      _ = _.gsub(/#{N}/,' ').squeeze(' ')
    rescue Exception => error
      cliner {
        opnn; e 'An error happened for the description:'
        pp @_
        pp error
      }
    end
    # ======================================================================= #
    # Next, if this is empty, but short_description? is not, then
    # simply copy-paste the content of short_description? onto
    # the main program description.
    # ======================================================================= #
    if _.to_s.empty? and !short_description?.to_s.strip.empty?
      _ = short_description?.dup
    end
    set_description _
  end

  # ========================================================================= #
  # === sanitize_has_to_be_compiled
  # ========================================================================= #
  def sanitize_has_to_be_compiled
    _ = has_to_be_compiled?
    case _
    when 'no'
      _ = false
    when 'yes'
      _ = true
    end
    set_has_to_be_compiled(_)
  end

  # ========================================================================= #
  # === sanitize_make_options
  #
  # This method will sanitize the make options used for a given program.
  #
  # Special macros can be used as part of the make options.
  #
  # For instance:
  #
  #   MY_PROGRAMS/Procps/PROGRAM_VERSION
  #
  # Will be expanded to whatever value MY_PROGRAMS was set to; usually it
  # is, or should be, pointing towards "/Programs/".
  #
  # Do also note that this means that the value for MY_PROGRAMS must
  # exist, otherwise this can not possibly work. Of course you can
  # omit using macros and just hardcode values there.
  #
  # Another example for macro-conversion would be the String 'RINSTALL1',
  # which will become expanded into the following three actions:
  #
  #   ruby install.rb config
  #   ruby install.rb setup
  #   ruby install.rb install
  #
  # The latter expansions are part of the file:
  #
  #   predefined_installation_instructions.yml
  #
  # So if you want to add new macros, you can do so by modifying that
  # yaml file.
  #
  # The make_options need to be An array in the end.
  # ========================================================================= #
  def sanitize_make_options
    _ = make_options?
    _ = 'make' if _.nil? || _.empty? # <- Use "make" this as default value if empty.
    _ = [_] unless _.is_a? Array
    # ========================================================================= #
    # Past this point here we have an Array.
    # ========================================================================= #
    unless _.empty?
      _.map! {|line|
        line = line.dup if line.frozen?
        line.strip!
        # =================================================================== #
        # Next, replace all ',' with ';' characters.
        # =================================================================== #
        line.tr!(',',';') if line.include? ','
        if Object.const_defined? :ConvertGlobalEnv
          line = ConvertGlobalEnv[line] if line.include? '$'
        end
        # =================================================================== #
        # === Replace the predefined installation instruction next
        # =================================================================== #
        if PREDEFINED_INSTALLATION_INSTRUCTIONS.include? line
          line = PREDEFINED_INSTALLATION_INSTRUCTIONS[line]
        end
        if line.include? 'VERSION'
          # ================================================================= #
          # Need to set @program_version first, hence we update some stuff.
          # ================================================================= #
          set_program_name(program_name?) # This equals, for example, to "gnome-mahjonng-5.08"
          if line.include? 'PROGRAM_VERSION' 
            line.gsub!(/PROGRAM_VERSION/, program_version?) if program_version?
          elsif line.include? 'VERSION'
            line.gsub!(/VERSION/, program_version?) if program_version?
          end
        end
        line = line.split(';') if line.include? ';' # This may turn it into an Array.
        line
      }
    end
    _.flatten!
    set_make_options(_)
  end

  # ========================================================================= #
  # === sanitize_use_this_make_command
  #
  # This will default to whatever is stored in the configuration file.
  #
  # Should be 'make' usually, but could be "cmake" or another make-like
  # command.
  # ========================================================================= #
  def sanitize_use_this_make_command
    if use_this_make_command?.to_s.empty?
      set_use_this_make_command(MAKE_COMMAND_TO_USE) # MAKE_COMMAND_TO_USE is defined in constants.rb
    end
  end

end; end