#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Gobolinux::CreateRecipe
#
# This can be used to generate a Gobolinux-recipe from scratch.
#
# Simply pass the name of the recipe that you wish to create to
# new(), such as in:
#
#   Cookbooks::Gobolinux::CreateRecipe.new('htop')
#
# Since this is a bit verbose, here is a class method as shortcut:
#
#   Cookbooks.create_gobolinux_recipe('htop')
#
# =========================================================================== #
# require 'cookbooks/gobolinux/create_recipe.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/gobolinux/gobolinux_naming_convention.rb'

module Cookbooks

module Gobolinux

class CreateRecipe < Base # === Cookbooks::Gobolinux::CreateRecipe

  # ========================================================================= #
  # === NEW_LINE_SEPARATOR
  # ========================================================================= #
  # NEW_LINE_SEPARATOR = '\\' # Not sure if this is required.

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === USE_THIS_GOBOLINUX_COMPILE_VERSION
  #
  # Which gobolinux-compile version to use. Currently this is hardcoded.
  #
  # On GoboLinux, we could probably determine the "Compile --version".
  # ========================================================================= #
  USE_THIS_GOBOLINUX_COMPILE_VERSION = '2.0' # '1.13.3'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_this_program(i)
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # The following entry point means that we will store e. g. 'gtk' into
      # the subdirectory 'g/gtk'.
      # ===================================================================== #
      when :store_in_alphanumeric_subdirectory
        use_this_first_character = @this_program[0,1].dup
        @store_in_this_base_directory << use_this_first_character+'/'
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === temporary_base_dir?
  # ========================================================================= #
  def temporary_base_dir?
    @store_in_this_base_directory
  end

  # ========================================================================= #
  # === set_store_in_this_base_directory
  #
  # This method will keep track as to where to store the recipe - the
  # base directory.
  # ========================================================================= #
  def set_store_in_this_base_directory(i = Dir.pwd)
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @store_in_this_base_directory = i
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i)
    i = i.first if i.is_a? Array
    @this_program = i
    set_dataset
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @main_string = ''.dup
    set_store_in_this_base_directory
  end

  # ========================================================================= #
  # === version?
  # ========================================================================= #
  def version?
    @dataset.program_version?
  end; alias return_version version? # === return_version

  # ========================================================================= #
  # === set_dataset
  # ========================================================================= #
  def set_dataset(i = @this_program)
    @dataset = ::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end; alias this_program        this_program? # === this_program
       alias program?            this_program? # === program?
       alias name_of_the_program this_program? # === name_of_the_program
       alias program_name?       this_program? # === program_name?

  # ========================================================================= #
  # === return_gobolinux_name
  #
  # This method will return the GoboLinux name. It will make use of the
  # class GobolinuxNamingConvention.
  # ========================================================================= #
  def return_gobolinux_name(i = @this_program)
    ::Cookbooks.return_gobolinux_name(i)
  end; alias return_gobolinux_name_for_this_program return_gobolinux_name # === return_gobolinux_name_for_this_program

  # ========================================================================= #
  # === notify_the_user_what_we_will_do
  # ========================================================================= #
  def notify_the_user_what_we_will_do
    _ = program?
    opnn; e 'We will next create a gobolinux-recipe for the '\
            'program '+simp(_)+'.'
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === return_file_size
  # ========================================================================= #
  def return_file_size
    @dataset.file_size?
  end

  # ========================================================================= #
  # === return_md5sum
  #
  # Here we need to keep in mind that the target path may not exist.
  # ========================================================================= #
  def return_md5sum(i = path)
    if File.exist? i
      result = `md5sum #{i}`.split(' ').first
      result.chomp
    else
      opnn; e swarn('No target file at `')+sfile(i)+
              swarn('`. We will continue nonetheless.')
      '0' # Return 0 in this case.
    end
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === path?
  # ========================================================================= #
  def path?
    dataset?.program_path?
  end; alias path path? # === path

  # ========================================================================= #
  # === program_name_and_version
  # ========================================================================= #
  def program_name_and_version
    name_of_the_program+'-'+version?
  end

  # ========================================================================= #
  # === licence?
  # ========================================================================= #
  def licence?
    _ = @dataset.licence?.to_s
    # ======================================================================= #
    # GoboLinux uses a slightly different scheme, hence the following
    # line which acts as a corrective.
    # ======================================================================= #
    _ = 'GNU GPL 2' if _ == 'GPLv2'
    return _
  end; alias license? licence? # === license?

  # ========================================================================= #
  # === url1
  # ========================================================================= #
  def url1
    @dataset.url1.to_s
  end

  # ========================================================================= #
  # === url2
  # ========================================================================= #
  def url2
    @dataset.url2.to_s
  end

  # ========================================================================= #
  # === dependencies?
  # ========================================================================= #
  def dependencies?
    @dataset.required_deps_on? # This is assumed to return an Array.
  end

  # ========================================================================= #
  # === pad_with_quotes
  # ========================================================================= #
  def pad_with_quotes(i)
    unless i.include? '"'
      i = '"'+i+'"'
    end
    return i
  end

  # ========================================================================= #
  # === return_symlink_options
  #
  # I assume that this will cause SymlinkProgram to overwrite, in case
  # there is a conflict.
  # ========================================================================= #
  def return_symlink_options
    'symlink_options=(--conflict=overwrite)'+N
  end

  # ========================================================================= #
  # === add
  # ========================================================================= #
  def add(i = N)
    @main_string << i
  end

  # ========================================================================= #
  # === populate_recipe_file                                     (recipe tag)
  # ========================================================================= #
  def populate_recipe_file
    add ''.dup
    add '# Recipe for version '+return_version+' (autogenerated via '\
        'file create_recipe.rb)'+N
    add '# Recipe (MakeRecipe) for '+return_gobolinux_name+N
    add 'compile_version='+USE_THIS_GOBOLINUX_COMPILE_VERSION+N
    add 'url='+pad_with_quotes(url1)+N
    add 'file_size='+return_file_size.to_s+N
    add 'file_md5='+return_md5sum.to_s+N
    add 'recipe_type=configure'+N
    add return_symlink_options
    # ======================================================================= #
    # Next, if the configure options for the given program in question exist,
    # then we will batch-add them to the recipe file. 
    # ======================================================================= #
    coptions = configure_options?
    unless coptions.empty? # Only when it is NOT empty, will we append it.
      add 'configure_options=('+N
      coptions.squeeze(' ').split(' ').each {|inner_coption|
        unless inner_coption.include? '--enable-udev'
          # ================================================================= #
          # Use udev-options a bit differently - see GoboLinux recipes
          # such as libdrm.
          # ================================================================= #
          add '  '+inner_coption+N
        end
      }
      add ')'+N
    end
    consider_appending_more_stuff
    into = @store_in_this_base_directory+program_name_and_version+'/Recipe'
    # ======================================================================= #
    # The main String was built up there.
    # ======================================================================= #
    write_what_into(@main_string, into)
  end

  # ========================================================================= #
  # === consider_appending_more_stuff
  #
  # Add some more exceptions and so forth.
  # ========================================================================= #
  def consider_appending_more_stuff
    if coptions?.include? '--enable-udev'
      add 'with_udev=('+N
      add '  --enable-udev'+N
      add ')'
    end
  end

  # ========================================================================= #
  # === configure_options?
  # ========================================================================= #
  def configure_options?
    @dataset.configure_options? # This appears to be a String.
  end; alias coptions? configure_options? # === coptions?

  # ========================================================================= #
  # === short_desc?
  #
  # Wrapper over the short_description of a program.
  # ========================================================================= #
  def short_desc?
    _ = @dataset.short_description?.to_s
    if _.size > 68
      _ = word_wrap(_, 68)
    end
    _.chomp!
    _.strip!
    return _
  end

  # ========================================================================= #
  # === create_directory_skeleton
  #
  # This method will create the required directories and files to
  # accomodate a gobolinux recipe.
  # ========================================================================= #
  def create_directory_skeleton
    _ = temporary_base_dir?+name_of_the_program+'-'+version?+'/'
    mkdir _
    opnn; touch rds(_+'/Recipe'), :be_verbose
    mkdir rds(_+'/Resources/')
    opnn; touch rds(_+'/Resources/BuildDependencies'), :be_verbose
    opnn; touch rds(_+'/Resources/BuildInformation'), :be_verbose
    opnn; touch rds(_+'/Resources/Dependencies'), :be_verbose
    opnn; touch rds(_+'/Resources/Description'), :be_verbose
    opnn; touch rds(_+'/Resources/Environment'), :be_verbose # This one is rarely needed.
    populate_description_file
    populate_dependencies_file
    populate_recipe_file # Need to fill up the file "Recipe" here.
  end

  # ========================================================================= #
  # === populate_description_file
  #
  # This method will generate a (hopefully) valid GoboLinux Recipe
  # description file. This file resides in the subdirectory
  # Resources/ hierarchy.
  #
  # For examples of such a file in use by GoboLinux, have a look at:
  #
  #   http://recipes.gobolinux.org/r/?list=LibDRM&ver=2.4.80-r1&file=Resources/Description
  #   http://recipes.gobolinux.org/r/?list=ZzipLib&ver=0.13.58-r1&file=Resources/Description
  # ========================================================================= #
  def populate_description_file(
      i = program_name?
    )
    _ = ''.dup
    _ << '[Name] '+return_gobolinux_name_for_this_program(i)+N # Unsure whether this line is needed - htop 2.0.x does not have it.
    _ << '[Summary] '+short_desc?+N
    _ << '[Description] '+description?.rstrip+N # .split(N).join(NEW_LINE_SEPARATOR+N)
    _ << '[License] '+license?+N
    if has_a_homepage_url?
      _ << '[Homepage] '+homepage_url?+N
    else # else we will simply use url2 instead
      _ << '[Homepage] '+url2+N
    end
    into = @store_in_this_base_directory+program_name_and_version+'/Resources/Description'
    write_what_into(_, into)
  end

  # ========================================================================= #
  # === has_a_homepage_url?
  #
  # Query whether the program has a registered homepage url.
  # ========================================================================= #
  def has_a_homepage_url?
    homepage_url?
  end

  # ========================================================================= #
  # === homepage_url?
  # ========================================================================= #
  def homepage_url?
    @dataset.homepage?
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    _ = @dataset.description?.delete(N)
    _ = word_wrap(_, 68) # Condense the description a bit.
    return _
  end

  # ========================================================================= #
  # === populate_dependencies_file
  # ========================================================================= #
  def populate_dependencies_file
    _ = ''.dup
    dependencies?.each {|dep|
      _ << dep+N
    }
    into = @store_in_this_base_directory+program_name_and_version+'/Resources/Dependencies'
    write_what_into(_, into)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    notify_the_user_what_we_will_do
    create_directory_skeleton
  end

end; end

  # ========================================================================= #
  # === Cookbooks.create_gobolinux_recipe
  #
  # Easier module-method instantiation.
  # ========================================================================= #
  def self.create_gobolinux_recipe(i = 'htop')
    yielded = nil
    if block_given?
      yielded = yield
    end
    ::Cookbooks::Gobolinux::CreateRecipe.new(i) { yielded }
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::Gobolinux::CreateRecipe.new(ARGV)  
end # gobo_recipe zenity