#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::AvailableCookbooks
#
# Use this class whenever you wish to get all cookbook entries.
#
# The result is stored in the instance variable @result.
#
# Usage example:
#   _ = AvailableCookbooks.new(apply_filter)
#   return _.result
# =========================================================================== #
# require 'cookbooks/misc/available_cookbooks.rb'
# =========================================================================== #
require 'cookbooks/base/prototype.rb'
require 'cookbooks/module_methods/all_binaries.rb'
require 'cookbooks/module_methods/e.rb'

module Cookbooks # === Cookbooks::AvailableCookbooks

class AvailableCookbooks < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      shall_we_use_a_filter_or_not = true,
      run_already                  = true
    )
    reset
    set_apply_filter(shall_we_use_a_filter_or_not)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === entries?
  #
  # Will return how many entries exist.
  # ========================================================================= #
  def entries?
    @entries
  end; alias entries entries? # === entries

  # ========================================================================= #
  # === set_target
  # ========================================================================= #
  def set_target
    @target = Cookbooks.individual_cookbooks_directory?+'*.yml'
  end

  # ========================================================================= #
  # === set_apply_filter
  # ========================================================================= #
  def set_apply_filter(i = true)
    i = false if i == :do_not_use_a_filter
    @apply_filter = i
  end

  # ========================================================================= #
  # === target?
  # ========================================================================= #
  def target?
    @target
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias results? result? # results? tag.

  # ========================================================================= #
  # === get_results
  #
  # This method may also apply some filtering. The results are the
  # names of the individual programs.
  # ========================================================================= #
  def get_results
    # ======================================================================= #
    # Here comes the code to define which programs are registered.
    # ======================================================================= #
    result = Dir[target?].map {|entry| # Fetch all entries from the cookbooks dir.
      if @apply_filter # if true we return only the name. This is the default behaviour.
        entry = File.basename(entry).sub(/\.yml$/, '')
        entry
      else
        entry
      end
    }
    set_result(result)
  end

  # ========================================================================= #
  # === sort_result
  # ========================================================================= #
  def sort_result
    set_result(result?.sort) # Always sort it and keep it in a sorted display.
  end

  # ========================================================================= #
  # === set_result
  # ========================================================================= #
  def set_result(i)
    @result = i # Assign to @result here.
  end

  # ========================================================================= #
  # === determine_how_many_entries_exist
  # ========================================================================= #
  def determine_how_many_entries_exist
    @entries = @result.size.to_s
  end

  # ========================================================================= #
  # === report_result
  #
  # Here we will report our result.
  # ========================================================================= #
  def report_result
    result?.each {|entry| e entry }
  end; alias report report_result # === report

  # ========================================================================= #
  # === Cookbooks::AvailableCookbooks[]
  #
  # Return all available cookbooks here.
  # ========================================================================= #
  def self.[](optional_argument = '') # This argument is not needed though.
    AvailableCookbooks.new(optional_argument)
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result    result? # === result
       alias programs? result? # === programs?

  # ========================================================================= #
  # === report_how_many_cookbook_entries_were_found
  # ========================================================================= #
  def report_how_many_cookbook_entries_were_found
    opnn; e 'How many cookbook entries have we been able '\
           'to find? '+simp(entries?)+'.'
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    set_target
    get_results
    determine_how_many_entries_exist
    sort_result
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_everything
  end

end

  GetCookbooks = AvailableCookbooks # An "alias" pointer.

  # ========================================================================== #
  # === get_cookbooks
  #
  # You can use this method whenever you ought to retrieve all available
  # cookbooks.
  #
  # A sorted array will be returned, consisting of all the various cookbooks.
  # ========================================================================== #
  def get_cookbooks(apply_filter = true)
    _ = AvailableCookbooks.new(apply_filter)
    return _.result
  end

  # ========================================================================= #
  # === Cookbooks.available_cookbooks?
  #
  # The resulting Array that will be returned here will always be sorted.
  #
  # Note that there are at the least 5 different aliases to this method,
  # such as Cookbooks.available_programs(). The variant that is preferred
  # is the one named here, aka "Cookbooks.available_cookbooks?".
  # ========================================================================= #
  def self.available_cookbooks?
    Cookbooks::AvailableCookbooks.new.results?
  end; self.instance_eval { alias get_cookbooks       available_cookbooks? } # === Cookbooks.get_cookbooks
       self.instance_eval { alias available_programs  available_cookbooks? } # === Cookbooks.available_programs
       self.instance_eval { alias available_programs? available_cookbooks? } # === Cookbooks.available_programs?
       self.instance_eval { alias available_cookbooks available_cookbooks? } # === Cookbooks.available_cookbooks
       self.instance_eval { alias programs?           available_cookbooks? } # === Cookbooks.programs?

  # ========================================================================= #
  # === Cookbooks.n_programs?
  #
  # Return how many programs are registered in the Cookbooks project.
  # ========================================================================= #
  def self.n_programs?
    Cookbooks.available_programs.size
  end; self.instance_eval { alias n_registered_programs? n_programs? } # === Cookbooks.n_registered_programs?

  # ========================================================================= #
  # === Cookbooks.feedback_help
  # ========================================================================= #
  def self.feedback_help
    Cookbooks.available_programs?.each_with_index {|entry, index|
      index += 1
      e '('+simp(index.to_s.rjust(3))+') '+sfancy(entry)
    }
  end; self.instance_eval { alias feedback_available_programs feedback_help } # === Cookbooks.feedback_available_programs

  # ========================================================================= #
  # === Cookbooks.does_include_this_binary?
  #
  # This will return true if we have a binary with that name.
  #
  # Usage example:
  #   Cookbooks.does_include_this_binary? 'xxd'
  # ========================================================================= #
  def self.does_include_this_binary?(i)
    ::Cookbooks.all_binaries?.has_key?(i)
  end; self.instance_eval { alias has_this_binary? does_include_this_binary? } # === Cookbooks.has_this_binary?

  # ========================================================================= #
  # === Cookbooks.is_registered?
  #
  # With this method you can check whether a given program is included.
  #
  # Note that for legacy reasons mostly, we will NOT check for aliases
  # here. If you must do a check that includes aliases then you MUST
  # use Cookbooks.really_does_include?() instead. 
  #
  # Usage examples:
  #   Cookbooks.is_registered? 'ruby'
  #   Cookbooks.is_registered? 'ruby2'
  # ========================================================================= #
  def self.is_registered?(i)
    Cookbooks.available_programs?.include?(i.to_s) # We require a String.
  end; self.instance_eval { alias is_available?          is_registered? } # === Cookbooks.is_available?
       self.instance_eval { alias has_this_program?      is_registered? } # === Cookbooks.has_this_program?
       self.instance_eval { alias does_include?          is_registered? } # === Cookbooks.does_include?
       self.instance_eval { alias includes_this_program? is_registered? } # === Cookbooks.includes_this_program?

end

if __FILE__ == $PROGRAM_NAME # Code to test the above.
  require 'colours/autoinclude'
  require 'opn'
  _ = Cookbooks::AvailableCookbooks.new
  _.report_result
  _.report_how_many_cookbook_entries_were_found
  e
  opn; e 'Is `xxd` a part of the cookbooks project?'
  e Cookbooks.does_include_this_binary?('xxd')
end # available_cookbooks