#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# This file keeps the whole configuration-logic in one place, mostly, so
# that it can be required. It will also include the SimpleConfgurationLoader
# class, which is just a small wrapper over loading the .yml dataset.
# =========================================================================== #
# require 'cookbooks/module_methods/initialize_configuration.rb'
# =========================================================================== #
require 'cookbooks/directory/cookbooks_configuration_dir.rb'
require 'cookbooks/configuration/simple_configuration_loader.rb'
require 'cookbooks/module_methods/permanently_set_source_directory.rb'
require 'cookbooks/module_methods/permanently_set_temp_directory.rb'
require 'cookbooks/module_methods/convert_global_env.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/module_methods/opn.rb'
require 'cookbooks/misc/save_file.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks::FILE_TEMP_DIRECTORY_YAML
  #
  # Pointer towards the project's temp_directory.yml file.
  # ========================================================================= #
  FILE_TEMP_DIRECTORY_YAML =
    "#{Cookbooks.cookbooks_configuration_dir?}temp_directory.yml" 

  # ========================================================================= #
  # === Cookbooks.file_temp_directory
  #
  # Return the path to the .yml file in question here. This is thus a
  # reader method.
  # ========================================================================= #
  def self.file_temp_directory
    FILE_TEMP_DIRECTORY_YAML
  end

  # ========================================================================= #
  # === @configuration_object
  # ========================================================================= #
  @configuration_object = nil

  # ========================================================================= #
  # === Cookbooks.set_configuration_object
  # ========================================================================= #
  def self.set_configuration_object(
      i = Cookbooks::SimpleConfigurationLoader.new
    )
    @configuration_object = i
  end

  # ========================================================================= #
  # === Cookbooks.configuration_object?
  #
  # Query the configuration object through this getter method.
  # ========================================================================= #
  def self.configuration_object?
    @configuration_object
  end; self.instance_eval { alias configuration? configuration_object? } # === Cookbooks.configuration?

  # ========================================================================= #
  # === Cookbooks.set_source_directory
  #  
  # Set the value of @source_directory here. A trailing "/" must be kept
  # for these variables, as they are directories.
  #
  # On my system this constant equals to the String "/Users/x/SRC/". On
  # other systems it can be different, settable via the commandline.
  # ========================================================================= #
  def self.set_source_directory(i)
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    i << '/' unless i.end_with? '/'
    @source_directory = i
  end

  # ========================================================================= #
  # === Cookbooks.set_temp_directory
  # ========================================================================= #
  def self.set_temp_directory(i)
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    i << '/' unless i.end_with? '/'
    @temp_directory = i
  end

  # ========================================================================== #
  # === Cookbooks.ensure_that_the_required_directories_exist
  #
  # This method ensures that the base-directory exists.
  # ========================================================================== #
  def self.ensure_that_the_required_directories_exist(
      use_this_as_directory = ::Cookbooks.store_into_this_directory?
    )
    unless Dir.exist? use_this_as_directory
      if File.writable?(File.dirname(use_this_as_directory))
        opn; e "Creating directory `#{use_this_as_directory}`."
        FileUtils.mkdir_p(use_this_as_directory)
      else
        e "Can not write into the directory #{use_this_as_directory}"\
          ". You should use another"
        e 'setting for temp_directory.yml perhaps.'
        e
        e 'This file can be found here:'
        e "  #{file_temp_directory}"
      end
    end
  end

  # ========================================================================= #
  # === Cookbooks.set_store_into_this_directory
  #
  # We will store Cookbooks-related output into this directory
  # preferentialy. This is the reason why the ensure-method is
  # called, to make sure that these directories exist.
  # ========================================================================= #
  def self.set_store_into_this_directory(i)
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    i << '/' unless i.end_with? '/'
    @store_into_this_directory = i
    ensure_that_the_required_directories_exist(i)
  end

  # ========================================================================= #
  # === Cookbooks.set_programs_directory
  # ========================================================================= #
  def self.set_programs_directory(i = '/Programs/')
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there are still '$' entries then this variable does not
    # exist. In this case, use a hardcoded entry.
    # ======================================================================= #
    i = '/Programs/' if i.include? '$'
    i << '/' unless i.end_with? '/'
    @programs_directory = i
  end

  # ========================================================================= #
  # === Cookbooks.set_system_directory
  #
  # The system directory.
  #
  # This will depend on the yaml configuration. Alternatively it will
  # be set to a hardcode default value.
  #
  # This constant equals to "/System/".
  # ========================================================================= #
  def self.set_system_directory(
      i = '/System/'
    )
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    i << '/' unless i.end_with? '/'
    @system_directory = i
  end

  # ========================================================================= #
  # === Cookbooks.set_configuration_dependent_variables
  #
  # This method will set some configuration-dependent values, such
  # as the location of the source directory.
  #
  # The various @ variables defined here should be preferentially used
  # as opposed to their CONSTANT values. But ideally, simply use
  # the getter methods, such as "::Cookbooks.source_directory?" or
  # "::Cookbooks.temp_directory?".
  # ========================================================================= #
  def self.set_configuration_dependent_variables(
      i = @configuration_object
    )
    if i.respond_to? :source_directory
      ::Cookbooks.set_source_directory(i.source_directory)
    else
      # ===================================================================== #
      # else simply hardcode it. This allows us to not have to
      # depend on any .yml file.
      # ===================================================================== #
      i = '/Users/x/SRC/'
      ::Cookbooks.set_source_directory(i)
    end
    if i.respond_to? :temp_directory
      ::Cookbooks.set_temp_directory(i.temp_directory)
    else
      # ===================================================================== #
      # else simply hardcode it. This allows us to not have to
      # depend on any .yml file.
      # ===================================================================== #
      i = '/tmp/'
      ::Cookbooks.set_temp_directory(i)
    end
    if i.respond_to? :system_directory
      ::Cookbooks.set_system_directory(i.system_directory)
    else
      # ===================================================================== #
      # else simply hardcode it. This allows us to not have to
      # depend on any .yml file.
      # ===================================================================== #
      i = '/System/'
      ::Cookbooks.set_system_directory(i)
    end
    if i.respond_to? :programs_directory
      ::Cookbooks.set_programs_directory(i.programs_directory)
    else
      # ===================================================================== #
      # else simply hardcode it. This allows us to not have to
      # depend on any .yml file.
      # ===================================================================== #
      i = '/Programs/'
      ::Cookbooks.set_programs_directory(i)
    end
    if i.respond_to? :store_into_this_directory
      ::Cookbooks.set_store_into_this_directory(i.store_into_this_directory)
    else
      # ===================================================================== #
      # else simply hardcode it. This allows us to not have to
      # depend on any .yml file.
      # ===================================================================== #
      i = '/Depot/Cookbooks/'
      ::Cookbooks.set_store_into_this_directory(i)
    end
    # ======================================================================= #
    # Define the instance variable @use_yaml_database_for_tags next, so
    # that we can, in some queries, use grep or a pure ruby solution to
    # find patterns in text.
    # ======================================================================= #
    @use_yaml_database_for_tags = i.use_yaml_database_for_tags
  end

  # ========================================================================= #
  # === Cookbooks.initialize_configuration
  #
  # This method will be used to initialize the configuration object.
  #
  # Currently we will use class Cookbooks::SimpleConfigurationLoader,
  # but in principle, we could also use other implementations,
  # such as class Configuration.
  # ========================================================================= #
  def self.initialize_configuration(
      initialize_a_new_object_no_matter_what = false
    )
    if initialize_a_new_object_no_matter_what
      set_configuration_object(Cookbooks::SimpleConfigurationLoader.new)
      set_configuration_dependent_variables
    else
      unless @configuration_object # Initialize anew here.
        set_configuration_object(Cookbooks::SimpleConfigurationLoader.new)
        set_configuration_dependent_variables
      end
    end
  end

  # ========================================================================= #
  # Initialize the configuration object next.
  # ========================================================================= #
  Cookbooks.initialize_configuration if @configuration_object.nil?

  # ========================================================================= #
  # === Cookbooks.source_directory?
  #
  # Simpler query method for the source directory in use.
  #
  # This defaults to /Users/x/SRC on my system - may be different on
  # other computers, which is why this has to be settable and
  # modifiable by the user.
  # ========================================================================= #
  def self.source_directory?
    @source_directory
  end; self.instance_eval { alias source_dir? source_directory? } # Cookbooks.source_dir?

  # ========================================================================= #
  # === Cookbooks.system_directory?
  # ========================================================================= #
  def self.system_directory?
    @system_directory
  end

  # ========================================================================= #
  # === SYSTEM_DIRECTORY
  # ========================================================================= #
  SYSTEM_DIRECTORY = ::Cookbooks.system_directory?
    SYS_DIR        = SYSTEM_DIRECTORY # "alias to the ^^^ above.
    SYSTEM_DIR     = SYSTEM_DIRECTORY # "alias to the ^^^ above.
    # ======================================================================= #
    # And we also define some more constants such as SYSLIB_DIR
    # SYSBIN_DIR and SYSINCLUDE_DIR. Since these depend on SYSTEM_DIRECTORY,
    # they must be defined here.
    # ======================================================================= #
    SYSLIB_DIR     = "{SYS_DIR}lib"
    SYSBIN_DIR     = "{SYS_DIR}bin"
    SYSINCLUDE_DIR = "{SYS_DIR}include"

  # ========================================================================= #
  # === Cookbooks.use_yaml_database_for_tags?
  # ========================================================================= #
  def self.use_yaml_database_for_tags?
    @use_yaml_database_for_tags
  end

  # ========================================================================= #
  # === Cookbooks.programs_directory?
  # ========================================================================= #
  def self.programs_directory?
    @programs_directory
  end; self.instance_eval { alias program_directory? programs_directory? } # === Cookbooks.program_directory?

  # ========================================================================= #
  # === PROGRAMS_DIRECTORY
  # ========================================================================= #
  PROGRAMS_DIRECTORY = ::Cookbooks.programs_directory?
        PROGRAMS_DIR = PROGRAMS_DIRECTORY # An "alias" to the ^^^ above.

  # ========================================================================= #
  # === Cookbooks.store_into_this_directory?
  # ========================================================================= #
  def self.store_into_this_directory?
    @store_into_this_directory
  end; self.instance_eval { alias store_into_this_directory store_into_this_directory? } # === Cookbooks.store_into_this_directory?
       self.instance_eval { alias store_where?              store_into_this_directory? } # === Cookbooks.store_where?

  # ========================================================================= #
  # === STORE_INTO_THIS_DIRECTORY
  # ========================================================================= #
  STORE_INTO_THIS_DIRECTORY = Cookbooks.store_into_this_directory?

  # ========================================================================= #
  # === Cookbooks.base_dir_to_store_expanded_cookbooks?
  # ========================================================================= #
  def self.base_dir_to_store_expanded_cookbooks?
    "#{::Cookbooks.store_into_this_directory}expanded_cookbooks/"
  end

  DIRECTORY_EXPANDED_COOKBOOKS = Cookbooks.base_dir_to_store_expanded_cookbooks?

  # ========================================================================= #
  # === STORE_THE_LAST_CONFIGURATION_OPTIONS_IN_THIS_FILE
  # ========================================================================= #
  STORE_THE_LAST_CONFIGURATION_OPTIONS_IN_THIS_FILE =
    "#{::Cookbooks.store_into_this_directory?}last_configure_options_used.md" 

  # ========================================================================= #
  # === Cookbooks.temp_directory?
  # ========================================================================= #
  def self.temp_directory?
    @temp_directory
  end; self.instance_eval { alias temp_dir? temp_directory? } # === Cookbooks.temp_directory?
       self.instance_eval { alias log_dir?  temp_directory? } # === Cookbooks.log_dir?

  # ========================================================================= #
  # === Cookbooks::TEMP_DIR
  # ========================================================================= #
  TEMP_DIR = ::Cookbooks.temp_dir?
    TMP_DIR       = TEMP_DIR # An "alias" to the above value.
    TMP_DIRECTORY = TEMP_DIR # An "alias" to the above value.

  # ========================================================================= #
  # === Cookbooks::SRC_DIR
  #
  # A constant is obviously static, but the user can also overrule the
  # source directory from the commandline. If you need to refer to
  # the source directory then you should not use this constant; instead,
  # make use of:
  #   Cookbooks.source_directory?
  #
  # The constant below exists mostly for legacy reasons.
  # ========================================================================= #
  SRC_DIR = ::Cookbooks.source_directory? # Just an alias-pointer.
    # ======================================================================= #
    # === SOURCE_DIR
    # ======================================================================= #
    SOURCE_DIR       = SRC_DIR
    # ======================================================================= #
    # === SOURCE_DIRECTORY
    # ======================================================================= #
    SOURCE_DIRECTORY = SRC_DIR

  # ========================================================================= #
  # === Cookbooks::FILE_SOURCE_DIRECTORY
  # ========================================================================= #
  FILE_SOURCE_DIRECTORY =
    "#{Cookbooks.cookbooks_configuration_dir?}source_directory.yml"
    FILE_SOURCE_DIRECTORY_YAML = FILE_SOURCE_DIRECTORY # === FILE_SOURCE_DIRECTORY_YAML

  # ========================================================================= #
  # === Cookbooks::Configuration.permanently_set_path_to_this_source_directory
  #
  # The first argument should be the path to a directory.
  #
  # This directory will then serve as the new base directory where you
  # may keep your archives.
  #
  # Usage example from the commandline:
  #
  #   cookbooks --source_directory=/Depot/Files
  #
  # ========================================================================= #
  def self.permanently_set_path_to_this_source_directory(
      i
    )
    i = i.to_s unless i.is_a? String
    i = i.dup if i.frozen?
    unless i.end_with? '/'
      i << '/'
    end
    opn; e "Setting a new source directory to: #{sdir(i)}"
    ::Cookbooks.set_source_directory(i)
    # ======================================================================= #
    # Next, we have to save this value as well, for it to make it
    # permanent.
    # ======================================================================= #
    write_what_into(i, FILE_SOURCE_DIRECTORY) # Must save it here.
  end

end