#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Blfs
#
# This class will display the BLFS entry, that is, the "Beyond Linux from
# Scratch" entry for a given program.
#
# Note that not every entry in the cookbooks will have such a BLFS entry -
# mostly just the more commonly available programs will have a BLFS entry.
# =========================================================================== #
# require 'cookbooks/utility_scripts/blfs.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
begin
  require 'xorg_buffer/module'
rescue LoadError; end

module Cookbooks

class Blfs < Base # Cookbooks::Blfs

  include XorgBuffer if Object.const_defined? :XorgBuffer

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = '',
      run_already = true
    )
    register_sigint
    reset # reset() must come before the remaining methods.
    set_program(i)
    if block_given?
      yielded = yield
      case yielded
      when :be_quiet_and_do_not_tamper_with_the_xorg_buffer
        @be_verbose = false
        @also_assign_the_xorg_buffer = false
      when :be_quiet
        @be_verbose = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @homepage = nil
    @run_in_batch_mode = false
    @also_assign_the_xorg_buffer = true
  end

  # ========================================================================= #
  # === set_program
  #
  # We seek this program.
  # ========================================================================= #
  def set_program(i)
    i = i.first if i.is_a? Array
    i = i.to_s.dup.downcase # Downcase since as of Sep 2017.
    i = i[0, i.index('.')] if i.include? '.'
    if i.empty?
      run_in_batch_mode
    end
    @program = i
  end; alias set_input set_program # === set_input

  # ========================================================================= #
  # === assign_xorg_buffer
  # ========================================================================= #
  def assign_xorg_buffer(i)
    XorgBuffer[i] if @also_assign_the_xorg_buffer
  end

  # ========================================================================= #
  # === run_in_batch_mode
  # ========================================================================= #
  def run_in_batch_mode
    @run_in_batch_mode = true
  end

  # ========================================================================= #
  # === program?
  #
  # This will yield to us which program we are searching for. In other
  # words, this will be equal to the input the user provided to this
  # class.
  # ========================================================================= #
  def program?
    @program
  end; alias input? program? # === input?

  # ========================================================================= #
  # === all_blfs_entries
  # ========================================================================= #
  def all_blfs_entries
    Cookbooks.return_all_blfs_entries
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    @homepage # Store the homepage here.
  end; alias homepage    homepage? # === homepage
       alias remote_url? homepage? # === remote_url?

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === check_blfs_entry_for
  # ========================================================================= #
  def check_blfs_entry_for(i)
    _ = Cookbooks::Cookbook.new(input?) { :bypass_menu }
    @homepage = _.obtain 'blfs' # Fetch the blfs entry.
    report_homepage # Defaults to @homepage as argument.
  end

  # ========================================================================= #
  # === report_that_no_blfs_entry_exists_for_the_program
  # ========================================================================= #
  def report_that_no_blfs_entry_exists_for_the_program
    if be_verbose?
      opnn; e 'No BLFS entry is available for `'+simp(program?)+'`.'
    end
  end

  # ========================================================================= #
  # === report_homepage
  # ========================================================================= #
  def report_homepage(
      i                              = homepage?,
      also_assign_to_the_xorg_buffer = true
    )
    if i.is_a? Array
      if i.empty?
        if be_verbose?
          report_that_no_blfs_entry_exists_for_the_program
        end
      else
        assign_xorg_buffer(i)
        i.each {|entry| report_homepage(entry, false) }
      end
    else
      if i.to_s.empty? and be_verbose?
        opnn; e 'No BLFS entry could be found for '\
                'the program `'+sfancy(input?)+'`.'
      else
        if be_verbose?
          opnn; e i
        end
        assign_xorg_buffer(i) if also_assign_to_the_xorg_buffer
      end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    if @run_in_batch_mode # In this case, we check for all BLFS entries here.
      all_blfs_entries.each {|program_name, url|
        if url # Some entries currently may be nil, since we use grep.
          set_input(program_name)
          report_homepage(url)
        end
      }
    else # else we check on only one entry.
      check_blfs_entry_for(input?)
    end
  end

end

  # ========================================================================= #
  # === Cookbooks.return_all_blfs_entries
  #
  # This method will (silently) return an Array with all entries that
  # have a specific tag.
  #
  # This currently uses grep.
  #
  # The result will look like this:
  #
  #   ["libpcap",
  #    "http://www.linuxfromscratch.org/blfs/view/svn/basicnet/libpcap.html"],
  #   ["xulrunner",
  #    "http://www.linuxfromscratch.org/blfs/view/7.5/x/xulrunner.html"],
  # ========================================================================= #
  def self.return_all_blfs_entries
    cmd = 'grep -r "^ blfs" '+Cookbooks.individual_cookbooks_directory?
    result = `#{cmd}`.split(N).map {|entry|
      entry.gsub!(/ blfs: /,' ')
      if entry.include? ':'
        splitted = entry.split ':'
        splitted[0] = File.basename(splitted[0]).gsub(/\.yml/,'')
        entry = splitted.join ':'
      end
      entry.split(': ')
    }
    return result
  end

  # ========================================================================= #
  # === Cookbooks.blfs
  # ========================================================================= #
  def self.blfs(i = 'curl')
    Cookbooks::Blfs.new(i)
  end

  # ========================================================================= #
  # === Cookbooks.return_blfs_page_of_this_program
  #
  # This method will return the BLfS page of a program, or nil if it
  # was not registered.
  #
  # Returns:
  #   - either a string
  #   - or nil
  #
  # Invocation example:
  #   result = Cookbooks.return_blfs_page_of_this_program('gcc')
  # ========================================================================= #
  def self.return_blfs_page_of_this_program(i)
    _ = Cookbooks::Blfs.new(i) { :be_quiet_and_do_not_tamper_with_the_xorg_buffer }
    result = _.remote_url?
    result = result.first if result.is_a? Array
    return result
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::Blfs.new(ARGV)
end # blfs ruby