#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CheckIfSourceDirectoriesExist
#
# This class will iterate through the various cookbook entries and count
# what we have got there. It will then report which directories do not
# exist.
# =========================================================================== #
# require 'cookbooks/utility_scripts/check_for_existing_source_directories.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
begin
  require 'class_docu_shower'
rescue LoadError; end

module Cookbooks

class CheckIfSourceDirectoriesExist < Base # === Cookbooks::CheckIfSourceDirectoriesExist

  # ========================================================================= #
  # === PADDING
  # ========================================================================= #
  PADDING = 35

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = nil,
      run_already    = true
    )
    reset
    set_commandline_arguments(optional_input)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @array_targets = []
    @be_verbose = false # if true then we will also report directories that exist.
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help(optional_then_exit = false)
    if Object.const_defined? :ClassDocuShower
      ClassDocuShower.new(__FILE__)
    end
    exit if optional_then_exit
  end

  # ========================================================================= #
  # === check_commandline_arguments
  # ========================================================================= #
  def check_commandline_arguments
    case @commandline_arguments.first # case tag
    when 'HELP',
         /-?-?help/
      show_help_then_exit
    end
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = nil)
    i = i.first unless i.is_a? Array
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === register_target_does_not_exist
  # ========================================================================= #
  def register_target_does_not_exist(i)
    @array_targets << i
  end

  # ========================================================================= #
  # === report_missing_entries
  #
  # Report all missing entries here.
  # ========================================================================= #
  def report_missing_entries # report them all now.
    sanitize_targets
    @array_targets.sort.each {|entry|
      e sdir(entry.ljust(PADDING))+' does not exist.'
    }
    unless @array_targets.empty?
      _ = @array_targets.size.to_s
      # ===================================================================== #
      # Always report non-existing directories.
      # ===================================================================== #
      opn; e "We found #{simp(_)} non-existing directories."
    end
  end

  # ========================================================================= #
  # === sanitize_targets
  # ========================================================================= #
  def sanitize_targets
    @array_targets.map! {|entry| entry << '/' }
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    PADDING
  end

  # ========================================================================= #
  # === check_each_entry
  # ========================================================================= #
  def check_each_entry
    Cookbooks.available_cookbooks.each {|entry|
      target = source_directory?+File.basename(entry).sub(/\.yml$/,'')
      shall_we_check_this_target = false
      if File.exist? target # ok, the target exists, all may be fine, so we pass through.
        shall_we_check_this_target = true
      end
      # ===================================================================== #
      # Some entries in the yaml file have a "do_not_download"
      # entry. In that case we will ignore them. 
      # ===================================================================== #
      location = individual_cookbooks_directory?+File.basename(entry)
      if File.exist? location
        hash = load_this_cookbook(location) # in module_methods.rb
        unless hash['do_not_download'].nil?
        end
      end
      if shall_we_check_this_target
        if be_verbose?
          e 'Target '+sfile(target.ljust(padding?))+' does exist.'
        end
      else # does not exist, report
        register_target_does_not_exist(target)
      end
    }
  end

  # ========================================================================= #
  # === load_this_cookbook
  # ========================================================================= #
  def load_this_cookbook(i)
    Cookbooks::Cookbook.new(i) { :bypass_menu }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_commandline_arguments
    check_each_entry
    report_missing_entries
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CheckIfSourceDirectoriesExist.new(ARGV)
end # cexist HELP