#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CreateEbuild
#
# This class allows you to create .ebuild files.
#
# An ebuild is a specialized bash script which automates compilation
# and installation procedures for software packages on Gentoo.
#
# The purpose of ebuilds called "metabuilds"  is to trigger the
# installation of other ebuilds, such as the GNOME or KDE
# metabuilds.
#
# All ebuilds have the following attributes:
#
#   - A category, e.g. app-accessibility
#   - A package name, e.g. gcc or nvidia-drivers
#   - A version number, e.g. 1.05_beta3 or 100.14.11
#
# It is important to specify these attributes correctly.
#
# For ebuild-examples, look here:
#
#   https://sources.gentoo.org/cgi-bin/viewvc.cgi/gentoo-x86/sys-process/htop/htop-1.0.3.ebuild?view=markup
#
# =========================================================================== #
# require 'cookbooks/utility_scripts/create_ebuild.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'

module Cookbooks

class CreateEbuild < Cookbooks::Base # ry htop make_ebuild

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #
  DEFAULT_PROGRAM = 'ruby'

  # ========================================================================= #
  # === DIRECTORY_FOR_EBUILDS
  #
  # Where to store the ebuilds.
  # ========================================================================= #
  DIRECTORY_FOR_EBUILDS = ::Cookbooks.temp_dir?+'DirectoryForEbuilds/'

  # ========================================================================= #
  # === PORTAGE_BASE_DIRECTORY
  #
  # Where portage normally keeps an .ebuild file.
  # ========================================================================= #
  PORTAGE_BASE_DIRECTORY = '/usr/local/portage/' # hello-world.ebuild 

  # ========================================================================= #
  # === initialize
  #
  # The first argument should contain the dataset.
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_data(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    set_program_name # Set to empty String.
  end

  # ========================================================================= #
  # === set_data
  #
  # The input should be a Hash.
  # ========================================================================= #
  def set_data(i = nil)
    i = DEFAULT_PROGRAM if i.nil?
    i = Cookbooks::Cookbook.new(i) { :bypass_menu }.dataset? if i.is_a? String
    if i.is_a? Array
      if i.empty? # Empty Arrays will be treated differently.
        i = Cookbooks::Cookbook.new(DEFAULT_PROGRAM) { :bypass_menu }.dataset?
      else
        i = i.join(' ')
      end
    end
    @data = i
    if @data.is_a? Hash
      set_program_name(@data['short_name'])
    end
  end

  # ========================================================================= #
  # === data?
  # ========================================================================= #
  def data?
    @data
  end

  # ========================================================================= #
  # === set_program_name
  # ========================================================================= #
  def set_program_name(i = '')
    @program_name = i
  end

  # ========================================================================= #
  # === store_where?
  #
  # We could also store in the directory PORTAGE_BASE_DIRECTORY.
  # ========================================================================= #
  def store_where?
    DIRECTORY_FOR_EBUILDS+program_name?+'.ebuild'
  end

  # ========================================================================= #
  # === short_name?
  # ========================================================================= #
  def short_name?
    data?.short_name?
  end

  # ========================================================================= #
  # === do_create_the_ebuild
  #
  # This method will create the specific .ebuild file in question.
  # ========================================================================= #
  def do_create_the_ebuild
    license = ''.dup # empty at default
    license = data?.license? unless data?.license?.empty?
    # ======================================================================= #
    # Generate the main string next.
    # ======================================================================= #
    string = ''.dup
    string << 
'EAPI="5"

PYTHON_COMPAT=( python2_7 )

AUTOTOOLS_AUTORECONF=true

inherit autotools-utils linux-info python-any-r1

ARTS_REQUIRED="never"

MY_P="'+short_name?+'"

DESCRIPTION="'+data?.description?+'"

HOMEPAGE="'+url1?+'"
SRC_URI="'+url1?+'"
LICENSE="'+license+'"

SLOT="0"
KEYWORDS="~amd64 ~x86"
IUSE="debug doc"
need-kde 3.5
S="${WORKDIR}/${MY_P}"

DEPEND="'+required_deps_on?+'"

RDEPEND="${DEPEND}
  x11-libs/qt-svg:4[debug?]"

src_compile () {
  mkdir build
  cd build
  cmake ..
  emake || die "emake failed"
}

src_install () {
  cd build
  emake DESTDIR=${D} install || die "emake install failed"
  dodoc ChangeLog README TODO
}
'
    write_what_into(string, store_where?) # Overwrite.
    opnn; e 'Created ebuild for '+program?+' at `'+sfile(store_where?)+'`.'
  end

  # ========================================================================= #
  # === required_deps_on?
  # ========================================================================= #
  def required_deps_on?
    data?.required_deps_on?.to_s
  end

  # ========================================================================= #
  # === url1?
  # ========================================================================= #
  def url1?
    data?.url1?
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end; alias program? program_name? # === program?

  # ========================================================================= #
  # === notify_the_user_what_we_are_about_to_do
  # ========================================================================= #
  def notify_the_user_what_we_are_about_to_do
    e
    cliner {
      opnn; e 'Now creating ebuild for '+sfancy(@program_name)+' in'
      opnn; e "  `#{sfile(store_where?)}`."
    }; e
  end

  # ========================================================================= #
  # === guess_program_name
  #
  # This method will try to guess the program name.
  # ========================================================================= #
  def guess_program_name
    _ = data?.real_short_name?
    set_program_name(_)
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    create_directory(DIRECTORY_FOR_EBUILDS)  # Create the basic ebuild directory.
    # ======================================================================= #
    # Next, consider creating the portage base directory.
    # Not everyone wants this though, so I have uncommented it for now.
    # ======================================================================= #
    # create_directory(PORTAGE_BASE_DIRECTORY)
    guess_program_name if program_name?.empty?
    notify_the_user_what_we_are_about_to_do
    do_create_the_ebuild
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CreateEbuild.new(ARGV)
end # cebuild