#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CreateNewCookbook
#
# This class will create a new cookbook file, a yaml file, with
# default values.
#
# You can pass the URL to this script and it will figure out the name
# of the yaml file in question.
#
# The class also allows for interactive use - see "--help" for more
# information. The interactive part can be started via "--interactive".
#
# Usage examples:
#   Cookbooks::CreateNewCookbook.new
# =========================================================================== #
# require 'cookbooks/utility_scripts/create_new_cookbook.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'program_information'

module Cookbooks
        
class CreateNewCookbook < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # Various constants
  # ========================================================================= #
  YAML_EXTENSION = '.yml'

  # ========================================================================= #
  # === CURRENT_MONTH
  # ========================================================================= #
  CURRENT_MONTH = Date::MONTHNAMES[
    Date.today.month
  ][0,3]+' '+Time.now.strftime('%Y') # => "Feb 2013"

  # ========================================================================= #
  # === CURRENT_DAY_AND_MONTH
  # ========================================================================= #
  CURRENT_DAY_AND_MONTH = Time.now.strftime('%d')+' '+CURRENT_MONTH

  # ========================================================================= #
  # === DEFAULT_NAME_FOR_THE_YAML_FILE
  # ========================================================================= #
  DEFAULT_NAME_FOR_THE_YAML_FILE = 'foobar'+YAML_EXTENSION

  # ========================================================================= #
  # === OPEN_IN_EDITOR_AFTER_CREATION
  # ========================================================================= #
  OPEN_IN_EDITOR_AFTER_CREATION = true

  # ========================================================================= #
  # === ROEBE_INDIVIDUAL_COOKBOOKS_DIRECTORY
  # ========================================================================= #
  ROEBE_INDIVIDUAL_COOKBOOKS_DIRECTORY =
    "#{ROEBE_PATH_TO_COOKBOOKS}yaml/individual_cookbooks/"

  # ========================================================================= #
  # === ALL_NEWLY_CREATED_YAML_FILES_MUST_RESIDE_IN_INDIVIDUAL_COOKBOOKS_DIRECTORY
  #
  # If the following constant is true, then we will move newly created
  # .yml files into the INDIVIDUAL_COOKBOOKS directory - unless they
  # already reside there.
  #
  # Note that in order for this to work, the constant
  # INDIVIDUAL_COOKBOOKS must exist.
  # ========================================================================= #
  ALL_NEWLY_CREATED_YAML_FILES_MUST_RESIDE_IN_INDIVIDUAL_COOKBOOKS_DIRECTORY = true

  # ========================================================================= #
  # === EDITOR
  #
  # Here we can decide which editor to use. For now we use vim.
  # ========================================================================= #
  if ENV['EDITOR']
    USE_THIS_EDITOR = ENV['EDITOR'].to_s
  else # Else hardcode it.
    USE_THIS_EDITOR = 'bluefish'
  end

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be the full URL to the target program
  # in question.
  # ========================================================================= #
  def initialize(
      input       = nil,
      run_already = true
    )
    reset
    set_commandline_input(input)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    set_compile_mode
    @interactive_mode = true
    @_ = ''
    @base_dir = ''
    @editor = USE_THIS_EDITOR
    @open_in_editor_after_creation = OPEN_IN_EDITOR_AFTER_CREATION
    @short_name = ''
    set_description
    set_file DEFAULT_NAME_FOR_THE_YAML_FILE # Default name.
    set_url1
  end

  # ========================================================================= #
  # === do_not_open_in_editor
  # ========================================================================= #
  def do_not_open_in_editor
    @open_in_editor_after_creation = false
  end

  # ========================================================================= #
  # === do_open_in_editor
  # ========================================================================= #
  def do_open_in_editor
    @open_in_editor_after_creation = false
  end

  # ========================================================================= #
  # === set_description
  # ========================================================================= #
  def set_description(
      i = 'Default description of the program.'
    )
    i = i.to_s
    @description = i
  end

  # ========================================================================= #
  # === add_hint_for_dependencies
  # ========================================================================= #
  def add_hint_for_dependencies
    if is_on_roebe?
      ''
    else # Add this to help newbies.
      '# The dependencies go in here.'+N
    end
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i = nil)
    i = i.first if i.is_a? Array
    case i
    # ======================================================================= #
    # === --interactive
    # ======================================================================= #
    when /^-?-?interactive/,'INTERACTIVE'
      enter_interactive_part
    # ======================================================================= #
    # === --help
    # ======================================================================= #
    when 'HELP','--help','help','h','-h'
      show_help
      exit
    end
  end

  # ========================================================================= #
  # === set_commandline_input
  # ========================================================================= #
  def set_commandline_input(
      i = nil
    )
    menu(i)
    @commandline_input = i
    if i
      if @commandline_input.is_a? Array
        set_url1(@commandline_input.first)
      elsif @commandline_input.is_a? String
        set_url1(@commandline_input)
      end
    end
  end

  # ========================================================================= #
  # === set_base_dir
  # ========================================================================= #
  def set_base_dir(i)
    i = i.to_s # We require a String here.
    @base_dir = ' base_dir: '+i+N
    update
  end

  # ========================================================================= #
  # === set_url1
  # ========================================================================= #
  def set_url1(
      i = ''
    )
    @url1 = i
    unless i.empty?
      set_filename(@url1)
      set_short_name(@url1)
    end
  end; alias set_url set_url1 # === set_url

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file
  end

  # ========================================================================= #
  # === set_short_name
  #
  # This sets the short name of the program in question.
  # ========================================================================= #
  def set_short_name(i = '')
    # First, do some sanitization.
    if i.include? '.' # Assume that a file extension was given. Chop it away thus.
      i = i.to_s.dup
      i.gsub!(/#{File.extname(i)}/,'')
    end
    i = return_program_information(i)
    i = i.delete('_') if i.include? '_'
    i = i.downcase
    @short_name = i # This variable can be inferred from the url.
    create_main_string
  end

  # ========================================================================= #
  # === open_in_editor?
  # ========================================================================= #
  def open_in_editor?
    @open_in_editor_after_creation
  end

  # ========================================================================= #
  # === set_filename
  #
  # This will give us the filename, i.e. where to store the file.
  # ========================================================================= #
  def set_filename(i)
    i = i.to_s.dup
    i = return_program_information(i, false)
    if i
      i << YAML_EXTENSION unless i.end_with? '.yml'
      # ===================================================================== #
      # Since as of March 2014 we will also downcase the name.
      # ===================================================================== #
      i.downcase!
    end
    set_file(i)
  end; alias set_real_name set_filename # === set_real_name

  # ========================================================================= #
  # === set_file
  #
  # This method is not allowed to do any modifications - it will simply
  # assign towards the @file variable.
  # ========================================================================= #
  def set_file(i)
    @file = i
  end

  # ========================================================================= #
  # === return_program_information
  #
  # Simple wrapper over class ProgramInformation.
  # ========================================================================= #
  def return_program_information(i, be_verbose = false)
    ::ProgramInformation.return_real_name(i)
  end

  # ========================================================================= #
  # === consider_creating_a_new_cookbook_file
  #
  # This method is the one that will create the new cookbook.
  # ========================================================================= #
  def consider_creating_a_new_cookbook_file
    if File.exist? @_
      if File.directory? @_
        opnn; ewarn sdir(@_)+' seems to be a directory. Will not overwrite it.'
      else
        opnn; ewarn 'But '+sfile(@_)+' already exists.'
        opnn; ewarn 'Shall we really overwrite this file?'
        case $stdin.gets.chomp
        when 'y','yes','j','ja'
          opnn; ewarn 'Overwriting `'+sfile(@_)+swarn('` now.')
          store
        else
          opnn; ewarn 'Ok, then not.'
        end
      end
    else
      store
    end
  end

  # ========================================================================= #
  # === report_to_user
  #
  # Report to the user that we have finished creating the new file.
  # ========================================================================= #
  def report_to_user
    opnn; e 'Finished creating the file `'+sfile(file?)+'`.'
    # ======================================================================= #
    # === Move it into the cookbooks directory if the constant was set
    # ======================================================================= #
    if ALL_NEWLY_CREATED_YAML_FILES_MUST_RESIDE_IN_INDIVIDUAL_COOKBOOKS_DIRECTORY
      unless Dir.pwd == INDIVIDUAL_COOKBOOKS
        # =================================================================== #
        # First, we will copy that file to my home directory
        # if we are on roebe.
        # =================================================================== #
        if is_on_roebe?
          target = ROEBE_INDIVIDUAL_COOKBOOKS_DIRECTORY+File.basename(file?)
          opnn
          e 'Copying the '+sfile(file?)+' into the directory '+
             sdir(ROEBE_INDIVIDUAL_COOKBOOKS_DIRECTORY)
          opnn
          e 'It is now available here:'
          opnn
          e sfile('  '+target)
          copy_file(file?, target)
          do_open_in_editor
          edit_this_file target
          do_not_open_in_editor # We only care about the main directory at home.
        end
        target_file = INDIVIDUAL_COOKBOOKS+File.basename(file?)
        if File.exist? target_file
          opnn; e 'Can not move the file to '+sfile(target_file)+' as there'
          opnn; e 'already is a file at that location.'
        else
          opnn; e 'Moving '+sfile(file?)+' into the directory '+sdir(INDIVIDUAL_COOKBOOKS)
          opnn; e 'as we have set the constant '+
                  simp(
                    'ALL_NEWLY_CREATED_YAML_FILES_MUST_RESIDE_IN_INDIVIDUAL_COOKBOOKS_DIRECTORY'
                  )
          target = INDIVIDUAL_COOKBOOKS+File.basename(file?)
          move_file(file?, target)
          set_file(target)
        end
      end
    end
  end

  # ========================================================================= #
  # === store                                                     (store tag)
  #
  # This method stores the dataset into a .yml file finally.
  # ========================================================================= #
  def store
    write_what_into(@_, file?)
  end; alias save store # === save

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === consider_opening_in_editor
  #
  # We may use the newly created file in our main editor.
  # ========================================================================= #
  def consider_opening_in_editor(this_file = file?)
    if open_in_editor?
      _ = @editor+' '+this_file
      opnn; e 'We will next open this file in our editor, as instructed due'
      opnn; e 'to the ivar '+sfancy('@open_in_editor_after_creation')
      opnn; e 'having been set to true.'
      system(_)
    end
  end; alias edit_this_file consider_opening_in_editor # === edit_this_file

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e 'How to create a new cookbook:' # newcookbook --help
    e
    e 'Simply provide the name of the .yml file, such as:'
    e
    e '   newcookbook php.yml'
    e
    e 'Alternatively, provide a remote URL which we will use to '\
      'generate such a file.'
    e
    e '  --interactive # this will ask you several questions before'
    e '                # the new .yml file will be created'
  end

  # ========================================================================= #
  # === enter_interactive_part
  # ========================================================================= #
  def enter_interactive_part
    @interactive_mode = true
    e 'We will next ask which values should be added to the yaml file.'
    e 'Remote URL?'
    @url1 = $stdin.gets.chomp
    e 'Description of the propram?'
    @description = $stdin.gets.chomp
    e 'Done!'
    create_main_string
  end
  
  # ========================================================================= #
  # === create_main_string
  #
  # This holds our recipe-skeleton.
  # ========================================================================= #
  def create_main_string
    @_ = @short_name+':
 # binaries:
 configure_options: "

  "
 description: |
  "'+@description.to_s+'"'+@base_dir.to_s+'
 extra_information: |
  "Extra information for this program to use."
 url1: '+@url1+'
 url2: '+File.dirname(@url1.to_s)+'/'+'
 prefix: '+@compile_mode+'
 keep_extracted: f
 tags: # gnome xorg
 required_deps_on: '+add_hint_for_dependencies+'
 # libraries: 
 last_update: '+CURRENT_DAY_AND_MONTH
  end; alias assign_to_main_string create_main_string # === assign_to_main_string
       alias update_main_string    create_main_string # === update_main_string
       alias ums                   create_main_string # === ums
       alias update                create_main_string # === update

  # ========================================================================= #
  # === set_compile_mode
  # ========================================================================= #
  def set_compile_mode(i = :traditional)
    case i.to_s
    when 't'
      i = 'prefix = t'
    when 'f'
      i = 'prefix = f'
    end
    i = i.to_s unless i.is_a? String
    if i.is_a?(String) and i.frozen?
      i = i.dup
    end
    i << N unless i.end_with? N
    @compile_mode = i
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    update unless @interactive_mode
    consider_creating_a_new_cookbook_file # This will create the file in question.
    report_to_user
    consider_opening_in_editor
  end

  # ========================================================================= #
  # === run                                                       (run tag)
  # ========================================================================= #
  def run
    run_everything
  end

end

  # ========================================================================= #
  # === Cookbooks.create_new_cookbook
  # ========================================================================= #
  def self.create_new_cookbook(i = 'test')
    CreateNewCookbook(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CreateNewCookbook.new(ARGV)
end # ccook http://ftp.gnome.org/pub/GNOME/sources/gnome-online-miners/3.9/gnome-online-miners-3.9.90.tar.xz