#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CreateRecipe
#
# The task of this class is to create a recipe, so use it when you have
# to create a recipe.
#
# Right now this class only supports the creation of simple yaml files,
# but perhaps the unctionality will be expanded one day.
#
# Since as of October 2015, we can optionally make use of
# class Cookbooks::Cookbook as well.
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'
begin
  require 'program_information'
rescue LoadError; end

module Cookbooks

class CreateRecipe < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE       = inspect

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #
  DEFAULT_PROGRAM = 'htop'

  # ========================================================================= #
  # === STORE_HERE
  # ========================================================================= #
  STORE_HERE      = ::Cookbooks.temp_dir?

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_program = DEFAULT_PROGRAM,
      run_already  = true
    )
    super()
    reset
    set_program(this_program)
    case run_already
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @use_sanitize_cookbook_dataset = true # Make use of SanitizeCookbookDataset in this case.
    set_store_where
  end

  # ========================================================================= #
  # === use_cookbook_dataset?
  # ========================================================================= #
  def use_cookbook_dataset?
    @use_sanitize_cookbook_dataset
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @program
  end; alias input? program? # === input?

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    @store_where
  end

  # ========================================================================= #
  # === try_to_initialize_the_dataset_via_sanitize_cookbook_dataset
  # ========================================================================= #
  def try_to_initialize_the_dataset_via_sanitize_cookbook_dataset
    @dataset = ::Cookbooks::Cookbook.new(input?)
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === create_the_recipe
  # ========================================================================= #
  def create_the_recipe
    try_to_initialize_the_dataset_via_sanitize_cookbook_dataset
    file_name = "#{@program}.yml"
    full_recipe_location = store_where?+file_name
    _ = full_recipe_location.dup
    create_directory(store_where?) if !File.exist?(store_where?)
    opnn; e 'Now creating a standalone recipe for '+sfancy(program?)+' at '
    opnn; e sdir(store_where?)+sfile(file_name)
    remove_file(_) # Get rid of the file first.
    create_file(_)
    store_where = _
    _ = "#{input?}:#{N}".dup
    _ << ' program_name: '+input?+N
    _ << ' short_description: '+short_desc?.to_s+N
    unless binaries?.empty?
      _ << ' binaries: '+binaries?.join(',')+N
    end
    _ << ' configure_options: "'+N+
         remove_leading_hyphens(configure_options?)+N+N+' "'+N
    _ << ' description: |'+N
    _ << '  '+word_wrap(description?).chomp+N
    _ << ' extra_information: |'+N
    _ << ' prefix: '+prefix?+N
    if prefix?.include? programs_dir?
      _ << ' symlink_pkgconfig_files: t'+N
    end
    _ << ' # postinstall: '+N
    _ << ' url1: '+url1?.to_s+N
    _ << ' url2: '+url2?.to_s+N
    _ << ' required_deps_on: '+required_deps?+N
    _ << ' last_update: '+today?+N
    append_what_into(_, store_where)
    opnn; e 'Finished creating recipe at '
    opnn; e '  `'+sfile(full_recipe_location)+'`.'
  end

  # ========================================================================= #
  # === remove_leading_hyphens
  # ========================================================================= #
  def remove_leading_hyphens(i)
    i.gsub(/ --/,' ')
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    dataset?.binaries?
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    if use_cookbook_dataset?
      dataset?.description?
    else
      ''
    end
  end

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    if use_cookbook_dataset?
      dataset?.url2?
    else
      ''
    end
  end

  # ========================================================================= #
  # === short_desc?
  # ========================================================================= #
  def short_desc?
    if use_cookbook_dataset?
      dataset?.short_desc?
    else
      ''
    end
  end

  # ========================================================================= #
  # === url1?
  # ========================================================================= #
  def url1?
    if use_cookbook_dataset?
      dataset?.url1?
    else
      ''
    end
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    if use_cookbook_dataset?
      dataset?.prefix?
    else
      ''
    end
  end

  # ========================================================================= #
  # === required_deps?
  # ========================================================================= #
  def required_deps?
    if use_cookbook_dataset?
      dataset?.required_deps?.join(',')
    else
      ''
    end
  end

  # ========================================================================= #
  # === configure_options?
  # ========================================================================= #
  def configure_options?
    if use_cookbook_dataset?
      dataset?.configure_options?.gsub(/--/,N+'    --')
    else
      ''
    end
  end

  # ========================================================================= #
  # === set_store_where
  #
  # Set the base directory, where to store, here.
  # ========================================================================= #
  def set_store_where(i = STORE_HERE+'IndividualCookbooks/') # overrule for now
    i = i.dup
    i << '/' unless i.end_with? '/'
    @store_where = i
  end; alias store_where set_store_where # === store_where

  # ========================================================================= #
  # === set_program
  # ========================================================================= #
  def set_program(i = DEFAULT_PROGRAM) # sets @program.
    i = DEFAULT_PROGRAM if i.nil?
    @original_input = i
    if i.include? '-'
      i = File.basename(i)
      i = ProgramInformation.return_real_short_name(i) 
    end
    @program = i
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # run tag
    create_the_recipe
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = Cookbooks::CreateRecipe.new(ARGV.first, :dont_run_yet)
  _.set_store_where Dir.pwd
  _.run
end # crecipe htop