#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::DownloadAllSourceArchives
#
# This class can be used to download all source archives into the
# corresponding "archive directory". On my system this directory
# usually is /Users/x/SRC/, but other users may designate another
# "archive directory" to be used.
#
# Into this so called "archive directory", source archives will be
# downloaded, but only if they do not yet exist there. This works
# in a batch mode, that is - everything will be downloaded one after
# the other.
#
# Note that this class here also allows you to restrict the candidates
# that will be downloaded. Presently, the only way that this is
# possible is via the registered "tags". So for example - if you
# pass in the argument "--tags=plasma", then the class here will
# only download entries that have that tag "plasma" registered.
#
# This way you can selectively download a subset of the cookbooks.
#
# Usage example:
#   Cookbooks::DownloadAllSourceArchives.new
# =========================================================================== #
# require 'cookbooks/utility_scripts/download_all_source_archives.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class DownloadAllSourceArchives < Base # === Cookbooks::DownloadAllSourceArchives.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(commandline_arguments)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @tag = nil
    @archive_directory = source_directory?
    @available_programs = available_programs? # <- Must come before the menu() call.
  end

  # ========================================================================= #
  # === set_use_these_cookbooks
  #
  # The input to this method should be an Array.
  # ========================================================================= #
  def set_use_these_cookbooks(i)
    if i.is_a? Array
      i.flatten!
      i.sort!
    end
    @available_programs = i
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @available_programs
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end; alias input? commandline_arguments? # === input?

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i = commandline_arguments?)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === download_all_source_archives --tags=plasma
      # ===================================================================== #
      when /^-?-?tags=(.+)$/,
           /^-?-?tag=(.+)$/
        register_this_tag($1.to_s.dup)
        set_use_these_cookbooks(select_only_these_tags(@tag))
      # ===================================================================== #
      # === download_all_source_archives --help
      # ===================================================================== #
      when /help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === register_this_tag
  # ========================================================================= #
  def register_this_tag(i)
    @tag = i
  end

  # ========================================================================= #
  # === report_the_archive_directory_in_use
  # ========================================================================= #
  def report_the_archive_directory_in_use
    if @tag
      opnn; e "All registered programs with the tag #{sfancy(@tag)}"\
              " will be downloaded into the directory:"
    else
      opnn; e 'All registered programs will be downloaded into the directory:'
    end
    e
    e sfile("  #{@archive_directory}")
    e
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    opnn; e 'This class allows you to download source archives.'
    opnn; e 'You can also selectively download only certain entries, in'
    opnn; e 'particular those who have registered tags.'
    e
    opnn; e 'For example, if you wish to download all KDE-plasma related entries'
    opnn; e 'then you can issue the following command:'
    e
    opnn; e sfancy('  download_all_source_archives --tags=plasma')
    e
  end

  # ========================================================================= #
  # === consider_downloading_this_program
  # ========================================================================= #
  def consider_downloading_this_program(i)
    unless Cookbooks.const_defined? :Cookbook
      require 'cookbooks/class/class.rb'
    end
    cd_to_the_archive_directory # Always enter the archive directory. 
    # ======================================================================= #
    # Next, check whether the program exists.
    # ======================================================================= #
    _ = Cookbooks::Cookbook.new(i) { :bypass_menu_check }
    program_path = _.program_path?
    remote_url   = _.remote_url?
    unless File.exist? program_path
      this_directory = "#{@archive_directory}#{i}/"
      unless File.directory? this_directory
        opnn; e "Creating the directory `#{sdir(this_directory)}` next."
        mkdir this_directory
      end
      cd this_directory
      unless File.exist? program_path
        download(remote_url)
        opnn; e "Next downloading `#{simp(remote_url)}`."
      end
    end
  end

  # ========================================================================= #
  # === download
  #
  # The download-functionality requires wget right now.
  # ========================================================================= #
  def download(i)
    esystem "wget #{i}"
  end

  # ========================================================================= #
  # === cd_to_the_archive_directory
  # ========================================================================= #
  def cd_to_the_archive_directory
    cd @archive_directory, :be_quiet
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === select_only_these_tags
  #
  # To invoke this method, try:
  #   download_all_source_archives --tags=plasma
  # ========================================================================= #
  def select_only_these_tags(i)
    # ======================================================================= #
    # Ok, the user provided a tag, so we have to selectively filter for
    # programs that include only that tag. For now, we only use the
    # expanded cookbook dataset for this. Of course we must check whether
    # this directory exists or whether it does not.
    # ======================================================================= #
    if File.directory? DIRECTORY_EXPANDED_COOKBOOKS
      entries = Dir[DIRECTORY_EXPANDED_COOKBOOKS+'*.yml']
      results = entries.map {|entry|
        YAML.load_file(entry)
      }.select {|entry| entry['tags'].include? i }
      results.map! {|entry| entry['real_short_name'] }
      return results
    else
      opnn; e "No directory called #{sdir(DIRECTORY_EXPANDED_COOKBOOKS)} exists."
      opnn; e 'This class currently requires this directory to work.'
      opnn; e 'If you wish to auto-generate the directory and the files or that'
      opnn; e 'directory, then issue this command:'
      e
      e sfancy('  cookbooks --expand')
      e
      exit
    end
  end

  # ========================================================================= #
  # === process_the_programs_one_after_the_other
  # ========================================================================= #
  def process_the_programs_one_after_the_other(
      i = @available_programs
    )
    i.each {|this_program|
      consider_downloading_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    report_the_archive_directory_in_use
    process_the_programs_one_after_the_other
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

  # ========================================================================= #
  # === Cookbooks.download_all_source_archives
  # ========================================================================= #
  def self.download_all_source_archives(i = ARGV)
    ::Cookbooks::DownloadAllSourceArchives.new(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::DownloadAllSourceArchives.new(ARGV)
end # download_all_source_archives