#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ExpandCookbooks
#
# This class will expand all the dataset stored in the various
# individual .yml files, from the cookbooks, and sanitize them
# into a big, sanitized hash.
#
# This hash will have all the information, in a proper way.
# =========================================================================== #
# require 'cookbooks/utility_scripts/expand_cookbooks.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/misc/available_cookbooks.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/create_program_version_url_file.rb'

module Cookbooks

class ExpandCookbooks < Base # === Cookbooks::ExpandCookbooks

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === STORE_HERE
  # ========================================================================= #
  STORE_HERE = ::Cookbooks.store_into_this_directory?

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      expand_which_programs = :all_of_them,
      run_already           = true
    )
    super()
    reset
    set_expand_these_programs(expand_which_programs)
    run if run_already
  end

  # ========================================================================= #
  # === set_expand_these_programs
  # ========================================================================= #
  def set_expand_these_programs(
      i = :all_of_them
    )
    if i.is_a? Array
      i = i.first
    end
    i = :all_of_them if i.nil? # Assign to the default value, in this case.
    case i
    # ======================================================================= #
    # === expand_cookbooks --help
    # ======================================================================= #
    when /^-?-?help/
      show_help(:then_exit)
    when :all_of_them
      i = @cookbooks
    end
    unless i.is_a? Array
      i = [i] # Turn it into an Array.
    end
    @expand_these_programs = i
  end

  # ========================================================================= #
  # === main_array?
  # ========================================================================= #
  def main_array?
    @expand_these_programs
  end

  # ========================================================================= #
  # === return_default_program_name_as_string
  # ========================================================================= #
  def return_default_program_name_as_string
    main_array?.first.to_s
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help(also_exit = false)
    case also_exit
    when :then_exit
      also_exit = true
    end
    opnn; e 'This class will expand the given arguments (program names) '\
            'into .yml files.'
    e
    e 'Invocation example:'
    e
    e sfancy('  expand_cookbooks rubyprogressbar')
    e
    exit if also_exit
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    obtain_available_cookbooks
  end

  # ========================================================================= #
  # === obtain_available_cookbooks
  # ========================================================================= #
  def obtain_available_cookbooks
    @cookbooks = ::Cookbooks.available_programs
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === ensure_that_the_base_directory_exists
  # ========================================================================= #
  def ensure_that_the_base_directory_exists
    unless File.directory? base_dir_to_store_expanded_cookbooks?
      mkdir base_dir_to_store_expanded_cookbooks?
    end
  end

  # ========================================================================= #
  # === inform_the_user_what_we_will_do_next
  # ========================================================================= #
  def inform_the_user_what_we_will_do_next
    if main_array?.size < 2
      opnn; e 'We will now expand the program named '+
               sfancy(return_default_program_name_as_string)+
              ' towards its individual yaml file.'
    else
      opnn; e 'We will now expand the whole dataset stored into '\
              'the individual yaml files.'
    end
  end

  # ========================================================================= #
  # === base_dir_to_store_expanded_cookbooks?
  # ========================================================================= #
  def base_dir_to_store_expanded_cookbooks?
    Cookbooks.base_dir_to_store_expanded_cookbooks?
  end

  # ========================================================================= #
  # === work_on_each_cookbook
  #
  # Work on each cookbook and save the dataset as a hash.
  # ========================================================================= #
  def work_on_each_cookbook
    @expand_these_programs.each {|program|
      _ = Cookbooks::Cookbook.new(program) { :bypass_menu }
      hash = _.hash?
      dataset_to_save = YAML.dump(hash)
      what = dataset_to_save
      into = base_dir_to_store_expanded_cookbooks?+program+'.yml'
      opnn; e "Now storing the hash from `#{simp(program)}"\
              "` into `#{sfile(into)}`."
      write_what_into(what, into)
    }
  end

  # ========================================================================= #
  # === consider_updating_program_version_and_url_file
  # ========================================================================= #
  def consider_updating_program_version_and_url_file
    if is_on_roebe?
      # ===================================================================== #
      # On my home system I will auto-update that file as well.
      # ===================================================================== #
      Cookbooks.create_program_version_url_file
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    ensure_that_the_base_directory_exists
    inform_the_user_what_we_will_do_next
    work_on_each_cookbook
    consider_updating_program_version_and_url_file
  end

end

# =========================================================================== #
# === Cookbooks.expand_cookbooks
#
# The first argument should be the program whose .yml file we wish to
# autogenerate, such as 'subversion' or 'htop' or 'ruby'.
# =========================================================================== #
def self.expand_cookbooks(
    i = ARGV
  )
  ExpandCookbooks.new(i)
end; self.instance_eval { alias expand_this_cookbook expand_cookbooks } # === Cookbooks.expand_this_cookbook

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ExpandCookbooks.new(ARGV)
end # expand_cookbooks
    # expand_cookbooks m2crypto
    # expand_cookbooks subversion