#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::FindAllArchiveTypes
#
# This class will scan through $SRC, which is for me at /Users/x/SRC/, and
# collect all archive types found in that directory.
#
# It will then report a little bit statistical data.
#
# Usage examples:
#   Cookbooks::FindAllArchiveTypes.new
#   archive_types?
# =========================================================================== #
# require 'cookbooks/utility_scripts/find_all_archive_types.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class FindAllArchiveTypes < Base

  # ========================================================================= #
  # === Cookbooks::FindAllArchiveTypes::EXIT_ON_MISSING_ARCHIVE
  # ========================================================================= #
  EXIT_ON_MISSING_ARCHIVE = false

  # ========================================================================= #
  # === Cookbooks::FindAllArchiveTypes::REPORT_POSSIBLE_PROBLEM
  # ========================================================================= #
  REPORT_POSSIBLE_PROBLEM = true

  # ========================================================================= #
  # === Cookbooks::FindAllArchiveTypes::LJUST
  # ========================================================================= #
  LJUST   = 12

  # ========================================================================= #
  # === Cookbooks::FindAllArchiveTypes::NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  #
  # This class does not require any argument input.
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    set_base_directory
    @exit_on_missing_archive = EXIT_ON_MISSING_ARCHIVE
    @report_possible_problem = REPORT_POSSIBLE_PROBLEM
    @hash = {}
    @hash.default = 0 # We start counting at 0.
    @array_for_tar_bz2 = [] # Store .tar.bz2 files here.
    @array_for_tar_gz  = [] # Store .tar.gz files here.
  end

  # ========================================================================= #
  # === set_base_directory
  # ========================================================================= #
  def set_base_directory(i = source_directory?)
    @base_directory = i
  end

  # ========================================================================= #
  # === all_directories?
  # ========================================================================= #
  def all_directories?
    @all_directories
  end

  # ========================================================================= #
  # === add_count_for
  #
  # Use this method to keep track of all the entries that are part of
  # our @hash.
  # ========================================================================= #
  def add_count_for(i)
    case i # case tag
    when /\.zip$/
      @hash[ZIP] += 1
    when /\.tar\.bz2$/
      @hash[TAR_BZ2] += 1
    when /\.tar\.gz$/
      @hash[TAR_GZ] += 1
    when /\.tar\.xz$/
      @hash[TAR_XZ] += 1
    end
  end

  # ========================================================================= #
  # === process_every_directory
  # ========================================================================= #
  def process_every_directory
    all_directories?.each {|dir|
      found_entry = Dir[dir+'*'].first
      if found_entry.nil?
        if @report_possible_problem
          opnn; e 'There might be an error at `'+sdir(dir)+'`.'
        end if be_verbose? # Only if we are verbose.
        exit if @exit_on_missing_archive
      else
        if    found_entry.include? TAR_BZ2
          @array_for_tar_bz2 << found_entry
        elsif found_entry.include? TAR_GZ
          @array_for_tar_gz  << found_entry
        end
        _ = File.basename(found_entry)
        add_count_for(_)
      end
    }
  end

  # ========================================================================= #
  # === obtain_all_directories_from_the_source_directory
  # ========================================================================= #
  def obtain_all_directories_from_the_source_directory
    @all_directories = Dir[@base_directory+'*/'].reject {|entry|
      ! File.directory? entry
    }.sort
  end

  # ========================================================================= #
  # === menu                                             (case tag, menu tag)
  # ========================================================================= #
  def menu(i = nil)
    case i # case tag
    # ======================================================================= #
    # === find_all_archive_types
    # ======================================================================= #
    when 'HELP',/-?-?help/
      opnn; e 'Simply type anything, like gz.'
      exit
    else # Default here
      opnn; e 'Next showing all results for '+sfancy('`'+TAR_BZ2+'`:')
      show_results_for('.tar.bz2')
      opnn; e 'Next showing all results for '+sfancy('`'+TAR_GZ+'`:')
      show_results_for('.tar.gz')
    end
  end

  # ========================================================================= #
  # === show_results_for
  # ========================================================================= #
  def show_results_for(i = TAR_BZ2)
    case i
    when TAR_GZ
      if @array_for_tar_gz.empty?
        opnn; e 'No '+swarn('.tar.gz')+' entries were found.'
      else
        @array_for_tar_gz.each {|entry|
          e swarn('  '+entry)
        }
      end
    when TAR_BZ2
      if @array_for_tar_bz2.empty?
        opnn; e 'No .tar.bz2 entries were found.'
      else
        @array_for_tar_bz2.each {|entry|
          e swarn('  '+entry)
        }
      end
    end 
  end

  # ========================================================================= #
  # === show_results
  #
  # This method will show the results, in a pretty-printed fashion.
  # ========================================================================= #
  def show_results
    opnn; e 'We found these results:'
    e '  '+(TAR_XZ+': ' ).ljust(LJUST)+sfancy(@hash[TAR_XZ].to_s)
    e '  '+(TAR_BZ2+': ').ljust(LJUST)+sfancy(@hash[TAR_BZ2].to_s)
    e '  '+(TAR_GZ+': ' ).ljust(LJUST)+sfancy(@hash[TAR_GZ].to_s)
    e '  '+(ZIP+': '    ).ljust(LJUST)+sfancy(@hash[ZIP].to_s)
  end; alias report show_results # === report

  # ========================================================================= #
  # === run_and_report
  #
  # Bundle run() and report() together.
  # ========================================================================= #
  def run_and_report
    run
    report
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_all_directories_from_the_source_directory
    process_every_directory
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = Cookbooks::FindAllArchiveTypes.new
  _.show_results
  _.menu(ARGV.first) unless ARGV.empty?
end # archive_types?