#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: false
# =========================================================================== #
# === Cookbooks::FindDuplicateBinaries
#
# This class will check the /Programs/ hierarchy respectively the 
# /System/ hierarchy for all binaries.
#
# Whenever we find a match that also exists at /usr/bin/, we will collect
# it. Then, we tell the user which binary files are duplicates.
#
# Usage example:
#   Cookbooks::FindDuplicateBinaries.new
# =========================================================================== #
# require 'cookbooks/utility_scripts/find_duplicate_binaries.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class FindDuplicateBinaries < Base # === Cookbooks::FindDuplicateBinaries.new

  # ========================================================================= #
  # === DELETE_DUPLICATE_FILES
  #
  # If this constant is true then we will also delete the duplicate
  # binaries. This can be toggled by the user on the commandline.
  #
  # By default, we will not delete the duplicate entries.
  # ========================================================================= #
  DELETE_DUPLICATE_FILES = false

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === delete_duplicates?
  # ========================================================================= #
  def delete_duplicates?
    @delete_duplicate_files
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @delete_duplicate_files = DELETE_DUPLICATE_FILES
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === grab_files_from_usr_bin
  # ========================================================================= #
  def grab_files_from_usr_bin
    @usr_bin = Dir[ubin_dir+'*']
  end

  # ========================================================================= #
  # === ubin_dir
  # ========================================================================= #
  def ubin_dir
    '/usr/bin/'
  end; alias ubin?     ubin_dir # === ubin?
       alias ubin_dir? ubin_dir # === ubin_dir?

  # ========================================================================= #
  # === found_duplicates?
  # ========================================================================= #
  def found_duplicates?
    !@duplicate_files.empty?
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === comment
  # ========================================================================= #
  def comment(i)
    Colours.comment(i) # Delegate to class Colours for this.
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i = @input)
    i = i.first if i.is_a? Array
    case i # case tag
    # ======================================================================= #
    # === duplicate_binaries --help
    # ======================================================================= #
    when '--help','HELP'
      show_help_options
      exit
    # ======================================================================= #
    # === duplicate_binaries --delete
    # ======================================================================= #
    when '--delete','DELETE','--toggle','--purge','PURGE'
      @delete_duplicate_files = true # Toggle.
    end
  end

  # ========================================================================= #
  # === show_help_options                                          (help tag)
  # ========================================================================= #
  def show_help_options
    e 'Currently we support only one option:'
    e
    Colours.ecomment '   --delete # delete duplicate files found in '\
      'the '+sdir(ubin?)+comment(' hierarchy') # finddup --help
    e
  end

  # ========================================================================= #
  # === sysdir                                                   (sysdir tag)
  #
  # This method will refer to /System/Executables.
  # ========================================================================= #
  def sysdir?
    SYSBIN_DIRECTORY
  end; alias sysbin?     sysdir? # === sysbin?
       alias sysdir      sysdir? # === sydir
       alias main_dir?   sysdir? # === main_dir?
       alias sysbin_dir? sysdir? # === sysbin_dir?

  # ========================================================================= #
  # === do_delete_duplicate_files
  #
  # This will delete only files in the /usr/bin/ hierarchy. 
  # ========================================================================= #
  def do_delete_duplicate_files
    if delete_duplicates? and !@duplicate_files.empty?
      opnn; e 'Now deleting duplicate files from the '+sdir(ubin?)+
              ' directory.'
      @duplicate_files.each {|entry|
        _ = sysbin?+File.basename(entry)
        if File.exist? _ # Extra safeguard here.
          e 'The file path at '+sfile(ubin?+entry)+' is a duplicate '\
            'of `'+sfile(File.readlink(_))+'`.'
          e 'Thus we can delete `'+sfile(ubin?+entry)+'`.'
          delete(ubin?+entry) # Delete it finally.
        end
      }
    end
  end

  # ========================================================================= #
  # === grab_files_from_system_executables
  # ========================================================================= #
  def grab_files_from_system_executables
    @system_executables = Dir[sysdir+'*']
  end

  # ========================================================================= #
  # === report_duplicate_files
  #
  # This method will report whether we have duplicate files.
  # ========================================================================= #
  def report_duplicate_files
    if found_duplicates?
      opnn; e 'We have found these '+simp(@duplicate_files.size.to_s)+' '\
              'duplicates.'
      opnn; e 'They originate from the directory `'+sdir(ubin_dir?)+'` and'
      opnn; e 'are compared to the directory `'+sdir(sysbin_dir?)+'.'
      long_string = @duplicate_files.join(', ').strip
      long_string.chop! if long_string.end_with? ',' # Get rid of traling ','.
      e '  '+long_string.strip
    else
      opnn; e 'We did not find any duplicate entries. Thus, we will do '\
              'no further action.'
    end
  end

  # ========================================================================= #
  # === detect_duplicate_files
  #
  # This will detect all duplicate files.
  # ========================================================================= #
  def detect_duplicate_files
    @duplicate_files = [] # Empty at the beginning.
    ubin_array   = @usr_bin.map {|entry| File.basename(entry) }.sort
    sysbin_array = @system_executables.map {|entry| File.basename(entry) }
    counter = 0
    # ======================================================================= #
    # Next, compare the ubin-array with the sysbin-array:
    # ======================================================================= #
    ubin_array.each {|entry|
      if sysbin_array.include? entry
        counter += 1
        _ = ('('+simp(counter.to_s)+') ').rjust(20)
        e _+'Yes, '+simp(entry)+' is included.'
        @duplicate_files << entry
      end
    }
    @duplicate_files.sort! # And keep them sorted alphabetically.
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do_next
  # ========================================================================= #
  def report_to_the_user_what_we_will_do_next
    opnn; e 'We will compare all entries from '+sdir(sysdir)+' with '\
            'entries from `'+sdir(ubin_dir)+'`.'
    opnn; e 'Duplicate entries will be reported.'
  end; private :report_to_the_user_what_we_will_do_next

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu # Check for the commandline-arguments first.
    report_to_the_user_what_we_will_do_next
    grab_files_from_system_executables
    grab_files_from_usr_bin
    detect_duplicate_files
    report_duplicate_files
    do_delete_duplicate_files if delete_duplicates?
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::FindDuplicateBinaries.new(ARGV)
end # find_duplicate_binaries