#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Highest
#
# This class will display the Top n programs in size, where n defaults to
# 80 - or rather, the value you will set for DISPLAY_N_PROGRAMS_BY_DEFAULT.
# =========================================================================== #
# To display the 200 largest programs, do:
#
#   highest 200
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'

begin
  require 'class_docu_shower'
rescue LoadError; end

module Cookbooks

class Highest < Base # === Cookbooks::Highest

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === EXIT_ON_MISSING_ENTRY
  # ========================================================================= #
  EXIT_ON_MISSING_ENTRY = false

  # ========================================================================= #
  # === DISPLAY_N_PROGRAMS_BY_DEFAULT
  #
  # How many programs to display by default.
  # ========================================================================= #
  DISPLAY_N_PROGRAMS_BY_DEFAULT = 80 # This will usually become @display_top_n.

  # ========================================================================= #
  # === initialize
  #
  # To use more than the defaults, you can do this:
  #   Cookbooks::Highest.new(150)
  # ========================================================================= #
  def initialize(
      display_n   = DISPLAY_N_PROGRAMS_BY_DEFAULT,
      run_already = true
    )
    reset
    startup
    set_display_top_n(display_n) # First set how many programs we will display.
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @dataset = nil
  end

  # ========================================================================= #
  # === available_programs?
  # ========================================================================= #
  def available_programs?
    ::Cookbooks.available_programs?
  end

  # ========================================================================= #
  # === startup
  # ========================================================================= #
  def startup
    @available_programs = available_programs?
  end

  # ========================================================================= #
  # === inform_the_user_what_we_will_do
  # ========================================================================= #
  def inform_the_user_what_we_will_do
    # Get the local URL.
    opnn; e 'Now fetching the top `'+simp(@display_top_n.to_s)+
            '` largest local programs from the directory'
    opnn; e '  `'+sdir(source_directory?)+'`.'
    opnn; e 'This may take a while. Please be patient.'
  end

  # ========================================================================= #
  # === notify_the_user_that_we_have_finished_fetching_the_local_urls
  # ========================================================================= #
  def notify_the_user_that_we_have_finished_fetching_the_local_urls
    opnn; e # Add a newline.
    opnn; e 'Finished fetching the local URLs. We will display them now.'+N
    opnn; e
  end

  # ========================================================================= #
  # === sort_results
  # ========================================================================= #
  def sort_results
    @results = @available_programs.sort_by {|name, size|
      size.to_i # We sort by size.
    }.reverse[0...@display_top_n]
  end

  # ========================================================================= #
  # === set_display_top_n
  # ========================================================================= #
  def set_display_top_n(
      i = DISPLAY_N_PROGRAMS_BY_DEFAULT
    )
    if i.is_a? Array # Handle arrays first.
      if i.empty?
        i = DISPLAY_N_PROGRAMS_BY_DEFAULT # Use default.
      else
        i = i.first
      end
    elsif i.is_a? Symbol
      case i # case tag
      when :display_ten_programs
        i = 10
      when :display_twenty_programs
        i = 20
      end
    end
    case i # case tag
    # ======================================================================= #
    # === highest --help
    # ======================================================================= #
    when /-?-?help/,'HELP'
      show_help; exit
    end
    i = i.to_i unless i.is_a? Integer
    # ======================================================================= #
    # Ensure that we will not grab too many entries.
    # ======================================================================= #
    i = available_programs?.size.to_i if i > available_programs?.size.to_i
    @display_top_n = i
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    if Object.const_defined? :ClassDocuShower
      ClassDocuShower[__FILE__]
    end
    e 'This class does not have many special commands.'
    e
    e 'You can pass in a number, which simply means how many'
    e 'programs will be displayed, e. g. as in:'
    e
    e '  highest 250'
    e
  end

  # ========================================================================= #
  # === beautify_file_size
  #
  # This will attempt to beautify the given input.
  #
  # We require 3 trailing positions after the , position.
  # ========================================================================= #
  def beautify_file_size(i)
    i  = i.to_s.strip
    x  = i.to_f
    kb = 1024
    mb = (1024 ** 2)
    gb = (1024 ** 3)
    # tb = (1024 ** 4)
    if x < 1024
      return "#{round(x.to_f)} bytes"
    elsif x > 1024 && x < mb
      return "#{round(x.to_f/kb)} kb"
    elsif x > mb && x < gb
      result = round(x.to_f/mb)
      result = '%.3f' % result
      return "#{result} MB"
    elsif x > gb
      return "#{round(x.to_f/gb)} GB"
    end
  end

  # ========================================================================= #
  # === round
  # ========================================================================= #
  def round(number, n_positions = 3)
    ( number * 10 ** n_positions ).floor.to_f / (10 ** n_positions)
  end

  # ========================================================================= #
  # === display_n?
  # ========================================================================= #
  def display_n?
    @display_top_n
  end

  # ========================================================================= #
  # === report_results
  # ========================================================================= #
  def report_results
    sort_results
    n_rjust = display_n?.to_s.size
    @results.each_with_index { |entry, index|
      index    += 1
      index     = index.to_s.rjust(n_rjust)
      # For now, we shorten the url.
      local_url = remove_file_suffix( File.basename(entry.first).strip ).strip
      local_url = '%-36s' % local_url # Then we pad it.
      _ = '%.3f' % entry[1].to_s
      filesize  = '%12s' % beautify_file_size(_)
      opnn; e '  ('+sfancy(index)+') '+local_url+' '+swarn(filesize)
    }
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === sanitize_available_programs
  # ========================================================================= #
  def sanitize_available_programs
    array = [] # Dump the information into this array here.
    _ = @available_programs
    _.each {|i|
      # ===================================================================== #
      # Obtain the dataset for this program next.
      # ===================================================================== #
      @dataset = Cookbooks::Cookbook.new(i) { :be_quiet_and_bypass_menu }
      url  = @dataset.local_url? # Keep the local URL here.
      if url.nil?
        e '`'+simp(i.to_s)+'` is nil, probably because it does not '\
          'exist locally.'
        e 'Please consider fixing this problem.'
        if EXIT_ON_MISSING_ENTRY
          e 'We will exit now as the constant EXIT_ON_MISSING_ENTRY '\
            'was set to true.'
          exit
        end
      end 
      size = @dataset.size?     # Keep the filesize here.
      if url
        array << [ url, size ]  # Then append into the main Array containing this information.
      end
    }
    @available_programs = array # Store it here finally.
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    inform_the_user_what_we_will_do
    sanitize_available_programs
    notify_the_user_that_we_have_finished_fetching_the_local_urls
    report_results # report results
  end

end

  # ========================================================================= #
  # === Cookbooks.highest
  #
  # Easier API.
  #
  # Usage example:
  #   Cookbooks.highest 123
  # ========================================================================= #
  def self.highest(n = DISPLAY_N_PROGRAMS_BY_DEFAULT)
    Cookbooks::Highest.new(n)
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::Highest.new(ARGV)
end # highest 120