#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Homepage
#
# This class can show the homepage of a given program.
#
# It can also report all URLs that do not have a homepage
# entry, such as via:
#
#   homepage --missing-url?
#
# Usage example:
#   Cookbooks::Homepage.new
# =========================================================================== #
# require 'cookbooks/utility_scripts/homepage.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class Homepage < Base # === Cookbooks::Homepage.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_program              = ARGV.first,
      optional_use_this_dataset = nil,
      run_already               = true
    )
    reset
    set_this_program(this_program)
    determine_the_dataset_to_use(optional_use_this_dataset)
    run if run_already
  end

  # ========================================================================= #
  # === determine
  # ========================================================================= #
  def determine_the_dataset_to_use(i = nil)
    if i.nil?
      unless Cookbooks.const_defined? :Cookbook
        require 'cookbooks/class/class.rb'
      end
      @dataset = Cookbooks::Cookbook.new(which_program?) { :bypass_menu }
    else
      @dataset = i
    end
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    menu(i)
    @this_program = i
  end

  # ========================================================================= #
  # === which_program?
  # ========================================================================= #
  def which_program?
    @this_program
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === homepage --help
      # ===================================================================== #
      when /^-?-?help$/
        show_help
        exit
      # ===================================================================== #
      # === homepage --missing-url?
      # ===================================================================== #
      when /^-?-?missing(-|_)?url\??/,
           /^-?-?missing/
        show_all_programs_that_miss_a_homepage_entry
        exit
      end
    end
  end
  
  # ========================================================================= #
  # === show_all_programs_that_miss_a_homepage_entry
  #
  # This method will show all programs that miss a homepage entry.
  # ========================================================================= #
  def show_all_programs_that_miss_a_homepage_entry
    require 'cookbooks/class/class.rb'
    available_programs?.each {|this_program|
      dataset = Cookbooks::Cookbook.new(this_program)
      homepage_entry = dataset.homepage?
      if homepage_entry.nil?
        _ = ('`'+this_program+'`').ljust(15)
        opnn; e "The program #{sfancy(_)} has no "\
                "#{sfancy('homepage: ')} entry."
      end
    }
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # Invoke this entry point via:
  #   homepage --help
  # ========================================================================= #
  def show_help
    opnn; e 'Currently these options are available for `'+
             sfancy('class '+NAMESPACE)+'`:'
    e
    e '  --missing-url?      # show which cookbook entries lack '\
      'a proper '+sfancy('homepage: entry')
    e
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_the_homepage
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === show_homepage_of_this_program
  # ========================================================================= #
  def show_homepage_of_this_program(
      this_program = which_program?
    )
    if this_program.include? '.yml'
      this_program.sub!(/\.yml$/,'')
    end
    _ = return_homepage
    if _
      opnn; e "The homepage of the program `#{sfancy(this_program)}"\
              "` is: #{simp(_)}"
    else
      opnn; e 'No homepage entry is registered in `'+
               sfancy(this_program)+'` (The .yml file should be at '
      opnn; e  simp('  '+individual_cookbooks_dir?+
               this_program+'.yml')+')'
    end
  end; alias report_the_homepage show_homepage_of_this_program # === report_the_homepage

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === return_homepage
  # ========================================================================= #
  def return_homepage
    dataset?.homepage?
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::Homepage.new(ARGV)
end # homepage bluefish