#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ScanForBlfsPrograms
#
# This class will scan the remote BLFS homepage and scan for all
# available programs. It will then report which local program
# entries do happen to NOT have a BLFS entry.
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/module_methods/return_array_of_programs_without_a_blfs_entry.rb'
require 'open-uri'

module Cookbooks

class ScanForBlfsPrograms < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === REMOTE_BASE_URL
  # ========================================================================= #
  REMOTE_BASE_URL = 'http://www.linuxfromscratch.org/blfs/view/svn/'

  # ========================================================================= #
  # === ALSO_REMOVE_NUMBERS_FROM_REMOTE_PROGRAM_NAMES
  #
  # If the following constant is set, then we will remove all numbers
  # from the given input.
  # ========================================================================= #
  ALSO_REMOVE_NUMBERS_FROM_REMOTE_PROGRAM_NAMES = false

  # ========================================================================= #
  # === ALSO_GIVE_HELP_FOR_MODIFYING_THESE_ENTRIES
  # ========================================================================= #
  ALSO_GIVE_HELP_FOR_MODIFYING_THESE_ENTRIES = true

  # ========================================================================= #
  # === REGEX_FOR_PROGRAMS_REGISTERED_AT_THE_REMOTE_BLFS_WEBPAGE
  # ========================================================================= #
  REGEX_FOR_PROGRAMS_REGISTERED_AT_THE_REMOTE_BLFS_WEBPAGE =
    /<li class="sect1">\s+<a href=\s*?"general\/(.+)\.html">/ # See: http://rubular.com/r/EJlKOGMW2m

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_arguments = ARGV,
      run_already        = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @dataset_from_the_remote_webpage = nil
    obtain_all_programs_that_miss_a_blfs_entry
    obtain_dataset_from_the_remote_webpage
  end

  # ========================================================================= #
  # === remote_blfs_page?
  # ========================================================================= #
  def remote_blfs_page?
    REMOTE_BASE_URL
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === remote_regex?
  # ========================================================================= #
  def remote_regex?
    REGEX_FOR_PROGRAMS_REGISTERED_AT_THE_REMOTE_BLFS_WEBPAGE
  end

  # ========================================================================= #
  # === obtain_dataset_from_the_remote_webpage
  # ========================================================================= #
  def obtain_dataset_from_the_remote_webpage
    @dataset_from_the_remote_webpage = open(REMOTE_BASE_URL).read
    # ======================================================================= #
    # The next variable will contain all programs that are available
    # at the BLFS webpage.
    # ======================================================================= #
    @remotely_available_programs = @dataset_from_the_remote_webpage.scan(/#{remote_regex?}/).flatten
  end

  # ========================================================================= #
  # === obtain_all_programs_that_miss_a_blfs_entry
  # ========================================================================= #
  def obtain_all_programs_that_miss_a_blfs_entry
    @programs_missing_a_blfs_entry = ::Cookbooks.return_array_of_programs_without_a_blfs_entry
  end

  # ========================================================================= #
  # === remove_all_numbers_from
  # ========================================================================= #
  def remove_all_numbers_from(i)
    i.gsub(/\d/,'')
  end

  # ========================================================================= #
  # === iterate_over_all_programs_missing_a_blfs_entry
  # ========================================================================= #
  def iterate_over_all_programs_missing_a_blfs_entry
    # ======================================================================= #
    # First, build up our Array that has the programs registered which
    # also have a remote BLFS page.
    # ======================================================================= #
    array_these_local_programs_do_not_have_a_blfs_entry = []
    @programs_missing_a_blfs_entry.each {|this_program|
      if ALSO_REMOVE_NUMBERS_FROM_REMOTE_PROGRAM_NAMES
        if this_program =~ /\d+/
          this_program = remove_all_numbers_from(this_program)
        end
      end
      if @remotely_available_programs.include? this_program
        array_these_local_programs_do_not_have_a_blfs_entry << this_program
      end
    }
    if array_these_local_programs_do_not_have_a_blfs_entry.empty?
      opnn; e 'All remote programs have already been registered '\
              'locally. This is good!' 
    else
      array_these_local_programs_do_not_have_a_blfs_entry.each {|this_program|
        remote_blfs_page = remote_blfs_page?
        opnn; e 'The program `'+sfancy(this_program)+'` is missing a blfs: entry.'
        remote_html_page = remote_blfs_page+'general/'+this_program+'.html'
        opnn; e 'The remote BLFS page is: '+
                 simp(remote_html_page)
        # =================================================================== #
        # Next, also report where it is available locally.
        # =================================================================== #
        if on_roebe?
          local_path = ROEBE_PATH_TO_COOKBOOKS+
                       'yaml/individual_cookbooks/'+this_program+'.yml'
        else
          local_path = individual_cookbooks_directory?+
                       this_program+'.yml'
        end
        if ALSO_GIVE_HELP_FOR_MODIFYING_THESE_ENTRIES
          e
          e '  bl '+sfile(local_path)
          e '  '+simp(remote_html_page)
          e
        end
      }
    end
  end

  # ========================================================================= #
  # === report_how_many_remote_entries_exist
  # ========================================================================= #
  def report_how_many_remote_entries_exist
    opnn; e 'The remote BLFS webpage has at the least '+
             simp(@remotely_available_programs.size.to_s)+' programs registered.'
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_how_many_remote_entries_exist
    iterate_over_all_programs_missing_a_blfs_entry 
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ScanForBlfsPrograms.new(ARGV)
end # scan_for_blfs_programs