#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ScanSourceArchive
#
# This class will scan through our archives and report if we have
# packages which are not included.
# =========================================================================== #
# require 'cookbooks/utility_scripts/scan_source_archive.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'

module Cookbooks

class ScanSourceArchive < Base # === Cookbooks::ScanSourceArchive

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === ARRAY_SKIP_THESE_DIRECTORIES
  #
  # Next comes a listing of Sub-Directories to skip (forbidden dirs).
  # Careful with what you enter here.
  # ========================================================================= #
  ARRAY_SKIP_THESE_DIRECTORIES = %w(
    /windows/
    java
    rpg
    firefox
    extensions
    opera
    firmware
    ruby
    patches
    gobolinux
    fonts
    hardware
    rarlinux
    libreoffice
    nvidia
    jdk
    bios
    flash
    jquery
    ntfs3g
    pdf
    polkitqt
    wlan
    youtubedl
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(run_already = true)
    register_sigint
    super()
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @cookbook_dataset = nil
    @array_registered_files = [] # Keep them registered.
  end

  # ========================================================================= #
  # === set_target_directory
  # ========================================================================= #
  def set_target_directory(i)
    @target_directory = i
  end

  # ========================================================================= #
  # === target_directory?
  # ========================================================================= #
  def target_directory?
    @target_directory
  end; alias target_directory target_directory? # === target_directory

  # ========================================================================= #
  # === register_file
  #
  # Simply append to @array_registered_file.
  #
  # The second argument shall specify the target location. 
  # ========================================================================= #
  def register_file(f, real_location)
    @array_registered_files << [
      f, real_location
    ] #unless @array_registered_files
  end

  # ========================================================================= #
  # === scan
  #
  # This is the actual scanning action.
  # ========================================================================= #
  def scan(
      target = source_directory?
    )
    set_target_directory(target)
    opnn; e "Scanning through `#{sdir(target_directory?)}"\
            "` and all of its subdirectories next."
    opnn; e 'This may take a while - please be patient.'+N
    all_files = Dir["#{target_directory?}**/*"].sort
    # ======================================================================= #
    # Filter away directories.
    # ======================================================================= #
    all_files.reject! {|entry| File.directory? entry }
    opnn; e simp(all_files.size.to_s)+' entries (== files) were found.'
    all_files.each { |file|
      original_file = file.to_s.dup
      next if ! file.include? '-'
      next if forbidden?(file)
      file = File.basename(file).downcase
      # ======================================================================= #
      # Obtain only the assumed "program name".
      # ======================================================================= #
      file = ProgramInformation::ProgramInformation.new(file).return_name
      _ = Cookbooks::Cookbook.new(file) { :bypass_menu }
      _ = file.to_s.gsub(/-/,'').downcase.gsub(/_/,'')
      if _.empty?
        opnn; e "Something went wrong for #{sfile(file)},#{sfancy(original_file)} (#{simp(_)})."
        opnn; e 'Please correct this mistake.'
        exit
      end
      begin
        register_file(_, file) # Register the file here.
      rescue Exception => error
        opnn; e 'An error happened (error type: '+sfancy(error.class.to_s)+
                ') at line '+swarn(__LINE__.to_s)
        opnn; e 'The original_file entry was: '+simp(original_file.to_s)
        pp error
      end
    }
    process_registered_programs
  end; alias run scan # === run (run tag)

  # ========================================================================= #
  # === forbidden?
  #
  # Skip directories listed in our array.
  # ========================================================================= #
  def forbidden?(i)
    ARRAY_SKIP_THESE_DIRECTORIES.any? { |entry|
      return true if i.include? entry
    }
    return false
  end

  # ========================================================================= #
  # === files?
  # ========================================================================= #
  def files?
    @array_registered_files
  end

  # ========================================================================= #
  # === array_registered_files?
  # ========================================================================= #
  def array_registered_files?
    @array_registered_files
  end; alias array_registered_files array_registered_files? # === array_registered_files

  # ========================================================================= #
  # === query_database?
  # ========================================================================= #
  def query_database?
    @query_database
  end; alias query_database query_database? # === query_database

  # ========================================================================= #
  # === process_registered_programs
  # ========================================================================= #
  def process_registered_programs(i = files?)
    i.each { |name_of_program, real_location|
      @cookbook_dataset = ::Cookbooks::Cookbook.new(name_of_program) { :bypass_menu }
      if @cookbook_dataset.program_was_found?
        # opnn; e sfancy(name_of_program)+' is included.' Then be silent.
      else # Else it is not included.
        opnn; e swarn('`'+sfancy(name_of_program.to_s)+'` is not included.')
        opnn; e '  It can be found at `'+sfile(real_location)+'`.'
      end
    }
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ScanSourceArchive.new
  # pp _.array_registered_files
end # sarchive