#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::SearchForTags
#
# This class will allow you to search through a list of tags
# and output the programs where these tags were found.
#
# For this to work, we will depend on grep right now - it simple is a 
# lot faster than plain ruby.
#
# Optionally, we will also allow direct-loading of the yaml files, or
# perhaps make use of a SQL database at a later time.
#
# You can search for one tag, or for more than one tag.
#
# If you search for more than one tag, we assume the query means "give me
# all entries that have ALL of these individual tags TOGETHER", so i.e.
# if you search for "gnome python" then this would mean "give me all
# entries that have both gnome and python defined".
# =========================================================================== #
# To use this script in your own project, do:
#
#   require 'cookbooks/utility_scripts/search_for_tags/search_for_tags.rb'
#
# Or just:
#
#   require 'cookbooks'
#
# To also output all URLs, do:
#
#   tags mate WITH_URL
#   tags ruby WITH_URL
#
# To combine more than one tag, do i.e.:
#
#   tags kde ftp
#
# =========================================================================== #
# require 'cookbooks/utility_scripts/search_for_tags/search_for_tags.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/module_methods/show_available_tags.rb'
require 'cookbooks/utility_scripts/registered_tags.rb'
require 'cookbooks/utility_scripts/search_for_tags/constants.rb'
require 'cookbooks/utility_scripts/search_for_tags/initialize.rb'
require 'cookbooks/utility_scripts/search_for_tags/menu.rb'
require 'cookbooks/class/class.rb'

begin
  require 'xorg_buffer/module'
rescue LoadError
  if RUBY_PLATFORM.include? 'linux'
    puts 'xorg_buffer is unavailable. Consider installing it.'
  end
end

module Cookbooks

class SearchForTags < Base # === Cookbooks::SearchForTags 

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @xorg_buffer_string = ''.dup
    @show_index = true
    @be_verbose = true # Just to make sure that we will be verbose here.
    @also_show_url = false # if true then we will also show the URL.
    @also_open_every_entry_in_the_editor = false
    @array_of_programs = []
  end

  # ========================================================================= #
  # === array_of_programs?
  # ========================================================================= #
  def array_of_programs?
    @array_of_programs
  end; alias array_of_programs array_of_programs? # === array_of_programs

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @xorg_buffer_string
  end; alias string  string? # === string?
       alias result  string? # === result
       alias result? string? # === result?

  # ========================================================================= #
  # === do_not_show_the_index
  # ========================================================================= #
  def do_not_show_the_index
    @show_index = false
  end

  # ========================================================================= #
  # === left_padding?
  # ========================================================================= #
  def left_padding?
    28
  end

  # ========================================================================= #
  # === url?
  #
  # Return the URL of a program. The method perform_the_search() will
  # display it.
  # ========================================================================= #
  def url?(i)
    i = i.to_s.dup
    _ = "#{cookbooks_directory?}"\
        "#{File.basename(i).split(' ').first.delete(':')}".dup
    _ << '.yml' unless _.end_with? '.yml'
    if File.exist? _ # We directly load this file.
      i = YAML.load_file(_)
      i = i[i.keys.first]['url1'].to_s
    else # File does not exist here. Should not happen.
    end
    return i
  end

  # ========================================================================= #
  # === also_show_url?
  # ========================================================================= #
  def also_show_url?
    @also_show_url
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === store_results_into_file
  # ========================================================================= #
  def store_results_into_file
    this_file = store_into_this_directory?+'these_tags'
    if be_verbose?
      e # A spacer here is in order.
      opnn; e 'Now storing these results into the file `'+sfile(this_file)+'`.'
    end
    save_file(@xorg_buffer_string, this_file)
  end

  # ========================================================================= #
  # === set_the_xorg_buffer
  # ========================================================================= #
  def set_the_xorg_buffer
    unless @xorg_buffer_string.empty?
      XorgBuffer[@xorg_buffer_string]
    end
  end

  # ========================================================================= #
  # === append_to_buffer
  # ========================================================================= #
  def append_to_buffer(i)
    @xorg_buffer_string << i
  end

  # ========================================================================= #
  # === calculate_and_display_the_filesize_of
  #
  # We will calculate the filesize of the given program at hand.
  # ========================================================================= #
  def calculate_and_display_the_filesize_of(this_file)
    this_file = this_file.dup.map {|entry|
      source_directory?+
        File.basename(entry.split(':').first).sub(/\.yml$/,'')+
        '/'
    }
    total_size = 0
    this_file.each {|entry|
      entries = Dir[entry+'*']
      if entries.size == 0
        # =================================================================== #
        # Report empty directories, but only if the entry "do_not_download"
        # was not set.
        # =================================================================== #
        program = File.basename(entry).downcase
        _ = Cookbooks::Cookbook.new(program)
        unless _.get_value_of 'do_not_download'
          if File.directory? entry # Only report it if it exists.
            opnn; e 'We found an empty directory here, at `'+sdir(entry)+'`.'
          end
        end
        exit if EXIT_ON_EMPTY_DIRECTORIES
      end
      if entries.first
        total_size += File.size(entries.first) if File.exist? entries.first
      end
    }
    total_filesize = ( total_size.to_f/1024/1024 ).round(2).to_s
    if be_verbose?
      result = N+'The total filesize is '+sfancy(total_size.to_s)+
                 ' bytes. (This is '+simp(total_filesize)+' MB)'
      e result
    end
  end

  # ========================================================================= #
  # === search_for?
  # ========================================================================= #
  def search_for?
    @search_for
  end; alias commandline? search_for? # === commandline?
       alias search_term? search_for? # === search_term?

  # ========================================================================= #
  # === show_help
  #
  # Invoke this method like so:
  #   tags --help
  # ========================================================================= #
  def show_help # help tag
    e 'These options are available so far for class Cookbooks::SearchForTags:'
    e
    eparse '  OPEN_ALL    # open all entries in the editor'
    eparse '  WITH_URL    # also show the URL'
    eparse '  NO_INDEX    # do not show the leading numbers when listing the tags'
    eparse '  ALL         # display every tag'
    eparse '  --available # show which tags are available'
    e
  end

  # ========================================================================= #
  # === eparse
  # ========================================================================= #
  def eparse(i)
    Colours.eparse(i)
  end

  # ========================================================================= #
  # === display_every_tag
  #
  # This method will show every available tag.
  #
  # If the argument also_show_the_specific_tags is set to true then we
  # will also show the specific tag.
  #
  # To invoke this, do:
  #   tags ALL
  # ========================================================================= #
  def display_every_tag(
      also_show_the_specific_tags = true
    )
    array = [] # This array will keep our results.
    all_programs = Cookbooks.available_programs
    all_programs.each {|program|
      opnn; e 'Checking on the program `'+sfancy(program)+'` next:'
      begin
        _ = Cookbooks::Cookbook.new(program) { :bypass_menu_check }
      rescue Exception => error
        opnn; pp error
        opnn; e program
      end
      if _
        begin
          if _.has_tags?
            e 'Yes, it has tags.'
            array << [ program, _.return_tags ] if also_show_the_specific_tags
          else # Else, it has no tags.
            # e 'No, it has no tags.'
            array << [ program, '']
          end
        rescue Exception; end
      else
        opnn; e swarn('We did not find anything about `')+simp(program)+swarn('`.')
        opnn; e swarn('This is considered a fatal error for now, thus exiting.')
        exit
      end
    }
    # ======================================================================= #
    # Now report that array:
    # ======================================================================= #
    opnn; e 'Next reporting these results:'
    array.sort_by {|key, value| value }.each {|key,value|
      value = Konsole.skyblue(value) if Object.const_defined? :Colours
      e (key+': ').ljust(left_padding?)+value
    }
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === set_search_for
  # ========================================================================= #
  def set_search_for(i = nil)
    i = DEFAULT_SEARCH_TAG if i.nil?
    if i.is_a? Array and i.empty?
      i = [DEFAULT_SEARCH_TAG] # Fill up empty arrays in this case.
    end
    if i.is_a? Symbol
      i = i.to_s
    end
    if i.is_a? String
      i = i.strip # Get rid of whitespace.
      # ===================================================================== #
      # Next, sanitize some common misspellings.
      # ===================================================================== #
      if i.include? ' ' # If it includes internal ' '.
        i = i.split(' ') # Then we turn it into an Array.
      end
    end
    if i.is_a? Array # Next handle Arrays here.
      i.map! {|entry|
        if entry.include? '-' # '-' characters are useless - they are not part of a tag.
          entry = entry.dup if entry.frozen?
          entry.delete!('-')
        end
        entry
      }
      original_input = i.first
      # ===================================================================== #
      # === Sanitize operations
      #
      # Perform some sanitize-operations next, out of user-convenience.
      # ===================================================================== #
      case i.first # case tag
      when 'games'
        opnn; e 'Need to correct `'+simp(original_input)+'` to `'+
                 simp('game')+'` tag.'
        i = 'game'
      when 'x11'
        i = 'xorg'
        opnn; e 'Reassigning input `'+simp(original_input)+' to `'+simp(i)+
                ' as we should have more'
        opnn; e 'tags registered that way.'
      when 'WM','windowsmanager'
        i = 'windowmanager'
        reassign_this_input_to_that_registered_entry(original_input, i)
      when 'programming_languages'
        i = 'programming_language'
        reassign_this_input_to_that_registered_entry(original_input, i)
      end
    end
    i.flatten! if i.is_a? Array
    i = [i] unless i.is_a? Array
    i.compact!
    @search_for = i # Must be an Array.
  end

  # ========================================================================= #
  # === perform_the_search
  #
  # This method will perform the actual search.
  #
  # We will search through our programs via grep.
  # ========================================================================= #
  def perform_the_search
    array_with_the_results = [] # We store the results here.
    # ======================================================================= #
    # The advantage of the yaml database is that it is faster.
    #
    # The disadvantage is that the yaml database must be manually updated.
    #
    # You can do so via file registered_tags.rb or as alias:
    #
    #   regtag
    #
    # ======================================================================= #
    if use_yaml_database_for_tags?
      if be_verbose?
        opnn; e 'The yaml-database will be queried for the tags.'
      end
      _ = search_term?
      _ = _.first if _.is_a? Array
      if Cookbooks.all_tags?.include? _
        array_with_the_results << Cookbooks.all_tags?[_]
      end
    # ======================================================================= #
    # === Use grep
    # ======================================================================= #
    else # else we use grep.
      if be_verbose?
        opnn; e 'The command grep will be used to perform this search.'
      end
      # ===================================================================== #
      # Next, we have an array stored in @search_for_these_tags since
      # April 2014. We will search for "tags: SEARCH_TERM_HERE".
      # ===================================================================== #
      search_for?.each {|this_tag|
        # =================================================================== #
        # Why do we use grep here and thus depend on it? Simply. Grep is
        # super-fast.
        # =================================================================== #
        _ = 'grep --ignore-case -r "^ tags: '+this_tag+'" '+individual_cookbooks_directory?+'*'
        e _ if be_verbose?
        _ = `#{_}`.split(N)
        _ = _.map {|entry|
          splitted = entry.split(': tags:')
          File.basename(splitted.first).gsub(/\.yml/,'')
        }
        # =================================================================== #
        # Now that we sanitized that temp dataset, we can add it to
        # our array.
        # =================================================================== #
        array_with_the_results << _
      }
    end
    # ======================================================================= #
    # Next, as there may be multiple search tags, we may have to display
    # only the conjoint tags. We will do so next, by using the &
    # intersection operator of ruby.
    # ======================================================================= #
    if array_with_the_results.size > 1
      array_with_the_results = array_with_the_results.first & array_with_the_results[1]
    end
    array_with_the_results.flatten!
    @array_of_programs = array_with_the_results
    unless array_with_the_results.empty?
      if be_verbose?
        e; opnn; e 'The results for programs with the tag(s): `'+
                    sfancy(search_for?.join(' '))+'` are:'+
                    N+N
      end
      # ===================================================================== #
      # Show either with or without index.
      # ===================================================================== #
      array_with_the_results.each_with_index {|the_name, index|
        index = (index+1).to_s
        index = ('%4s' % index)+') '
        the_name = ('%-25s' % the_name.strip) if also_show_url?
        append_to_buffer(index) if @show_index
        append_to_buffer(the_name)
        _ = ''.dup
        _ << sfancy(index) if @show_index
        _ << the_name # Composite index and program-name.
        if also_show_url?
          the_url = url?(the_name)
          the_url = '   '+('%58s' % the_url)
          the_url = the_url.strip
          append_to_buffer(the_url)
          _ << simp(the_url)
        end
        append_to_buffer(N) # Prepare the xorg-buffer string.
        @result = _
        e _ if be_verbose?
        if @also_open_every_entry_in_the_editor
          this_file = COOKBOOK_DIRECTORY+the_name+'.yml'
          if File.exist? this_file
            if be_verbose?
              e 'Next opening '+sfile(this_file)+' after a '+
                'delay of '+simp(USE_THIS_DELAY.to_s)+' seconds.'
            end
            system('bluefish '+this_file)
            sleep USE_THIS_DELAY
          end
        end
      }
      if array_with_the_results.size > 1
        calculate_and_display_the_filesize_of(array_with_the_results)
      end
      store_results_into_file # Store these results.
    else # Else we did not find any results.
      if be_verbose?
        opnn; e 'Not found any tag-result for search entry `'+
                sfancy(search_for?.first)+'`.'
      end
    end
    set_the_xorg_buffer
  end

  # ========================================================================= #
  # === reassign_this_input_to_that_registered_entry
  # ========================================================================= #
  def reassign_this_input_to_that_registered_entry(original_input, i)
    opnn; e "Reassigning input `#{simp(original_input)} to the "\
            "registered entry `#{simp(i)}`."
  end

  # ========================================================================= #
  # === show_available_tags
  #
  # Simply delegate towards Cookbooks.show_available_tags.
  #
  # Invocation example:
  #   search_for_tags --show-available-tags
  # ========================================================================= #
  def show_available_tags(
      use_this_path = FILE_ALLOWED_TAGS
    )
    opnn; Cookbooks.show_available_tags(use_this_path, :do_not_report_the_namespace)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_against_menu # Check against the menu first.
    perform_the_search # We use grep to find the tags.
  end

end

  # ========================================================================= #
  # === Cookbooks.search_for_tags
  #
  # Use a module-method to search for tags.
  #
  # Usage example:
  #   result = Cookbooks.search_for_tags :xorg
  # ========================================================================= #
  def self.search_for_tags(i)
    SearchForTags.new(i)
  end; self.instance_eval { alias search_for_tag search_for_tags } # === Cookbooks.search_for_tag

  # ========================================================================= #
  # === Cookbooks.return_array_of_this_tag
  #
  # This method will return an Array of programs that qualify for a
  # given tag.
  #
  # Usage example:
  #   Cookbooks.return_array_of_this_tag 'php' # => ["drupal", "eaccelerator"]
  #   Cookbooks.return_array_of_this_tag 'kde5'
  # ========================================================================= #
  def self.return_array_of_this_tag(i)
    SearchForTags.new(i) { :be_quiet }.array_of_programs.sort
  end; self.instance_eval { alias return_all_programs_that_match_this_tag return_array_of_this_tag } # === Cookbooks.return_all_programs_that_match_this_tag 

end

if __FILE__ == $PROGRAM_NAME
  # ========================================================================= #
  # === test_this_file
  # ========================================================================= #
  def test_this_file
    Cookbooks.search_for_tags('video audio WITH_URL')
    Cookbooks.search_for_tags('gnome video WITH_URL')
  end
  # test_this_file
  Cookbooks.search_for_tags(ARGV)
end # stag
    # stag kde
    # stag mate WITH_URL
    # stag mate NO_NUMBERING