#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ShowDependenciesOn
#
# This class will list all programs that depend on "X" e.g. all that
# depend on something such as "ruby" or "webkitgtk" or "php".
#
# It is part of the Cookbooks project.
#
# Usage Examples:
#   sdo gtk2
#   sdo php
#   sdo x11
#   sdo qt
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
begin
  require 'xorg_buffer'
rescue LoadError; end

module Cookbooks

class ShowDependenciesOn < Cookbooks::Base # === Cookbooks::ShowDependenciesOn

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    super()
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @array_programs = []
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    i = i.join(' ') if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === add
  # ========================================================================= #
  def add(i)
    if i.include? '.'
      i = File.basename(i).gsub(File.extname(i), '')
    end
    @array_programs << i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === report_which_programs_depend_on_the_given_input          (report tag)
  # ========================================================================= #
  def report_which_programs_depend_on_the_given_input(
      i = @array_programs
    )
    opnn; e 'These '+sfancy(i.size.to_s)+' programs '\
            'depend on '+simp(input?)+':'
    i.each_with_index {|entry, index|
      index += 1
      index = index.to_s.rjust(i.size.to_s.size)
      if Object.const_defined? :Colours
        index = slateblue(index.to_s)
      end
      index = ('('+index.to_s+')').rjust(4)
      entry = teal(entry)
      e '  '+index.to_s+' '+entry
    }
  end; alias report report_which_programs_depend_on_the_given_input # === report

  # ========================================================================= #
  # === notify_the_user_that_this_might_take_a_while
  # ========================================================================= #
  def notify_the_user_that_this_might_take_a_while
    opnn; e 'Scanning through the available programs next to find out'
    opnn; e 'which programs depend on '+sfancy(input?)+'. This may take a while,'
    opnn; e 'please be patient.'
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === fetch_all_programs_that_depend_on_the_given_input
  # ========================================================================= #
  def fetch_all_programs_that_depend_on_the_given_input
    all_cookbook_files = cookbook_files?(:show_full_path)
    all_cookbook_files.each {|file|
      file = File.basename(file)
      dataset = Cookbooks::Cookbook.new(file) { :bypass_menu }
      data = dataset.dependencies_on?
      unless data.nil?
        add(file) if data.include? input? # Add it if the input is part of the dependencies. 
      end
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    notify_the_user_that_this_might_take_a_while
    fetch_all_programs_that_depend_on_the_given_input
    report_which_programs_depend_on_the_given_input
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ShowDependenciesOn.new(ARGV)
end # sdo php