#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CheckValidityOfCookbooks
#
# The class Cookbooks::CheckValidityOfCookbooks will ensure that all
# cookbook entries are valid.
#
# It can do several different analysis-modes, although for now, only one
# is fully supported - to check whether url1 is the same as the name.
# If no url1 entry is found, this will be reported.
#
# If more functionality is put into this class, it will be properly
# documented here.
#
# In order for this to work, in the mode described above, we can not rely
# on YAML.load_file() as it may lead to an error. We have to read in the
# dataset on our own and handle it past that point.
#
# sigpot:
#  description: |
#   "SigOpt Python API Client."
#  url1: https://pypi.python.org/packages/source/s/sigopt/sigopt-1.3.0.tar.gz#md5=ccd67d5c514c1a20659f6217686f0782
#  url2: https://pypi.python.org/packages/source/s/sigopt/
#  keep_extracted: t
#  prefix: t
#  make_options: PINSTALL2
#  required_deps_on: python
#  tags: python
#  last_update: 04 Apr 2016
# 
# ^^^ such a program is NOT valid. It is invalid because the name is invalid.
#
# it should conform to the URL being the name, hence the name
# should be "sigopt" rather than "sigpot". Also notice how difficult it is
# to see the difference at first glance, which was one main reason why
# this class here was written.
# =========================================================================== #
require 'cookbooks/base/base.rb'
begin
  require 'program_information'
rescue LoadError; end

module Cookbooks

class CheckValidityOfCookbooks < Base # === Cookbooks::CheckValidityOfCookbooks

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      name_of_the_program_to_check = ARGV,
      run_already                  = true
    )
    reset
    unless name_of_the_program_to_check.empty?
      set_available_files(name_of_the_program_to_check)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @hash = {}
    obtain_all_available_files # Must come after @hash.
  end

  # ========================================================================= #
  # === hash?
  # ========================================================================= #
  def hash?
    @hash
  end

  # ========================================================================= #
  # === obtain_all_available_files
  # ========================================================================= #
  def obtain_all_available_files
    set_available_files Dir[cookbook_directory?+'*.yml'].sort
  end

  # ========================================================================= #
  # === set_available_files
  # ========================================================================= #
  def set_available_files(i)
    i = [i] unless i.is_a? Array # Must always be an Array.
    @hash[:available_files] = i
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === available_files?
  # ========================================================================= #
  def available_files?
    @hash[:available_files]
  end

  # ========================================================================= #
  # === work_on_each_file
  #
  # The input should be an Array.
  # ========================================================================= #
  def work_on_each_file(i = available_files?)
    i.each {|file|
      program_short_name = File.basename(file).gsub(/\.yml$/,'')
      # ===================================================================== #
      # Ok, read in the dataset next. We assume that the input is either
      # a file or a string.
      # ===================================================================== #
      possible_location_of_the_yaml_file = individual_cookbooks_dir?+program_short_name+'.yml'
      if File.exist? possible_location_of_the_yaml_file
        dataset = File.read(possible_location_of_the_yaml_file)
      else
        dataset = file
      end
      # ===================================================================== #
      # Now we determine "component a", which is the program name.
      # ===================================================================== #
      regex_to_use = /^(\w+):/ # See http://rubular.com/r/8hKCAYXZqY
      dataset =~ regex_to_use
      a = $1.to_s.dup
      # ===================================================================== #
      # Next, we grab the entry for "b". This can work if we have an entry
      # called url1, or a program_name. Since the entry program_name is
      # more important, we will check on it first.
      # ===================================================================== #
      if dataset.include?('program_name') and !dataset.include?('use_this_program_name') and
        !dataset.include?('use_this_program_name_on_gobolinux')
        regex_to_use = /^ program_name: ([a-zA-Z\-_\d\.]+)$/ # See: http://rubular.com/r/TK5ceX1woW
        dataset =~ regex_to_use
        b = $1.to_s.dup
        case b # case tag
        when 'URL_REPLACE_UNDERSCORE','URL_CONVERT_UNDERSCORE',
             'USE_URL_SANITIZED','USE_URL_DOWNCASE','USE_URL'
          b = program_short_name
        end
        b = ProgramInformation.return_real_short_name(b.strip)
        b.chop! if b.end_with? '-'
        b.delete!('-') if b.include? '-'
      # ===================================================================== #
      # Ok, now we can check for the url1 entry, as there was no entry
      # called program_name in that particular yaml file.
      # ===================================================================== #
      elsif dataset.include? 'url1'
        regex_to_use = /^ url1: (.+)\w/ # See: http://rubular.com/r/SOdBdhGXgw
        dataset =~ regex_to_use
        b = $1.to_s.dup
        # =================================================================== #
        # Ok, we still have to do some sanitizing to do, such as ignoring
        # all past a '#' token:
        # =================================================================== #
        if b.include? '#'
          b = b[0..b.index('#')]
        end
        if b.include? 'URL_REPLACE_UNDERSCORE'
          b = program_short_name
        else
          b = ProgramInformation.return_real_short_name(b.strip)
          b = b.first.downcase if b.is_a? Array
        end
      else
        opnn; e 'Neither "program_name:" nor an "url1:" entry were '\
                'found for `'+simp(file)+'`.'
        opnn; e 'This is not allowed, thus we will exit now.'
        exit
      end
      a.downcase!
      b.downcase!
      b.delete!('.') if b.include? '.'
      if a == b # Everything is ok here, thus we do not have to report anything at all in this case.
        # opnn; e 'All is fine for `'+simp(program_short_name)+'`.'
        # ^^^ Enabling the above may be too verbose.
      else
        # =================================================================== #
        # Compare the two names.
        # =================================================================== #
        opnn; e 'We found an invalid program file at '+sfile(file)+'.'
        opnn; e 'The names are not the same: '+simp(a)+' != '+simp(b)+'.'
      end
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    work_on_each_file
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CheckValidityOfCookbooks.new(ARGV)
end # cvalidity
    # cvalidity sigopt