#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ReportUselessDescriptions
#
# This class will first scan over all available cookbooks. It will query
# the description of said cookbooks entries. If this specific entry is either
# empty or defaults to a generic entry, then this will be noted down, and
# usually reported on the commandline. This may then help the user to fix
# these entries.
#
# A flag exists called ALSO_OPEN_VIA_EDITOR. If true then this class
# will also open that file via the local editor, after a short delay.
# Note that this will become an instance variable, which can be toggled
# on the commandline as well, such as via --do-not-open.
#
# The main idea is that this class helps you clean up incomplete or
# erroneous descriptions of programs. Additionally, some stats can
# be reported as well, after we gathered information
#
# Usage example:
#   Cookbooks::ReportUselessDescriptions.new
# =========================================================================== #
# require 'cookbooks/utility_scripts/report_useless_descriptions.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/misc/available_cookbooks.rb'
begin
  require 'open_in_editor'
rescue LoadError; end

module Cookbooks

class ReportUselessDescriptions < Base # === ReportUselessDescriptions.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === ALSO_OPEN_VIA_EDITOR
  # ========================================================================= #
  ALSO_OPEN_VIA_EDITOR = true

  # ========================================================================= #
  # === DEFAULT_DESCRIPTION_OF_THE_PROGRAM
  # ========================================================================= #
  DEFAULT_DESCRIPTION_OF_THE_PROGRAM =
    'Default description of the program.'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    menu
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @array_these_programs_are_problematic = []
    @also_open_via_editor = ALSO_OPEN_VIA_EDITOR
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i = @input)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === report_useless_descriptions --do-not-open
      # ===================================================================== #
      when /-?-?do-?not-?open/
        do_not_open_files_in_the_editor
      end
    end
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === obtain_available_programs
  # ========================================================================= #
  def obtain_available_programs
    @available_programs = ::Cookbooks.available_programs?
  end

  # ========================================================================= #
  # === iterate_over_every_available_program
  # ========================================================================= #
  def iterate_over_every_available_program
    @available_programs.each {|this_program|
      _ = individual_cookbooks_dir?+this_program+'.yml'
      dataset = YAML.load_file(_)[this_program]
      if dataset.nil?
        this_program_is_problematic(this_program)
      else
        description = dataset['description']
        # =================================================================== #
        # Nil descriptions or empty descriptions will be reported.
        # =================================================================== #
        if description.nil? or description.strip.empty?
          this_program_is_problematic(this_program)
        # =================================================================== #
        # As will those programs that conform to a default, but
        # =================================================================== #
        # pretty useless, description.
        elsif description.strip == DEFAULT_DESCRIPTION_OF_THE_PROGRAM
          this_program_is_problematic(this_program)
        else # Else all seems fine.
        end
      end
    }
  end

  # ========================================================================= #
  # === this_program_is_problematic
  # ========================================================================= #
  def this_program_is_problematic(this_program)
    @array_these_programs_are_problematic << this_program
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === report_results                                           (report tag)
  # ========================================================================= #
  def report_results
    _ = @array_these_programs_are_problematic
    opnn; e "We found a total of #{sfancy(_.size.to_s)} problematic programs."
    if _.empty?
      opnn; e 'This is '+lightgreen('excellent')+'! All programs '\
              'appear to have a useful description.'
    else
      _.each {|this_program|
        e '  - '+simp(this_program)
        if @also_open_via_editor
        open_in_editor this_program
          sleep 0.6
        end
      }
    end
  end; alias report report_results # === report

  # ========================================================================= #
  # === do_not_open_files_in_the_editor
  # ========================================================================= #
  def do_not_open_files_in_the_editor
    @also_open_via_editor = false
  end

  # ========================================================================= #
  # === open_in_editor
  # ========================================================================= #
  def open_in_editor(i)
    i = i.to_s.dup
    unless i.end_with? '.yml'
      i << '.yml'
    end
    if is_on_roebe?
      i.prepend "#{RUBY_SRC_DIR_AT_HOME}cookbooks/lib/cookbooks/yaml/individual_cookbooks/"
    else
      i.prepend individual_cookbooks_dir?
    end
    OpenInEditor[i]
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_available_programs
    iterate_over_every_available_program
    report_results
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ReportUselessDescriptions.new(ARGV)
end # report_useless